#ifndef __CAMERA_H
#define __CAMERA_H

#include <sys/types.h>

#include <X11/X.h>
#include <X11/Xlib.h>

#include "devsec_events.h"
#include "dtk_widget.h"
#include "dtk_socket.h"
#include "dtk_popup.h"
#include "dtk_timer.h"

#include "config.h"

#define BACKEND_CONFIG DTK_CONFDIR "/devsec/mp4live/camera%d.conf"
#define CAMERA_CONFIG BACKEND_CONFIG
//#define CAMERA_CONFIG "/etc/devsec/video%d.conf"
#define CAMERA_SOCKET "/tmp/video%d.sock"
#define CAMERA_PID "/tmp/video%d.pid"
#define STREAM_DIR "/streams"
#define STREAM_FILE STREAM_DIR "/%s/live.mp4"

class Camera : public Widget {
public:
	Camera(Widget *parent, int x, int y, int w, int h, int cameraNo);
	virtual ~Camera();
	virtual int SetPopup(Popup *popup);
	virtual int EventHandler(TEvent *ev);
	virtual int Resize(int w, int h);
	virtual int MoveResize(int x, int y, int w, int h);
	virtual inline int IsLocked() { return mLocked; };
	virtual int Lock(int lock);

	virtual int GlobalROM(int flag);
	virtual int GlobalNOM(int flag);

	virtual int LoadConfig();
	virtual int SaveConfig();

	virtual int CutStream();

	// Config commands
	virtual int SetCameraName(char *name);
	virtual char *GetCameraName();
	virtual inline int GetSensitivity() { return mSensitivity; };
	virtual inline int GetBoundsX1() { return mBoundsX1; };
	virtual inline int GetBoundsY1() { return mBoundsY1; };
	virtual inline int GetBoundsX2() { return mBoundsX2; };
	virtual inline int GetBoundsY2() { return mBoundsY2; };
	virtual inline int GetShowMotion() { return mShowMotion; };
	virtual inline int GetShowBounds() { return mShowBounds; };
	virtual inline int GetBrightness() { return mBrightness; };
	virtual inline int GetContrast() { return mContrast; };
	virtual inline int GetColor() { return mColor; };
	virtual inline int GetBitrate() { return mBitrate; };

	// General BE command
	virtual int RestartBackend();
	virtual int StartBackend();
	virtual int StopBackend();
	virtual int IsZOMBIE();
	virtual inline int IsActive() { return mActive; };
	virtual inline int IsAlive() { return mAlive; };
	virtual inline int GetCameraNo() { return mCameraNo; };

	// V4L commands
	virtual int V4LBrightness(unsigned int brightness);
	virtual int V4LContrast(unsigned int contrast);
	virtual int V4LColor(unsigned int color);

	// Motion commands
	virtual int MotionBounds(int x1, int y1, int x2, int y2);
	virtual int SetSensitivity(int sensitivity);
	virtual int ShowBounds(int flag);
	virtual int ShowMotion(int flag);
	virtual int SetMotionRecord(int flag);
	virtual int SetNotification(int flag);
	virtual int RecordAlways(int flag);
	virtual inline int GetMotionRecord() { return mRecordMotion; };
	virtual inline int GetNotification() { return mNotifyMotion; };
	virtual inline int GetRecordAlways() { return mRecordAlways; };

	// Preview commands
	virtual int PreviewSize(int w, int h);
	virtual int PreviewParent(Window parent);
	virtual int PreviewDisplay(char *display);
	virtual int PreviewShow();
	virtual int PreviewHide();

	virtual int RemoteSize(int w, int h);
	virtual int RemoteParent(Window parent);
	virtual int RemoteDisplay(char *display);
	virtual int RemoteShow();
	virtual int RemoteHide();
	virtual int SetRemoteInViewList(int flag);
	virtual int GetRemoteInViewList();

	// Mpeg commands
	virtual int MpegStart();
	virtual int MpegStop();
	virtual int SetBitrate(unsigned int bitrate);

	// Remote commands
	virtual int UnicastDisplay(char *addr, char *port);

	virtual int OnTimerCallback();
protected:
	virtual int GetNextEvent(SockEvent *sev);
	virtual int DoMotionDetect(int flag);
	
	virtual int DeleteOldFiles();
	virtual pid_t Execute(int argc, char *argv[]);
	virtual int DumpConfiguration();

	virtual int RenameFile(int flag);

	static void TimerCallback(void *data1, void *data2);

	int mStoppedPending; // flags to allow us to async the cutting
	int mStartWaiting;

	time_t mStartTime;

	Timer *mRecordLengthTimer;
	Popup *mPopup;
	Socket *mSocket;
	int mCameraNo;
	pid_t mBackendPID;
	int mBackendPrio;
	char *mConfFile;
	char *mSockFile;
	char *mPIDFile;
	char *mSection;
	int mAlive;
	int mActive;

	char *mCameraName;
	int mRecording;
	unsigned int mBrightness;
	unsigned int mContrast;
	unsigned int mColor;
	int mLocked;

	int mBoundsX1;
	int mBoundsY1;
	int mBoundsX2;
	int mBoundsY2;
	int mSensitivity;
	int mShowBounds;
	int mShowMotion;
	int mRecordMotion;
	int mNotifyMotion;
	int mRecordAlways;
	int mGlobalROM;
	int mGlobalNOM;
	unsigned int mBitrate;

	int mRemoteW;
	int mRemoteH;
	char *mRemoteDisplay;
	Window mRemoteParent;
	int mRemoteVisible;
	int mRemoteInViewList;

	off_t mPrevSize;
};

#endif
