#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include "dtk_skin.h"
#include "dtk_config_dialog.h"
#include "devsec.h"
#include "log.h"

ConfigDialog::ConfigDialog(int numCameras, Camera **c, Devsec *d)
{
	mApp = d;
	mNumCameras = numCameras;
	mCameras = c;
	CreateWindows();
}

ConfigDialog::~ConfigDialog()
{
}

int ConfigDialog::Show()
{
	int i;

	write_log("starting configuration");
	for (i = 0; i < mNumCameras; i++) {
		mCameras[i]->MpegStop();
	}
	if (mAboutPanel) mAboutPanel->entering();
	if (mCameraPanel) mCameraPanel->entering();
	if (mGlobalPanel) mGlobalPanel->entering();
	if (mMotionPanel) mMotionPanel->entering();
	if (mTVPanel) mTVPanel->entering();
	if (mUserPanel) mUserPanel->entering();
	if (mSystemPanel) mSystemPanel->entering();

	if (mAboutPanel) DoToolbarBtnPressed(mAboutPanel);
	return mMainWindow->Show();
}

int ConfigDialog::Hide()
{
	int i;

	write_log("stopping configuration");
	if (mAboutPanel) mAboutPanel->leaving();
	if (mCameraPanel) mCameraPanel->leaving();
	if (mGlobalPanel) mGlobalPanel->leaving();
	if (mMotionPanel) mMotionPanel->leaving();
	if (mTVPanel) mTVPanel->leaving();
	if (mUserPanel) mUserPanel->leaving();
	if (mSystemPanel) mSystemPanel->leaving();

	for (i = 0; i < mNumCameras; i++) {
		mCameras[i]->RestartBackend();
	}
	return mMainWindow->Hide();
}

int ConfigDialog::EventHandler(TEvent *ev)
{
	return Object::EventHandler(ev);
}

int ConfigDialog::DoWindowDestroy(Widget *window)
{
	if (window == NULL) return 0;
	if (window == mMainWindow) {
		Hide();
	}
	return 0;
}

int ConfigDialog::OnBtnSave()
{
	int i;

	Hide();
	for (i = 0; i < mNumCameras; i++) {
		mCameras[i]->SaveConfig();
	}
	mApp->SaveConfig();
	return 0;
}

int ConfigDialog::OnBtnCancel()
{
	int i;

	Hide();
	for (i = 0; i < mNumCameras; i++) {
		mCameras[i]->LoadConfig();
	}
	mApp->LoadConfig();
	return 0;
}

int ConfigDialog::CreateWindows()
{
	Widget *widget;
	Button *button;

	mMainWindow = new Widget(NULL, 0, 0, 720, 576);
	mMainWindow->SetCallback((void*)this, NULL, WindowCallback);
	mMainWindow->SetTitle("Devolution Security - Configuration");
	mMainWindow->Hide();
	mMainWindow->SetResizable(0);
	AddChild(mMainWindow);

	// Create the panels first
	mAboutPanel = new DialogAbout(mMainWindow, 56, 0, 664, 544,
			mNumCameras, mCameras, mApp);
	mAboutPanel->Hide();

	mCameraPanel = new DialogCamera(mMainWindow, 56, 0, 664, 544,
			mNumCameras, mCameras, mApp);
	mCameraPanel->Hide();

	mGlobalPanel = new DialogGlobal(mMainWindow, 56, 0, 664, 544,
			mNumCameras, mCameras, mApp);
	mGlobalPanel->Hide();

	mMotionPanel = new DialogMotion(mMainWindow, 56, 0, 664, 544,
			mNumCameras, mCameras, mApp);
	mMotionPanel->Hide();

	mTVPanel = new DialogTV(mMainWindow, 56, 0, 664, 544,
			mNumCameras, mCameras, mApp);
	mTVPanel->Hide();

	mUserPanel = new DialogUsers(mMainWindow, 56, 0, 664, 544,
			mNumCameras, mCameras, mApp);
	mUserPanel->Hide();

	mSystemPanel = new DialogSystem(mMainWindow, 56, 0, 664, 544, mApp);
	mSystemPanel->Hide();

	// Toolbar
	widget = new Widget(mMainWindow, 0, 0, 56, 544);
	widget->DrawFrame(0);
	widget->SetPixmap(Skin::GetSkinPixmap(CONFIG_TOOLBAR));
	widget->Show();

	// Global Button
	button = new Button(widget, 0, 0, 56, 56);
	button->Show();
	button->DrawFrame(0);
	button->SetPixmap(Skin::GetSkinPixmap(BTNGLOBAL_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(BTNGLOBAL_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(BTNGLOBAL_PRESSED),STATE_PRESSED);
	button->SetText("Global", 6, 52);
	button->SetCallback((void*)this, (void*)mGlobalPanel,
			ToolbarBtnCallback);
	button->SetToolTipText("Global configuration options");
	button->SetCanHaveFocus(1);

	// Camera Button
	button = new Button(widget, 0, 56, 56, 56);
	button->Show();
	button->DrawFrame(0);
	button->SetPixmap(Skin::GetSkinPixmap(BTNCAMERAS_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(BTNCAMERAS_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(BTNCAMERAS_PRESSED),STATE_PRESSED);
	button->SetText("Cameras", 2, 52);
	button->SetCallback((void*)this, (void*)mCameraPanel,
			ToolbarBtnCallback);
	button->SetToolTipText("Configuration of cameras");
	button->SetCanHaveFocus(1);

	// TV Button
	button = new Button(widget, 0, 112, 56, 56);
	button->Show();
	button->DrawFrame(0);
	button->SetPixmap(Skin::GetSkinPixmap(BTNTV_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(BTNTV_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(BTNTV_PRESSED),STATE_PRESSED);
	button->SetText("TV", 20, 52);
	button->SetCallback((void*)this, (void*)mTVPanel,
			ToolbarBtnCallback);
	button->SetToolTipText("TV out options");
	button->SetCanHaveFocus(1);

	// Users Button
	button = new Button(widget, 0, 168, 56, 56);
	button->Show();
	button->DrawFrame(0);
	button->SetPixmap(Skin::GetSkinPixmap(BTNUSERS_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(BTNUSERS_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(BTNUSERS_PRESSED),STATE_PRESSED);
	button->SetText("Users", 10, 52);
	button->SetCallback((void*)this, (void*)mUserPanel,
			ToolbarBtnCallback);
	button->SetToolTipText("User management");
	button->SetCanHaveFocus(1);

	// Motion Button
	button = new Button(widget, 0, 224, 56, 56);
	button->Show();
	button->DrawFrame(0);
	button->SetPixmap(Skin::GetSkinPixmap(BTNMOTION_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(BTNMOTION_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(BTNMOTION_PRESSED),STATE_PRESSED);
	button->SetText("Motion", 6, 52);
	button->SetCallback((void*)this, (void*)mMotionPanel,
			ToolbarBtnCallback);
	button->SetToolTipText("Motion Detection configuration");
	button->SetCanHaveFocus(1);

	// System Button
	button = new Button(widget, 0, 280, 56, 56);
	button->Show();
	button->DrawFrame(0);
	button->SetPixmap(Skin::GetSkinPixmap(BTNGLOBAL_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(BTNGLOBAL_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(BTNGLOBAL_PRESSED),STATE_PRESSED);
	button->SetText("System", 6, 52);
	button->SetCallback((void*)this, (void*)mSystemPanel,
			ToolbarBtnCallback);
	button->SetToolTipText("System configuration");
	button->SetCanHaveFocus(1);

	// Status bar
	widget = new Widget(mMainWindow, 0, 544, 720, 32);
	widget->SetPixmap(Skin::GetSkinPixmap(STATUSBAR));
	widget->DrawFrame(0);
	widget->Show();

	// Cancel Button
	button = new Button(widget, 560, 0, 80, 32);
	button->Show();
	button->DrawFrame(0);
	button->SetText("Cancel", 28, 20);
	button->SetPixmap(Skin::GetSkinPixmap(CANCEL_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(CANCEL_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(CANCEL_PRESSED), STATE_PRESSED);
	button->SetCallback((void*)this, (void*)1, StatusBtnCallback);
	button->SetCanHaveFocus(1);
	button->SetDrawStipple(1);

	// Save Button
	button = new Button(widget, 640, 0, 80, 32);
	button->Show();
	button->DrawFrame(0);
	button->SetText("Save", 32, 20);
	button->SetPixmap(Skin::GetSkinPixmap(OK_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(OK_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(OK_PRESSED), STATE_PRESSED);
	button->SetCallback((void*)this, (void*)2, StatusBtnCallback);
	button->SetCanHaveFocus(1);
	button->SetDrawStipple(1);

	mCurrentPanel = mAboutPanel;
	mCurrentPanel->Show();
	return 0;
}

int ConfigDialog::DoToolbarBtnPressed(Widget *newPanel)
{
	if (!newPanel) return 0;

	if (newPanel == mCurrentPanel) return 0;

	newPanel->Show();

	if (mCurrentPanel) {
		mCurrentPanel->Hide();
	}
	mCurrentPanel = newPanel;
	return 0;
}


// Static callback
void ConfigDialog::WindowCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	ConfigDialog *c = (ConfigDialog*)d1;

	if (e->type != DTKEVENT) return;

	if (e->dev->type == DTK_WINDOW_DESTROY)
		c->DoWindowDestroy((Widget*)e->dev->destroy.object);
}


void ConfigDialog::ToolbarBtnCallback(Object *o,TEvent *e,void *d1,void *d2)
{
	ConfigDialog *c = (ConfigDialog*)d1;
	Widget *w = (Widget*)d2;

	if (e->type != XEVENT) return;
	if (e->xev->type != ButtonRelease) return;
	c->DoToolbarBtnPressed(w);
}

void ConfigDialog::StatusBtnCallback(Object *o,TEvent *e,void *d1,void *d2)
{
	ConfigDialog *c = (ConfigDialog*)d1;
	int btn = (int)d2;

	if (e->type != XEVENT) return;
	if (e->xev->type != ButtonRelease) return;
	if (btn == 1) c->OnBtnCancel();
	else if (btn == 2) c->OnBtnSave();
}


