#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "dcconfig.h"
#include "dtk_config.h"
#include "config.h"

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#define XK_MISCELLANY
#include <X11/keysymdef.h>

#define NEW_TEXT_ENTRY(title) { \
	if (mTextEntry == NULL) { \
		mTextEntry = new DCConfigTextEntry(NULL, \
				mX, mY+mH/2-96, mW, 160); \
		mTextEntry->Init(); \
		mTextEntry->SetParentMenu(this); \
		AddChild(mTextEntry); \
	} \
	mTextEntry->SetTitle(/**/title); \
}

#define NEW_SLIDER(title) { \
	if (mSlider == NULL) { \
		mSlider = new DCConfigSlider(NULL, \
				mX, mY+mH/2-96, mW, 160); \
		mSlider->Init(); \
		mSlider->SetParentMenu(this); \
		AddChild(mSlider); \
	} \
	mSlider->SetTitle(/**/title); \
}

// GENERIC MODULE
Widget *DCConfigModule::sFocusedWidget = NULL;

DCConfigModule::DCConfigModule(Widget *parent, int x, int y, int w, int h)
	:Widget(parent, x, y, w, h)
{
	mParentMenu = NULL;
	mPrevFocusedWidget = NULL;
	mNumButtons = 0;
	mSelected = -1;
	mHilighted = -1;
	WMIgnore(true);
	SetOpacity(0.75);
	SetTextSize(32);
	DrawFrame(true);
	SetColor(0x0, 0x0, 0x7f7f);
	SetForegroundColor(0xffff, 0xffff, 0xffff);
}

DCConfigModule::~DCConfigModule()
{
}

int DCConfigModule::Init()
{
	Select(0);
	return 0;
}

int DCConfigModule::DoButtonPress()
{
	return 0;
}

int DCConfigModule::SetTitle(char *title)
{
	SetText(title, 48, 48);
}

int DCConfigModule::Show()
{
	int rv = 0;
	if (mParentMenu) mParentMenu->Hide();
	rv = Widget::Show();
	sFocusedWidget = this;
	return rv;
}

int DCConfigModule::Hide()
{
	sFocusedWidget = mParentMenu;
	return Widget::Hide();
}

int DCConfigModule::SetHilighted(int button)
{
	if (mHilighted > -1) {
		mButtons[mHilighted]->SetColor(0x0, 0x0, 0x7f7f);
	}
	mHilighted = button;
	mButtons[mHilighted]->SetColor(0x0, 0x0, 0xffff);
	return 0;
}

int DCConfigModule::GetHilighted()
{
	return mHilighted;
}

int DCConfigModule::SetParentMenu(DCConfigModule *menu)
{
	mParentMenu = menu;
	return 0;
}

int DCConfigModule::EventHandler(TEvent *ev)
{
	if (ev->type == XEVENT) {
		switch (ev->xev->type) {
			case KeyRelease:
				if (sFocusedWidget && sFocusedWidget == this) {
					if (HandleKeyPress(ev->xev))
						return 1;
				}
		}
	}
	return Widget::EventHandler(ev);
}

int DCConfigModule::CreateButtons()
{
	int i;
	if (mNumButtons <= 0) return 0;

	mButtons = (Widget**)malloc(sizeof(Widget*)*mNumButtons);
	for (i = 0; i < mNumButtons; i++) {
		mButtons[i] = new Widget(this, 48, 96+i*48, mW-96, 48);
		mButtons[i]->SetColor(0x0, 0x0, 0x7f7f);
		mButtons[i]->SetForegroundColor(0xffff, 0xffff, 0xffff);
		mButtons[i]->DrawFrame(false);
		mButtons[i]->SetTextSize(32);
		mButtons[i]->Show();
		AddChild(mButtons[i]);
	}
	return 0;
}

int DCConfigModule::HandleKeyPress(XEvent *ev)
{
	char c = 0;
	int i = mSelected;
	KeySym keysym;

	if (ev == NULL) return 0;
	if (ev->type == ButtonRelease) return 0;

	if (XLookupString(&(ev->xkey), &c, 1, &keysym, NULL) == NoSymbol) {
		keysym = XKeycodeToKeysym(Widget::GetXDisplay(),
				ev->xkey.keycode, 0);
	}

	switch (keysym) {
		case XK_Up:
			i--;
			break;
		case XK_Down:
			i++;
			break;
		case XK_Return:
			DoButtonPress();
			break;
	}
	if (i < 0) i == 0;
	if (i >= mNumButtons) i = mNumButtons-1;
	Select(i);
	return 1;
}

int DCConfigModule::Select(int button)
{
	if (mNumButtons <= 0) return 0;
	if (button >= mNumButtons) return 0;
	if (button < 0) return 0;

	if (mSelected >= 0) {
		if (mSelected == mHilighted)
			mButtons[mSelected]->SetColor(0x0, 0x0, 0xffff);
		else
			mButtons[mSelected]->SetColor(0x0, 0x0, 0x7f7f);
		mButtons[mSelected]->Hide();
		mButtons[mSelected]->Show();
	}
	mSelected = button;

	mButtons[mSelected]->SetColor(0x0, 0x7f7f, 0x7f7f);
	mButtons[mSelected]->Hide();
	mButtons[mSelected]->Show();
	return 0;
}


// 0 LEVEL MENUS
// MAIN MENU

DCConfig::DCConfig(Widget *parent, int x, int y, int w, int h)
	: DCConfigModule(parent, x, y, w, h)
{
	mConfigAudio = NULL;
	mConfigCameras = NULL;
	mConfigNetwork = NULL;
	SetTitle("DEVSEC Configuration");
}

DCConfig::~DCConfig()
{
}

int DCConfig::Init()
{
	LoadConfig();
	mNumButtons = 4;
	CreateButtons();
	mButtons[0]->SetText("Audio", 0, 32);
	mButtons[1]->SetText("Camera", 0, 32);
	mButtons[2]->SetText("Network", 0, 32);
	mButtons[3]->SetText("Exit Menu", 0, 32);
	return DCConfigModule::Init();
}

int DCConfig::DoButtonPress()
{
	switch (mSelected) {
		case 0:
			if (!mConfigAudio) {
				mConfigAudio = new DCConfigAudio(NULL,
						mX, mY, mW, mH);
				mConfigAudio->Init();
				mConfigAudio->SetParentMenu(this);
				AddChild(mConfigAudio);
			}
			mConfigAudio->Show();
			break;
		case 1:
			if (!mConfigCameras) {
				mConfigCameras = new DCConfigCameras(NULL,
						mX, mY, mW, mH);
				mConfigCameras->Init();
				mConfigCameras->SetParentMenu(this);
				AddChild(mConfigCameras);
			}
			mConfigCameras->Show();
			break;
		case 2:
			if (!mConfigNetwork) {
				mConfigNetwork = new DCConfigNetwork(NULL,
						mX, mY, mW, mH);
				mConfigNetwork->Init();
				mConfigNetwork->SetParentMenu(this);
				AddChild(mConfigNetwork);
			}
			mConfigNetwork->Show();
			break;
		case 3:
			Hide();
			SaveConfig();
			if (mParentMenu) mParentMenu->Show();
			break;
	}
	return 0;
}

int DCConfig::LoadConfig()
{
	char section[32];
	char filename[256];
	int numCameras;
	int i;

	Config::ReadConfig("global", DTK_CONFDIR "/devsec/global.conf");
	numCameras = Config::GetIntValue("global", "numCameras", 4);

	for (i = 0; i < numCameras; i++) {
		snprintf(section, 32, "camera%d", i);
		snprintf(filename, 256,
				DTK_CONFDIR "/devsec/mp4live/camera%d.conf", i);
		Config::ReadConfig(section, filename);
	}
	return 0;
}

int DCConfig::SaveConfig()
{
	char section[32];
	char filename[256];
	int numCameras;
	int i;

	numCameras = Config::GetIntValue("global", "Cameras", 4);
	for (i = 0; i < numCameras; i++) {
		snprintf(section, 32, "camera%d", i);
		snprintf(filename, 256,
				DTK_CONFDIR "/devsec/mp4live/camera%d.conf", i);
		Config::WriteConfig(section, filename);
	}

	Config::WriteConfig("global", DTK_CONFDIR "/devsec/global.conf");
	exit(0);
}

// 1 LEVEL MENUS
// AUDIO MENU

DCConfigAudio::DCConfigAudio(Widget *parent, int x, int y, int w, int h)
	: DCConfigModule(parent, x, y, w, h)
{
	SetTitle("Audio Settings");
}

DCConfigAudio::~DCConfigAudio()
{
}

int DCConfigAudio::Init()
{
	int audible = Config::GetIntValue("global", "AudibleRecord", 0);
	mNumButtons = 3;
	CreateButtons();
	mButtons[0]->SetText("Beep when motion detected", 0, 32);
	mButtons[1]->SetText("Do not beep", 0, 32);
	mButtons[2]->SetText("Exit Menu", 0, 32);
	SetHilighted((audible ? 0 : 1));
	return DCConfigModule::Init();
}

int DCConfigAudio::DoButtonPress()
{
	switch (mSelected) {
		case 0:
			Config::SetIntValue("global", "AudibleRecord", 1);
			SetHilighted(mSelected);
			break;
		case 1:
			Config::SetIntValue("global", "AudibleRecord", 0);
			SetHilighted(mSelected);
			break;
		case 2:
			Hide();
			if (mParentMenu) mParentMenu->Show();
			break;
	}
	return 1;
}


// CAMERAS MENU

DCConfigCameras::DCConfigCameras(Widget *parent, int x, int y, int w, int h)
	: DCConfigModule(parent, x, y, w, h)
{
	SetTitle("Camera Settings");
}

DCConfigCameras::~DCConfigCameras()
{
}

int DCConfigCameras::Init()
{
	char buf[64];
	int i;

	mNumButtons = Config::GetIntValue("global", "mCameras", 4) + 1;
	mCameras = (DCConfigCamera**)malloc(sizeof(DCConfigCamera*)*(mNumButtons-1));
	CreateButtons();
	for (i = 0; i < mNumButtons - 1; i++) {
		mCameras[i] = NULL;
		snprintf(buf, 64, "Camera %d", i+1);
		mButtons[i]->SetText(buf, 0, 32);
	}
	mButtons[i]->SetText("Exit Menu", 0, 32);
	return DCConfigModule::Init();
}

int DCConfigCameras::DoButtonPress()
{
	if (mSelected == mNumButtons - 1) {
		Hide();
		if (mParentMenu) mParentMenu->Show();
	} else {
		if (mCameras[mSelected] == NULL) {
			mCameras[mSelected] = new DCConfigCamera(NULL,
					mX, mY, mW, mH);
			mCameras[mSelected]->Init();
			mCameras[mSelected]->SetParentMenu(this);
			mCameras[mSelected]->SetCameraNum(mSelected);
			AddChild(mCameras[mSelected]);
		}
		mCameras[mSelected]->Show();
	}
	return 1;
}


// NETWORK MENU

DCConfigNetwork::DCConfigNetwork(Widget *parent, int x, int y, int w, int h)
	: DCConfigModule(parent, x, y, w, h)
{
	SetTitle("Network Settings");
	mTextEntry = NULL;
	mMachineName = NULL;
	mIPAddress = NULL;
	mNetmask = NULL;
	mDefaultGW = NULL;
	mDNS = NULL;
}

DCConfigNetwork::~DCConfigNetwork()
{
	if (mMachineName) free((void*)mMachineName);
	if (mIPAddress) free((void*)mIPAddress);
	if (mNetmask) free((void*)mNetmask);
	if (mDefaultGW) free((void*)mDefaultGW);
	if (mDNS) free((void*)mDNS);
}

int DCConfigNetwork::Init()
{
	mNumButtons = 6;
	CreateButtons();
	mButtons[0]->SetText("Machine Name", 0, 32);
	mButtons[1]->SetText("IP Address", 0, 32);
	mButtons[2]->SetText("Netmask", 0, 32);
	mButtons[3]->SetText("Default Gateway", 0, 32);
	mButtons[4]->SetText("Domain Name Server", 0, 32);
	mButtons[5]->SetText("Exit Menu", 0, 32);

	mMachineName = (char*)malloc(sizeof(char)*128);
	mIPAddress = (char*)malloc(sizeof(char)*64);
	mNetmask = (char*)malloc(sizeof(char)*64);
	mDefaultGW = (char*)malloc(sizeof(char)*128);
	mDNS = (char*)malloc(sizeof(char)*128);
	memset(mMachineName, 0, 128);
	memset(mIPAddress, 0, 64);
	memset(mNetmask, 0, 64);
	memset(mDefaultGW, 0, 128);
	memset(mDNS, 0, 128);

	LoadNetwork();
	return DCConfigModule::Init();
}

int DCConfigNetwork::DoButtonPress()
{
	switch (mSelected) {
		case 0:
			NEW_TEXT_ENTRY("Machine Name");
			mTextEntry->SetTextBuffer(mMachineName, 128);
			mTextEntry->Show();
			break;
		case 1:
			NEW_TEXT_ENTRY("IP Address");
			mTextEntry->SetTextBuffer(mIPAddress, 64);
			mTextEntry->Show();
			break;
		case 2:
			NEW_TEXT_ENTRY("Netmask");
			mTextEntry->SetTextBuffer(mNetmask, 64);
			mTextEntry->Show();
			break;
		case 3:
			NEW_TEXT_ENTRY("Default Gateway");
			mTextEntry->SetTextBuffer(mDefaultGW, 128);
			mTextEntry->Show();
			break;
		case 4:
			NEW_TEXT_ENTRY("Domain Name Server");
			fprintf(stderr, "dns\n");
			mTextEntry->SetTextBuffer(mDNS, 128);
			mTextEntry->Show();
			break;
		case 5:
			fprintf(stderr, "machine name: %s\n", mMachineName);
			fprintf(stderr, "ip address: %s\n", mIPAddress);
			fprintf(stderr, "netmask: %s\n", mNetmask);
			fprintf(stderr, "default gw: %s\n", mDefaultGW);
			fprintf(stderr, "DNS: %s\n", mDNS);
			Hide();
			if (mParentMenu) mParentMenu->Show();
			SaveAndRestartNetwork();
			break;
	}
	return 1;
}

#define NETWORK "/etc/sysconfig/network"
#define IFCFG_ETH0 "/etc/sysconfig/network-scripts/ifcfg-eth0"
#define HOSTS "/etc/hosts"
#define RESOLV_CONF "/etc/resolv.conf"

int DCConfigNetwork::LoadNetwork()
{
	FILE *f;
	char line[256];
	int l;

	f = fopen(NETWORK, "r");
	if (f) {
		while (fgets(line, 256, f) != NULL) {
			char *c = strrchr(line, '\n');
			if (c) *c = '\0';
			if (!strncmp(line, "HOSTNAME", 8)) {
				l = (strlen(&line[9]) < 128 ? strlen(&line[9]) : 128);
				memcpy(mMachineName, &line[9], l);
			} else if (!strncmp(line, "GATEWAY", 7)) {
				l = (strlen(&line[8]) < 128 ? strlen(&line[8]) : 128);
				memcpy(mDefaultGW, &line[8], l);
			}
		}
		fclose(f);
	}

	f = fopen(IFCFG_ETH0, "r");
	if (f) {
		while (fgets(line, 256, f) != NULL) {
			char *c = strrchr(line, '\n');
			if (c) *c = '\0';
			if (!strncmp(line, "IPADDR", 6)) {
				l = (strlen(&line[7]) < 64 ? strlen(&line[7]) : 64);
				memcpy(mIPAddress, &line[7], l);
			} else if (!strncmp(line, "NETMASK", 7)) {
				l = (strlen(&line[8]) < 64 ? strlen(&line[8]) : 64);
				memcpy(mNetmask, &line[8], l);
			}
		}
		fclose(f);
	}
	f = fopen(RESOLV_CONF, "r");
	if (f) {
		while (fgets(line, 256, f) != NULL) {
			char *c = strrchr(line, '\n');
			if (c) *c = '\0';
			if (!strncmp(line, "nameserver", 10)) {
				l = (strlen(&line[11]) < 128 ? strlen(&line[11]) : 128);
				memcpy(mDNS, &line[11], l);
			}
		}
	}

	return 0;
}

int DCConfigNetwork::SaveAndRestartNetwork()
{
	FILE *f;
	char line[256];

	f = fopen(NETWORK, "w");
	if (f) {
		fprintf(f, "HOSTNAME=%s\n", mMachineName);
		fprintf(f, "GATEWAY=%s\n", mDefaultGW);
		fprintf(f, "WORKGROUP=WORKGROUP\n");
		fclose(f);
	}
	f = fopen(IFCFG_ETH0, "w");
	if (f) {
		fprintf(f, "DEVICE=eth0\n");
		fprintf(f, "IPADDR=%s\n", mIPAddress);
		fprintf(f, "NETMASK=%s\n", mNetmask);
		fprintf(f, "ONBOOT=yes\n");
		fclose(f);
	}
	f = fopen(RESOLV_CONF, "w");
	if (f) {
		fprintf(f, "nameserver %s\n", mDNS);
		fclose(f);
	}
	f = fopen(HOSTS, "w");
	if (f) {
		fprintf(f, "# Automatically generated by DCConfig\n");
		fprintf(f, "# (C) 2005 Peter Hartshorn\n");
		fprintf(f, "127.0.0.1 localhost.localdomain localhost\n");
		fprintf(f, "%s %s %s\n",
				mIPAddress, mMachineName, mMachineName);
		fclose(f);
	}

	system("/etc/rc.d/rc.networking stop; /etc/rc.d/rc.networking start; rm -f /samba/lib/smb.conf; /etc/rc.d/rc.samba stop; /etc/rc.d/rc.samba start");
	return 0;
}

// 2 LEVEL MENUS
// CAMERA MENU

DCConfigCamera::DCConfigCamera(Widget *parent, int x, int y, int w, int h)
	: DCConfigModule(parent, x, y, w, h)
{
	SetTitle("Camera Settings");
	mConfigMotion = NULL;
	mTextEntry = NULL;
	mCameraName = NULL;
	mSlider = NULL;
	mBrightness = 0;
	mContrast = 0;
	mCameraNum = 0;
}

DCConfigCamera::~DCConfigCamera()
{
}

int DCConfigCamera::Init()
{
	mNumButtons = 5;
	CreateButtons();
	mButtons[0]->SetText("Camera name", 0, 32);
	mButtons[1]->SetText("Motion detect", 0, 32);
	mButtons[2]->SetText("Brightness", 0, 32);
	mButtons[3]->SetText("Contrast", 0, 32);
	mButtons[4]->SetText("Exit Menu", 0, 32);
	return DCConfigModule::Init();
}

int DCConfigCamera::DoButtonPress()
{
	char section[32];
	switch (mSelected) {
		case 0:
			NEW_TEXT_ENTRY("Camera Name");
			if (mCameraName == NULL) {
				mCameraName = (char*)malloc(sizeof(char)*64);
				memset(mCameraName, 0, 64);
			}
			mTextEntry->SetTextBuffer(mCameraName, 64);
			mTextEntry->Show();
			break;
		case 1:
			if (!mConfigMotion) {
				mConfigMotion = new DCConfigMotion(NULL,
						mX, mY, mW, mH);
				mConfigMotion->Init();
				mConfigMotion->SetParentMenu(this);
				mConfigMotion->SetCameraNum(mCameraNum);
				AddChild(mConfigMotion);
			}
			mConfigMotion->Show();
			break;
		case 2:
			NEW_SLIDER("Brightness");
			mSlider->SetValue(mBrightness);
			mSlider->SetReturnValue(&mBrightness);
			mSlider->Show();
			break;
		case 3:
			NEW_SLIDER("Contrast");
			mSlider->SetValue(mContrast);
			mSlider->SetReturnValue(&mContrast);
			mSlider->Show();
			break;
		case 4:
			snprintf(section, 32, "camera%d", mCameraNum);
			if (mCameraName) Config::SetCharValue(section,
					"CameraName", mCameraName);
			Config::SetIntValue(section, "Brightness",
					mBrightness * 65525 / 25);
			Config::SetIntValue(section, "Contrast",
					mContrast * 65535 / 25);
			Hide();
			if (mParentMenu) mParentMenu->Show();
			break;
	}
	return 1;
}

int DCConfigCamera::SetCameraNum(int num)
{
	char section[32];
	char dfl[64];

	mCameraNum = num;
	snprintf(section, 32, "camera%d", mCameraNum);
	snprintf(dfl, 64, "Video%d", mCameraNum);
	mCameraName = Config::GetCharValue(section, "CameraName", dfl);
	mBrightness = Config::GetIntValue(section, "Brightness", 32767);
	mContrast = Config::GetIntValue(section, "Contrast", 32767);

	mBrightness = 25 * mBrightness / 65535;
	mContrast = 25 * mContrast / 65535;
	return 0;
}

// 3 LEVEL MENUS
// RECORDING MENU

DCConfigMotion::DCConfigMotion(Widget *parent, int x, int y, int w, int h)
	: DCConfigModule(parent, x, y, w, h)
{
	mSlider = NULL;
	mSensitivity = 0;
}

DCConfigMotion::~DCConfigMotion()
{
}

int DCConfigMotion::Init()
{
	mNumButtons = 5;
	CreateButtons();
	mButtons[0]->SetText("Record when motion detected", 0, 32);
	mButtons[1]->SetText("Record always", 0, 32);
	mButtons[2]->SetText("Never record", 0, 32);
	mButtons[3]->SetText("Sensitivity", 0, 32);
	mButtons[4]->SetText("Exit Menu", 0, 32);
	return DCConfigModule::Init();
}

int DCConfigMotion::DoButtonPress()
{
	char section[32];
	snprintf(section, 32, "camera%d", mCameraNum);
	switch (mSelected) {
		case 0:
			SetHilighted(mSelected);
			Config::SetIntValue(section, "RecordOnMotion", 1);
			Config::SetIntValue(section, "RecordAlways", 0);
			break;
		case 1:
			Config::SetIntValue(section, "RecordOnMotion", 0);
			Config::SetIntValue(section, "RecordAlways", 1);
			SetHilighted(mSelected);
			break;
		case 2:
			Config::SetIntValue(section, "RecordOnMotion", 0);
			Config::SetIntValue(section, "RecordAlways", 0);
			SetHilighted(mSelected);
			break;
		case 3:
			NEW_SLIDER("Sensitivity");
			mSlider->SetValue(mSensitivity);
			mSlider->SetReturnValue(&mSensitivity);
			mSlider->Show();
			break;
		case 4:
			Config::SetIntValue(section, "Sensitivity",
					mSensitivity * 320 / 25);
			Hide();
			if (mParentMenu) mParentMenu->Show();
			break;
	}
	return 1;
}

int DCConfigMotion::SetCameraNum(int num)
{
	char section[32];
	int val;

	mCameraNum = num;
	snprintf(section, 32, "camera%d", mCameraNum);
	val = Config::GetIntValue(section, "RecordOnMotion", 0);
	if (val) {
		SetHilighted(0);
		return 0;
	}
	val = Config::GetIntValue(section, "RecordAlways", 0);
	if (val) {
		SetHilighted(1);
		return 0;
	}
	SetHilighted(2);
	return 0;
}


DCConfigTextEntry::DCConfigTextEntry(Widget *parent, int x, int y, int w, int h)
	: DCConfigModule(parent, x, y, w, h)
{
	mTextBuffer = NULL;
	mTextBufSize = 0;
}

DCConfigTextEntry::~DCConfigTextEntry()
{
}

int DCConfigTextEntry::Init()
{
	mTextEntry = new TextEntry(this, 48, mH/2-24, mW-96, 48);
	mTextEntry->SetPixmap(0);
	mTextEntry->SetColor(0x0, 0x0, 0x0);
	mTextEntry->SetForegroundColor(0xffff, 0xffff, 0xffff);
	mTextEntry->SetTextSize(32);
	mTextEntry->Show();
	AddChild(mTextEntry);
	return DCConfigModule::Init();
}

int DCConfigTextEntry::DoButtonPress()
{
	char *p = mTextEntry->GetBufferPtr();
	int l;

	if (mParentMenu) mParentMenu->Show();
	Hide();

	if (mTextBuffer == NULL) return 0;
	if (p && strlen(p)) {
		l = (mTextBufSize < strlen(p)+1 ? mTextBufSize : strlen(p)+1);
		memset(mTextBuffer, 0, mTextBufSize);
		memcpy(mTextBuffer, p, l);
	}
	return 1;
}

int DCConfigTextEntry::SetTextBuffer(char *buffer, int bufsize)
{
	if (buffer == NULL) return 0;

	mTextBuffer = buffer;
	mTextBufSize = bufsize;
	if (mTextEntry) {
		mTextEntry->SetBuffer(NULL);
		mTextEntry->SetFocus();
		mTextEntry->SetBuffer(mTextBuffer);
	}
	return 0;
}

DCConfigSlider::DCConfigSlider(Widget *parent, int x, int y, int w, int h)
	: DCConfigModule(parent, x, y, w, h)
{
	mSlider = NULL;
	mSliderValue = 0;
	mRV = NULL;
}

DCConfigSlider::~DCConfigSlider()
{
}

int DCConfigSlider::Init()
{
	mSlider = new Widget(this, 48, mH/2-24, mW-96, 48);
	mSlider->SetPixmap(0);
	mSlider->SetColor(0x0, 0x0, 0x7f7f);
	mSlider->SetForegroundColor(0xffff, 0xffff, 0xffff);
	mSlider->SetTextSize(32);
	mSlider->SetText("I-------------------------", 8, 32);
	mSlider->Show();
	AddChild(mSlider);
	return DCConfigModule::Init();
}

int DCConfigSlider::DoButtonPress()
{
	if (mParentMenu) mParentMenu->Show();
	Hide();

	if (mRV) *mRV = mSliderValue;
	return 0;
}

int DCConfigSlider::SetValue(int value)
{
	char buf[27];
	if (value < 0) value = 0;
	if (value > 25) value = 25;

	snprintf(buf, 27, "----------------------------");
	buf[value] = 'I';
	mSliderValue = value;
	mSlider->SetText(buf, 8, 32);
	return 0;
}

int DCConfigSlider::SetReturnValue(int *rv)
{
	mRV = rv;
	return 0;
}

int DCConfigSlider::HandleKeyPress(XEvent *ev)
{
	char c = 0;
	int i = mSliderValue;
	int rv = 0;
	KeySym keysym;

	if (ev == NULL) return 0;
	if (ev->type == ButtonRelease) return 0;

	if (XLookupString(&(ev->xkey), &c, 1, &keysym, NULL) == NoSymbol) {
		keysym = XKeycodeToKeysym(Widget::GetXDisplay(),
				ev->xkey.keycode, 0);
	}

	switch (keysym) {
		case XK_Left:
			i--;
			rv = 1;
			break;
		case XK_Right:
			i++;
			rv = 1;
			break;
	}
	if (rv) {
		SetValue(i);
		return 1;
	}
	return DCConfigModule::HandleKeyPress(ev);
}
