#ifndef __DCCONFIG_H
#define __DCCONFIG_H

#include "dtk_widget.h"
#include "dtk_textentry.h"

// Menu hirachy:
class DCConfigModule;
class DCConfig;
	class DCConfigAudio;
	class DCConfigCameras;
		class DCConfigCamera;
			class DCConfigMotion;
	class DCConfigNetwork;
class DCConfigTextEntry;
class DCConfigSlider;



// Class declarations:
class DCConfigModule : public Widget {
public:
	DCConfigModule(Widget *parent, int x, int y, int w, int h);
	virtual ~DCConfigModule();
	virtual int Init();
	virtual int EventHandler(TEvent *ev);
	virtual int DoButtonPress();
	virtual int SetTitle(char *title);
	virtual int Show();
	virtual int Hide();
	virtual int SetHilighted(int button);
	virtual int GetHilighted();
	virtual int SetParentMenu(DCConfigModule *menu);
protected:
	virtual int CreateButtons();
	virtual int HandleKeyPress(XEvent *ev);
	virtual int Select(int button);
	int mNumButtons;
	Widget **mButtons;
	int mSelected;
	int mHilighted;
	DCConfigModule *mParentMenu;
	Widget *mPrevFocusedWidget;
	static Widget *sFocusedWidget;

};

class DCConfig : public DCConfigModule {
public:
	DCConfig(Widget *parent, int x, int y, int w, int h);
	virtual ~DCConfig();
	virtual int Init();
	virtual int DoButtonPress();
protected:
	virtual int LoadConfig();
	virtual int SaveConfig();
	DCConfigAudio *mConfigAudio;
	DCConfigCameras *mConfigCameras;
	DCConfigNetwork *mConfigNetwork;
};

class DCConfigAudio : public DCConfigModule {
public:
	DCConfigAudio(Widget *parent, int x, int y, int w, int h);
	virtual ~DCConfigAudio();
	virtual int Init();
	virtual int DoButtonPress();
};

class DCConfigCameras : public DCConfigModule {
public:
	DCConfigCameras(Widget *parent, int x, int y, int w, int h);
	virtual ~DCConfigCameras();
	virtual int Init();
	virtual int DoButtonPress();
protected:
	DCConfigCamera **mCameras;
};

class DCConfigNetwork : public DCConfigModule {
public:
	DCConfigNetwork(Widget *parent, int x, int y, int w, int h);
	virtual ~DCConfigNetwork();
	virtual int Init();
	virtual int DoButtonPress();
protected:
	int LoadNetwork();
	int SaveAndRestartNetwork();
	DCConfigTextEntry *mTextEntry;
	char *mMachineName;
	char *mIPAddress;
	char *mNetmask;
	char *mDefaultGW;
	char *mDNS;
};

class DCConfigCamera : public DCConfigModule {
public:
	DCConfigCamera(Widget *parent, int x, int y, int w, int h);
	virtual ~DCConfigCamera();
	virtual int Init();
	virtual int DoButtonPress();
	virtual int SetCameraNum(int num);
protected:
	DCConfigMotion *mConfigMotion;
	DCConfigTextEntry *mTextEntry;
	DCConfigSlider *mSlider;
	int mCameraNum;
	char *mCameraName;
	int mBrightness;
	int mContrast;
};

class DCConfigMotion : public DCConfigModule {
public:
	DCConfigMotion(Widget *parent, int x, int y, int w, int h);
	virtual ~DCConfigMotion();
	virtual int Init();
	virtual int DoButtonPress();
	virtual int SetCameraNum(int num);
protected:
	DCConfigSlider *mSlider;
	int mSensitivity;
	int mCameraNum;
};

// MISC MENUS
class DCConfigTextEntry : public DCConfigModule {
public:
	DCConfigTextEntry(Widget *parent, int x, int y, int w, int h);
	virtual ~DCConfigTextEntry();
	virtual int Init();
	virtual int DoButtonPress();
	virtual int SetTextBuffer(char *buffer, int bufsize);
protected:
	TextEntry *mTextEntry;
	char *mTextBuffer;
	int mTextBufSize;
};

class DCConfigSlider : public DCConfigModule {
public:
	DCConfigSlider(Widget *parent, int x, int y, int w, int h);
	virtual ~DCConfigSlider();
	virtual int Init();
	virtual int DoButtonPress();
	virtual int SetValue(int value);
	virtual int SetReturnValue(int *rv);
protected:
	virtual int HandleKeyPress(XEvent *ev);
	Widget *mSlider;
	int mSliderValue;
	int *mRV;
};
#endif
