#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/vfs.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/select.h>
#include <time.h>

#include "dtk_skin.h"
#include "dtk_config.h"
#include "dtk_meter.h"

#include "devsec.h"
#include "layouts.h"
#include "authdialog.h"

#include "config.h"

int Devsec::sCycling = 0;
int Devsec::sFullscreen = 0;
XF86VidModeModeInfo **Devsec::sVidModes = NULL;
int Devsec::sModeCount = 0;
int Devsec::sDeleteOldFiles = 0;
int Devsec::ZOMBIES = 0;
int Devsec::sAuthentication = 0;
int Devsec::sAudibleRecord = 0;
Widget *Devsec::mMainWindow = NULL;

char *commands[] = {
	"/usr/bin/devsec_helper powerdown",	// power 
	"/usr/bin/hardinfo",			// sysinfo
	"/usr/bin/gmplayer -vo x11 -zoom",	// media player
	"/usr/bin/xffind",			// find
	"/usr/bin/xsg",				// screen capture
	"/usr/bin/graveman",			// cd recorder
	//"/opt/kde/bin/kdirstat /streams",	// stats
	"echo",
	"xffm /streams",			// browser
	"xftrash4",
	"/usr/bin/xfdate",
	"/etc/sysconfig/network-scripts/netsettings.sh"
};

typedef void (*sighandler_t)(int);

int my_system(char *cmd)
{
	sighandler_t old;
	int rv;

	//fprintf(stderr, "executing [%s]\n", cmd);
	old = signal(SIGCHLD, SIG_IGN);
	rv = system(cmd);
	signal(SIGCHLD, old);
	return rv;
}

Devsec::Devsec()
{
}

Devsec::~Devsec()
{
}

int Devsec::Init(int argc, char **argv)
{
	int headless = 0;
	int i;
	Application::Init();

	CheckPidFile(0);

	Skin::OpenSkin("devsec_ice2.so");

	mCameras = NULL;

	mCycleTimer = NULL;
	mRemoteSocket = NULL;
	mCurrentLayout = 1;
	LoadConfig();

	mDeleteTimer = new Timer(300, NULL, NULL, DeleteTimerCallback);
	mDeleteTimer->Start();
	AddChild(mDeleteTimer);

	mRemoteControlTimer = new Timer(1, (void*)this, NULL, 
			RemoteControlTimerCallback);
	mRemoteControlTimer->Start();
	AddChild(mRemoteControlTimer);

#ifdef USE_WATCHDOG_FD
	int watchdogFD = 0, count=10;
	while (watchdogFD == 0 && count--) {
		watchdogFD = open("/tmp/watchdog3", O_WRONLY);
		if (!watchdogFD) sleep(1);
	}
	if (watchdogFD > 0) {
		mWatchdogTimer = new Timer(1, (void*)watchdogFD, NULL,
				WatchdogTimerCallback);
		mWatchdogTimer->Start();
		AddChild(mWatchdogTimer);
	} else {
		fprintf(stderr, "could not open watchdog pipe after 10 attempts\n");
		exit(1);
	}
#endif

	mZOMBIETimer = new Timer(1, (void*)this, NULL, ZOMBIETimerCallback);
	mZOMBIETimer->Start();
	AddChild(mZOMBIETimer);

	mRemoteSocket = new Socket();
	mRemoteSocket->Connect("/tmp/devsecremote.sock");
	AddChild(mRemoteSocket);

	my_system("devsec --remote &");

	for (i = 1; i < argc; i++) {
		if (!strncmp(argv[i], "--headless", 10)) headless = 1;
	}

	mConfigDialog = NULL;
	if (mMainWindow == NULL) CreateMainWindow();
	if (headless) mMainWindow->Hide();
	else mMainWindow->Show();
	AddChild(mMainWindow);
	DoLayout(mCurrentLayout);

	LoadConfig();

	signal(SIGHUP, Devsec::SigHandler);
	signal(SIGCHLD, Devsec::SigHandler);

	return 0;
}

int Devsec::LoadConfig()
{
	Config::ReadConfig("global", DTK_CONFDIR "/devsec/global.conf");

	sDeleteOldFiles = Config::GetIntValue("global", "deleteOldFiles", 0);
	mCycleTimeout = Config::GetIntValue("global","CycleTimeout",5);
	mRemoteCycle = Config::GetIntValue("global", "remoteCycle", 0);
	mRemoteLayout = Config::GetIntValue("global", "remoteLayout", 0);
	mRecordMotion = Config::GetIntValue("global", "motionDetect", 0);
	mNotifyMotion = Config::GetIntValue("global", "motionNotify", 0);
	mPhoneNumber = Config::GetCharValue("global", "PhoneNumber", NULL);
	sAuthentication = Config::GetIntValue("global", "Authentication", 0);
	sAudibleRecord = Config::GetIntValue("global", "AudibleRecord", 0);

	if (mCycleTimer == NULL) {
		mCycleTimer = new Timer(mCycleTimeout, (void*)this, NULL,
				CycleTimerCallback);
		mCycleTimer->Stop();
		AddChild(mCycleTimer);
	} else {
		mCycleTimer->SetDuration(mCycleTimeout);
	}

	if (mCameras != NULL)
		for (int i = 0; i < mNumCameras; i++) {
			char buf[256];
			int j = 0;
			snprintf(buf, 256, "remoteCamera%d", i);
			j = Config::GetIntValue("global", buf, 1);
			if (mCameras[i]) {
				SetRemoteInViewList(i, j);
			}
		}

	DoRemoteCycle(mRemoteCycle);
	DoRemoteLayout(mRemoteLayout);
	SetGlobalDetect(mRecordMotion);
	SetGlobalNotify(mNotifyMotion);
}

int Devsec::SaveConfig()
{
	Config::SetIntValue("global", "deleteOldFiles", sDeleteOldFiles);
	Config::SetIntValue("global", "CycleTimeout", mCycleTimeout);
	Config::SetIntValue("global", "remoteCycle", mRemoteCycle);
	Config::SetIntValue("global", "remoteLayout", mRemoteLayout);
	Config::SetIntValue("global", "motionDetect", mRecordMotion);
	Config::SetIntValue("global", "motionNotify", mNotifyMotion);
	Config::SetCharValue("global", "PhoneNumber", mPhoneNumber);
	Config::SetIntValue("global", "Authentication", sAuthentication);
	Config::SetIntValue("global", "AudibleRecord", sAudibleRecord);

	char *c = Config::GetCharValue("global", "button0", "NotSet");
	fprintf(stderr, "%s\n", c);
	free((void*)c);
	if (mCameras != NULL)
		for (int i = 0; i < mNumCameras; i++) {
			char buf[256];
			int j = 0;
			snprintf(buf, 256, "remoteCamera%d", i);
			j = GetRemoteInViewList(i);
			Config::SetIntValue("global", buf, j);
		}

	Config::WriteConfig("global", DTK_CONFDIR "/devsec/global.conf");
}

int Devsec::SetPhoneNumber(char *number)
{
	if (mPhoneNumber)
		free((void*)mPhoneNumber);

	mPhoneNumber = strdup(number);
	return 0;
}

char *Devsec::GetPhoneNumber()
{
	return mPhoneNumber;
}

int Devsec::SetGlobalDetect(int flag)
{
	int i;
	mRecordMotion = flag;

	for (i = 0; i < mNumCameras; i++) {
		mCameras[i]->GlobalROM(flag);
	}

	return 0;
}

int Devsec::GetGlobalDetect()
{
	return mRecordMotion;
}

int Devsec::SetGlobalNotify(int flag)
{
	int i;
	mNotifyMotion = flag;

	for (i = 0; i < mNumCameras; i++) {
		mCameras[i]->GlobalNOM(flag);
	}

	return 0;
}

int Devsec::GetGlobalNotify()
{
	return mNotifyMotion;
}

int Devsec::Destroy()
{
	Application::Destroy();
	return 0;
}

int Devsec::DoRemoteLayout(int layout)
{
	char buf[256];
	snprintf(buf, 256, "EV_LAYOUT %d\n", layout);

	mRemoteLayout = layout;
	if (mRemoteSocket) mRemoteSocket->Write(buf);
	return 0;
}

int Devsec::DoRemoteCycle(int flag)
{
	char buf[256];
	snprintf(buf, 256, "EV_CYCLE %d\n", flag);

	mRemoteCycle = flag;
	if (mRemoteSocket) mRemoteSocket->Write(buf);
	return 0;
}

int Devsec::DoUpdateMeters()
{
	double hdd = 0.0;
	double mem = 0.0;
	struct statfs buf;
	FILE *f;
	int mT, mF, mB, mC;
	char line[256];

	mT = mF = mB = mC = 0;
	statfs("/streams/images", &buf);

	hdd = 100.0 - (((double)(buf.f_bfree)*100.0)/(double)buf.f_blocks);

	f = fopen("/proc/meminfo", "r");
	if (f == NULL) return 0;
	while (fgets(line, 256, f) > 0) {
		int i;
		if (sscanf(line, "MemTotal: %d kB", &i)) mT = i;
		if (sscanf(line, "MemFree: %d kB", &i)) mF = i;
		if (sscanf(line, "Buffers: %d kB", &i)) mB = i;
		if (sscanf(line, "Cached: %d kB", &i)) mC = i;
	}
	fclose(f);
	mem = (double)(mT - (mF + mB + mC)) * 100.0 / mT;

	mHDDMeter->SetPercent(hdd);
	if (hdd >95.0)
		mHDDMeter->SetPixmap(Skin::GetSkinPixmap(METER_CRITICAL),
				METER_LEVEL);
	else if (hdd > 80.0)
		mHDDMeter->SetPixmap(Skin::GetSkinPixmap(METER_HIGH),
				METER_LEVEL);
	else
		mHDDMeter->SetPixmap(Skin::GetSkinPixmap(METER_NORMAL),
				METER_LEVEL);

	mMEMMeter->SetPercent(mem);
	if (mem >90.0)
		mMEMMeter->SetPixmap(Skin::GetSkinPixmap(METER_CRITICAL),
				METER_LEVEL);
	else if (mem > 70.0)
		mMEMMeter->SetPixmap(Skin::GetSkinPixmap(METER_HIGH),
				METER_LEVEL);
	else
		mMEMMeter->SetPixmap(Skin::GetSkinPixmap(METER_NORMAL),
				METER_LEVEL);

	if (mem > 95.0) {
		write_log("WARNING: memory usage at 95%%. Attempting restart");
		write_log("-- BEGIN PS DUMP --------------------------------");
		my_system("ps -axv >> /streams/log/error.log");
		write_log("-- END PS DUMP ----------------------------------");
		my_system("shutdown -nr now");
	}
	return 0;
}

int Devsec::GetAuthentication()
{
	return sAuthentication;
}

int Devsec::SetAuthentication(int flag)
{
	sAuthentication = flag;
	return 0;
}

int Devsec::GetRemoteInViewList(int cameraNo)
{
	if (cameraNo < 0) return 0;
	if (cameraNo > mNumCameras) return 0;

	if (mRemoteSocket) mRemoteSocket->Write("EV_CONFIG\n");
	DoRemoteLayout(mRemoteLayout);
	return mCameras[cameraNo]->GetRemoteInViewList();
}

int Devsec::SetRemoteInViewList(int cameraNo, int flag)
{
	if (cameraNo < 0) return 0;
	if (cameraNo > mNumCameras) return 0;

	return mCameras[cameraNo]->SetRemoteInViewList(flag);
}

int Devsec::SetCycleTimeout(unsigned int timeout)
{
	mCycleTimeout = timeout;
	mCycleTimer->SetDuration(timeout);
	return 0;
}

unsigned int Devsec::GetCycleTimeout()
{
	return mCycleTimeout;
}

int Devsec::SetDeleteOldFiles(int flag)
{
	sDeleteOldFiles = flag;
	return 0;
}

int Devsec::GetDeleteOldFiles()
{
	return sDeleteOldFiles;
}

int Devsec::CreateMainWindow()
{
	int i;
	Button *button;
	Widget *widget;
	Camera *camera;
	Meter *meter;
	Timer *timer;
	char *txt;

	// mainWindow
	mMainWindow = new Widget(NULL, 0, 0, 900, 600);
	mMainWindow->SetCallback((void*)this, NULL, WindowCallback);
	mMainWindow->SetTitle("Devolution Security");
	mMainWindow->DrawFrame(0);
	mMainWindow->SetResizable(0);

	// toolbar
	widget = new Widget(mMainWindow, 0, 0, 32, 576);
	widget->SetPixmap(Skin::GetSkinPixmap(TOOLBAR));
	widget->Show();
	widget->DrawFrame(0);

	// view buttons
	char *tttext[] = { 
		"1 camera view",
		"4 camera view",
		"6 camera view",
		"8 camera view",
		"9 camera view",
		"10 camera view",
		"13 camera view",
		"16 camera view"
	};

	for (i = 0; i < 8; i++) {
		button = new Button(widget, 4, 24+28*i, 24, 24);
		button->SetToolTipText(tttext[i]);
		button->SetPixmap(Skin::GetSkinPixmap(VIEW1_NORMAL+i*3),
				STATE_NORMAL);
		button->SetPixmap(Skin::GetSkinPixmap(VIEW1_HOVER+i*3),
				STATE_HOVER);
		button->SetPixmap(Skin::GetSkinPixmap(VIEW1_PRESSED+i*3),
				STATE_PRESSED);
		button->SetCallback((void*)this, (void*)i,
				Devsec::ViewBtnCallback);
		button->DrawFrame(0);
		button->Show();
	}

	// other buttons
	
	// fullscreen button
	button = new Button(widget, 4, 312, 24, 24);
	button->SetToolTipText("Fullscreen Mode");
	button->SetToggleMode(1);
	button->SetPixmap(Skin::GetSkinPixmap(FULLSCREEN_NORMAL),
			STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(FULLSCREEN_HOVER),
			STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(FULLSCREEN_PRESSED),
			STATE_PRESSED);
	button->SetCallback((void*)this, NULL, FullscreenBtnCallback);
	button->Show();
	button->DrawFrame(0);

	// cycle button
	button = new Button(widget, 4, 340, 24, 24);
	button->SetToolTipText("Cycle Camera Views");
	button->SetToggleMode(1);
	button->SetPixmap(Skin::GetSkinPixmap(CYCLE_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(CYCLE_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(CYCLE_PRESSED), STATE_PRESSED);
	button->SetCallback((void*)this, NULL, CycleBtnCallback);
	button->Show();
	button->DrawFrame(0);

	// Config button
	button = new Button(widget, 4, 368, 24, 24);
	button->SetToolTipText("System Configuration");
	button->SetPixmap(Skin::GetSkinPixmap(CONFIG_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(CONFIG_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(CONFIG_PRESSED), STATE_PRESSED);
	button->SetCallback((void*)this, NULL, ConfigBtnCallback);
	button->Show();
	button->DrawFrame(0);

	// Fixate button
	button = new Button(widget, 4, 396, 24, 24);
	button->SetToolTipText("Cut live stream for playback");
	button->SetPixmap(Skin::GetSkinPixmap(FIXATE_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(FIXATE_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(FIXATE_PRESSED), STATE_PRESSED);
	button->SetCallback((void*)this, NULL, FixateBtnCallback);
	button->Show();
	button->DrawFrame(0);

	// statusbar
	widget = new Widget(mMainWindow, 0, 576, 900, 24);
	widget->SetPixmap(Skin::GetSkinPixmap(STATUSBAR));
	widget->Show();
	widget->DrawFrame(0);

	// Right hand toolbar
	widget = new Widget(mMainWindow, 800, 0, 100, 576);
	widget->SetPixmap(Skin::GetSkinPixmap(RIGHT_TOOLBAR));
	widget->Show();
	widget->DrawFrame(0);

	// HDD meter
	mHDDMeter = new Meter(widget, 18, 92, 26, 72, VERTICAL);
	mHDDMeter->Show();
	mHDDMeter->DrawFrame(0);
	mHDDMeter->SetPercent(0.0);
	mHDDMeter->SetPixmap(Skin::GetSkinPixmap(STATUSBAR), METER_BACKGROUND);
	mHDDMeter->SetPixmap(Skin::GetSkinPixmap(METER_NORMAL), METER_LEVEL);

	// MEM meter
	mMEMMeter = new Meter(widget, 56, 92, 26, 72, VERTICAL);
	mMEMMeter->Show();
	mMEMMeter->DrawFrame(0);
	mMEMMeter->SetPercent(0.0);
	mMEMMeter->SetPixmap(Skin::GetSkinPixmap(STATUSBAR), METER_BACKGROUND);
	mMEMMeter->SetPixmap(Skin::GetSkinPixmap(METER_NORMAL), METER_LEVEL);

	timer = new Timer(1, (void*)this, NULL, MeterTimerCallback);
	timer->Start();
	AddChild(timer);

	// Power button
	button = new Button(widget, 20, 6, 57, 49);
	button->SetPixmap(Skin::GetSkinPixmap(POWER_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(POWER_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(POWER_PRESSED), STATE_PRESSED);
	button->DrawFrame(0);
	button->Show();
	button->SetToolTipText("Power Button");
	button->SetCallback((void*)this,NULL, PowerButtonCallback);

	// Sysinfo
	txt = Config::GetCharValue("global", "button3", "Not Configured");
	button = new Button(widget, 50, 245, 39, 39);
	button->SetPixmap(Skin::GetSkinPixmap(SYSINFO_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(SYSINFO_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(SYSINFO_PRESSED), STATE_PRESSED);
	button->DrawFrame(0);
	button->Show();
	button->SetToolTipText(txt);
	button->SetCallback((void*)this,(void*)txt, AppButtonCallback);

	// Mplayer
	button = new Button(widget, 50, 226, 39, 39);
	button->SetPixmap(Skin::GetSkinPixmap(MPLAYER_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(MPLAYER_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(MPLAYER_PRESSED), STATE_PRESSED);
	button->DrawFrame(0);
	button->Hide();
	button->SetToolTipText(txt);
	button->SetCallback((void*)this,(void*)txt, AppButtonCallback);

	// kfind
	txt = Config::GetCharValue("global", "button0", "Not Configured");
	button = new Button(widget, 10, 245, 39, 39);
	button->SetPixmap(Skin::GetSkinPixmap(KFIND_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(KFIND_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(KFIND_PRESSED), STATE_PRESSED);
	button->DrawFrame(0);
	button->Show();
	button->SetToolTipText(txt);
	button->SetCallback((void*)this,(void*)txt, AppButtonCallback);

	// screen capture
	button = new Button(widget, 50, 265, 39, 39);
	button->SetPixmap(Skin::GetSkinPixmap(SCREENCAPTURE_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(SCREENCAPTURE_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(SCREENCAPTURE_PRESSED), STATE_PRESSED);
	button->DrawFrame(0);
	button->Hide();
	button->SetToolTipText(txt);
	button->SetCallback((void*)this,(void*)txt, AppButtonCallback);

	// cd bakeovem
	txt = Config::GetCharValue("global", "button1", "Not Configured");
	button = new Button(widget, 10, 284, 39, 39);
	button->SetPixmap(Skin::GetSkinPixmap(CDBAKE_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(CDBAKE_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(CDBAKE_PRESSED), STATE_PRESSED);
	button->DrawFrame(0);
	button->Show();
	button->SetToolTipText(txt);
	button->SetCallback((void*)this,(void*)txt, AppButtonCallback);

	// kdirstat
	txt = Config::GetCharValue("global", "button4", "Not Configured");
	button = new Button(widget, 50, 284, 39, 39);
	button->SetPixmap(Skin::GetSkinPixmap(DIRSTAT_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(DIRSTAT_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(DIRSTAT_PRESSED), STATE_PRESSED);
	button->DrawFrame(0);
	button->Show();
	button->SetToolTipText(txt);
	button->SetCallback((void*)this,(void*)txt, AppButtonCallback);

	// xffm
	txt = Config::GetCharValue("global", "button2", "Not Configured");
	button = new Button(widget, 10, 323, 39, 39);
	button->SetPixmap(Skin::GetSkinPixmap(BROWSE_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(BROWSE_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(BROWSE_PRESSED), STATE_PRESSED);
	button->DrawFrame(0);
	button->Show();
	button->SetToolTipText(txt);
	button->SetCallback((void*)this,(void*)txt, AppButtonCallback);

	// trash
	txt = Config::GetCharValue("global", "button5", "Not Configured");
	button = new Button(widget, 50, 323, 39, 39);
	button->SetPixmap(Skin::GetSkinPixmap(TRASH_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(TRASH_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(TRASH_PRESSED), STATE_PRESSED);
	button->DrawFrame(0);
	button->Show();
	button->SetToolTipText(txt);
	button->SetCallback((void*)this,(void*)txt, AppButtonCallback);

	// time settings
	txt = Config::GetCharValue("global", "button6", "Not Configured");
	button = new Button(widget, 10, 439, 39, 39);
	button->SetPixmap(Skin::GetSkinPixmap(TIME_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(TIME_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(TIME_PRESSED), STATE_PRESSED);
	button->DrawFrame(0);
	button->Show();
	button->SetToolTipText(txt);
	button->SetCallback((void*)this,(void*)txt, AppButtonCallback);

	// network settings
	txt = Config::GetCharValue("global", "button7", "Not Configured");
	button = new Button(widget, 50, 439, 39, 39);
	button->SetPixmap(Skin::GetSkinPixmap(NETWORK_NORMAL), STATE_NORMAL);
	button->SetPixmap(Skin::GetSkinPixmap(NETWORK_HOVER), STATE_HOVER);
	button->SetPixmap(Skin::GetSkinPixmap(NETWORK_PRESSED), STATE_PRESSED);
	button->DrawFrame(0);
	button->Show();
	button->SetToolTipText(txt);
	button->SetCallback((void*)this,(void*)txt, AppButtonCallback);

	// view window
	widget = new Widget(mMainWindow, 32, 0, 768, 576);
	widget->SetColor(0, 0, 0);
	widget->Show();
	widget->DrawFrame(0);

	// create the cameras
	CreateCameras(widget);
	return 0;
}

int Devsec::CreateCameras(Widget *parent)
{
	int i, j;
	int cameras;
	Popup *popup;
	char menustr[256];

	cameras = Config::GetIntValue("global", "Cameras", 1);

	if (!cameras) return 0;
	if (cameras > CAMERAS) cameras = CAMERAS;
	mNumCameras = cameras;

	mCameras = (Camera**)malloc(sizeof(Camera*)*mNumCameras);
	mLayoutList = (Camera**)malloc(sizeof(Camera*)*mNumCameras);

	memset(menustr, 0, 256);

	for (i = 0; i < mNumCameras; i++) {
		mCameras[i] = new Camera(parent, 0, 0, 384, 288, i);
		mCameras[i]->DrawFrame(0);
		mCameras[i]->Hide();
		mLayoutList[i] = mCameras[i];
	}
	
	for (j = 0; j < mNumCameras; j++) {
		memset(menustr, 0, 256);
		if (mCameras[j]->IsLocked()) {
			strncat(menustr, "Unlock View", 256);
		} else {
			strncat(menustr, "Lock View", 256);
		}
		for (i = 0; i < mNumCameras; i++) {
			if (i == j) continue;
			strncat(menustr, ",", 256);
			strncat(menustr, mCameras[i]->GetCameraName(), 256);
		}
		popup = new Popup(mMainWindow, menustr, mNumCameras);
		popup->SetCallback((void*)this, (void*)j, Devsec::PopupCallback);
		popup->SetPixmap(Skin::GetSkinPixmap(MENU_NORMAL),
				STATE_NORMAL);
		popup->SetPixmap(Skin::GetSkinPixmap(MENU_HOVER), STATE_HOVER);
		popup->SetPixmap(Skin::GetSkinPixmap(MENU_PRESSED),
				STATE_PRESSED);
		popup->DrawFrame(0);
		mCameras[j]->SetPopup(popup);
	}

	return 0;
}

int Devsec::DoLayout(int l)
{
	int i;

	mCurrentLayout = l;
	for (i = 0; i < mNumCameras; i++) {
		if (layout[l].data[i][0]) {
			mLayoutList[i]->Show();
			mLayoutList[i]->MoveResize(layout[l].data[i][1],
					layout[l].data[i][2],
					layout[l].data[i][3],
					layout[l].data[i][4]);
		} else {
			mLayoutList[i]->Hide();
		}
	}
	return 0;
}

int Devsec::DoFullscreen()
{
	int vm_event, vm_error;
	int i, j, have_vm = 0, x, y, mx, my;

	if (sFullscreen) {
		x = 800;
		y = 600;
	} else {
		x = 1024;
		y = 768;
	}

	if (XF86VidModeQueryExtension(Widget::GetXDisplay(), &vm_event, &vm_error)) {
		have_vm = 1;
	}

	if (have_vm) {
		if (sVidModes == NULL)
			XF86VidModeGetAllModeLines(Widget::GetXDisplay(), 0,
					&sModeCount, &sVidModes);

		j = 0;
		mx = sVidModes[0]->hdisplay;
		my = sVidModes[0]->vdisplay;

		for (i = 1; i < sModeCount; i++) {
			if ((sVidModes[i]->hdisplay >= x) &&
					(sVidModes[i]->vdisplay >= y)) {
				if ((sVidModes[i]->hdisplay <= mx) &&
						(sVidModes[i]->vdisplay <= my)){
					mx = sVidModes[i]->hdisplay;
					my = sVidModes[i]->vdisplay;
					j = i;
					goto found;
				}
			}
		}
found:
		XF86VidModeLockModeSwitch(Widget::GetXDisplay(), 0, 0);
		XF86VidModeSwitchToMode(Widget::GetXDisplay(), 0, sVidModes[j]);
		XF86VidModeSwitchToMode(Widget::GetXDisplay(), 0, sVidModes[j]);
		if (sFullscreen) {
			Window w;
			int x, y;
			XTranslateCoordinates(Widget::GetXDisplay(),
					mMainWindow->GetXWindow(),
					DefaultRootWindow(Widget::GetXDisplay()),
					0, 0, &x, &y, &w);
			XF86VidModeSetViewPort(Widget::GetXDisplay(), 0, x, y);
			XGrabPointer(Widget::GetXDisplay(),
					mMainWindow->GetXWindow(),
					True, PointerMotionMask,
					GrabModeAsync, GrabModeAsync,
					mMainWindow->GetXWindow(),
					None, CurrentTime);
		} else {
			XF86VidModeSetViewPort(Widget::GetXDisplay(), 0, 0, 0);
			XUngrabPointer(Widget::GetXDisplay(), CurrentTime);
		}
	}
}

int Devsec::DoSwapCameras(int camera1, int camera2)
{
	int x1, y1, w1, h1, v1;
	int x2, y2, w2, h2, v2;
	Camera *tmp;

	v1 = mCameras[camera1]->IsVisible();
	v2 = mCameras[camera2]->IsVisible();
	mCameras[camera1]->GetSize(&x1, &y1, &w1, &h1);
	mCameras[camera2]->GetSize(&x2, &y2, &w2, &h2);

	mCameras[camera1]->MoveResize(x2, y2, w2, h2);
	mCameras[camera2]->MoveResize(x1, y1, w1, h1);

	tmp = mLayoutList[camera1];
	mLayoutList[camera1] = mLayoutList[camera2];
	mLayoutList[camera2] = tmp;

	if (v1 == v2) return 0;

	if (v1) {
		mCameras[camera1]->Hide();
		mCameras[camera2]->Show();
	} else {
		mCameras[camera1]->Show();
		mCameras[camera2]->Hide();
	}
	return 0;
}

int Devsec::DoLockCamera(int camera)
{
	int locked = mCameras[camera]->IsLocked();
	mCameras[camera]->Lock(1 - locked);
	return 0;
}

int Devsec::DoCycle(int flag)
{
	if (flag == 1) {
		if (sCycling) mCycleTimer->Start();
		else mCycleTimer->Stop();
	} else if (flag == 0) {
		Camera *tmp;
		int i, j;
		int numUnlocked, unlocked[CAMERAS];

		numUnlocked = 0;
		for (i = 0; i < mNumCameras; i++) {
			if (mLayoutList[i]->IsLocked()) continue;
			unlocked[numUnlocked++] = i;
		}

		tmp = mLayoutList[unlocked[0]];
		for (i = 1; i < numUnlocked; i++) {
			mLayoutList[unlocked[i-1]] = mLayoutList[unlocked[i]];
		}
		mLayoutList[unlocked[i-1]] = tmp;
		DoLayout(mCurrentLayout);
	}
	return 0;
}

int Devsec::DoFixate()
{
	int i;
	for (i = 0; i < mNumCameras; i++) {
		mCameras[i]->CutStream();
	}
	return 0;
}

int Devsec::DoConfig()
{
	if (sFullscreen) DoFullscreen();
	if (mConfigDialog == NULL) {
		mConfigDialog = new ConfigDialog(mNumCameras, mCameras, this);
		mConfigDialog->Show();
		AddChild(mConfigDialog);
	} else mConfigDialog->Show();
	return 0;
}

int Devsec::DoWindowDestroy(Widget *window)
{
	if (window == NULL) return 0;

	if (window == mMainWindow) {
		mMainWindow->Hide();
		return 0;
	}
	return 0;
}

int Devsec::DoZombieKiller()
{
	int i;

	for (i = 0; i < mNumCameras; i++) {
		if (mCameras[i]->IsZOMBIE()) {
			fprintf(stderr, "restarting camera %d\n", i);
			mCameras[i]->RestartBackend();
			ZOMBIES--;
		}
	}
	if (ZOMBIES != 0) {
		ZOMBIES = 0;
	}
	return 0;
}

// Command format:
// CAMERA <CAMERA_NO> <UNICAST_ADDRESS> <UNICAST_PORT>
// CAMERA <CAMERA_NO> OFF
int Devsec::HandleRemoteControlCommand(char *command)
{
	int camera_no;
	char unicast_addr[64];
	char unicast_port[32];
	int rv = 0;

	if (command == NULL) return 0;
	if (strlen(command) == 0) return 0;
	rv = sscanf(command, "CAMERA %d %s %s", &camera_no, &unicast_addr,
			&unicast_port);
	if (rv == 0) return 0;
	else if (rv == 2) {
		if (!strcasecmp(unicast_addr, "OFF")) {
			mCameras[camera_no]->UnicastDisplay(NULL, NULL);
		}
	} else if (rv == 3) {
		mCameras[camera_no]->UnicastDisplay(unicast_addr, unicast_port);
	}
	return 0;
}

int Devsec::HandleRemoteControlCommands(char *commands)
{
	char *p, *d;

	p = commands;
	while (p) {
		d = strchr(p, '\n');
		if (d)  *d = '\0', d++;
		HandleRemoteControlCommand(p);
		p = d;
	}
	return 0;
}

int Devsec::DoRemoteControl()
{
	static int rcfd = -1;

	fd_set fds;
	struct timeval tv;
	int rv;

	if (rcfd < 0) {
		mkfifo(REMOTE_CONTROL_PIPE, 0666);
		rcfd = open(REMOTE_CONTROL_PIPE, O_RDONLY|O_NONBLOCK);
		fchmod(rcfd, 0666);
	}

	tv.tv_sec = 0;
	tv.tv_usec = 10;
	FD_ZERO(&fds);
	FD_SET(rcfd, &fds);

	rv = select(rcfd+1, &fds, NULL, NULL, &tv);
	if (rv > 0) {
		if (FD_ISSET(rcfd, &fds)) {
			char buf[256];
			rv = read(rcfd, buf, 256);
			if (rv < 0) {
				close(rcfd);
				rcfd = -1;
				return 0;
			} else if (rv > 0) {
				buf[rv] = '\0';
				HandleRemoteControlCommands(buf);
			}
		}
	}
	return 0;
}

int Devsec::DoShutdown()
{
	int i;
	DoFixate();
	for (i = 0; i < mNumCameras; i++) {
		if (mCameras[i]) {
			mCameras[i]->StopBackend();
			delete mCameras[i];
			mCameras[i] = NULL;
		}
	}
	exit(0);
}

int Devsec::CheckPidFile(int iter)
{
	FILE *f;
	if (iter > 1) {
		fprintf(stderr, "couldn't communicate with self\n");
		exit(-1);
	}

	f = fopen("/tmp/devsec.pid", "r");
	if (f == NULL) {
		int pid = getpid();
		f = fopen("/tmp/devsec.pid", "w");
		if (f == NULL) {
			fprintf(stderr, "couldn't write pid file\n");
			exit(-1);
		}
		fprintf(f, "%d\n", pid);
		fclose(f);
		return 0;
	} else {
		int rv = -1;
		pid_t pid = -1;
		fscanf(f, "%d", &pid);
		if (pid) rv = kill(pid, SIGHUP);
		fclose(f);
		if (rv != 0) {
			unlink("/tmp/devsec.pid");
			iter++;
			CheckPidFile(iter);
		} else {
			exit(0);
		}
	}
	return 0;
}

int Devsec::GetRemoteEvent(RemoteEvent *ev)
{
	char buf[256];
	int val = 0;
	Window window = 0;
	int w, h, visible;

	if (ev == NULL) return 0;
	memset(ev, 0, sizeof(RemoteEvent));

	if (mRemoteSocket == NULL) return 0;

	if (mRemoteSocket->Read(buf, 255) < 0) {
		fprintf(stderr, "remote view died\n");
		mRemoteSocket->Disconnect();
		return 0;
	}

	if (!strncmp(buf, "EV_VIEW_ID", 10)) {
		sscanf(buf, "EV_VIEW_ID %d %d %d %d %d", &val, &window,
				&w, &h, &visible);
		ev->type = VIEW_ID_EVENT_TYPE;
		ev->viewid.camera = val;
		ev->viewid.window = window;
		ev->viewid.w = w;
		ev->viewid.h = h;
		ev->viewid.visible = visible;
		return 1;
	}
	return 0;
}

int Devsec::DoViewId(int view, Window window, int w, int h, int visible)
{
	if (window != -1) mCameras[view]->RemoteParent(window);
	if ((w > 0) && (h > 0)) mCameras[view]->RemoteSize(w, h);
	if (visible < 0) return 0;
	if (visible) mCameras[view]->RemoteShow();
	else mCameras[view]->RemoteHide();

	return 0;
}

int Devsec::EventHandler(TEvent *ev)
{
	int rv = 0;
	int i;

	rv = Application::EventHandler(ev);

	if (ev == NULL) return 0;
	if (ev->type == NOEVENT) return 0;
	if (ev->type != TIMEEVENT) return 0;

	for (i = 0; i < mNumCameras; i++) {
		rv += mCameras[i]->IsAlive();
	}

	if (rv != mNumCameras) return 0;

	if (mRemoteSocket->Pending()) {
		RemoteEvent rev;
		if (GetRemoteEvent(&rev)) {
			switch (rev.type) {
				case VIEW_ID_EVENT_TYPE:
					DoViewId(rev.viewid.camera,
							rev.viewid.window,
							rev.viewid.w,
							rev.viewid.h,
							rev.viewid.visible);
					break;
			}
		}
	}
	return 0;
}

//---------------------------------------------------------------------------
// CALLBACKS


// d1, Pointer to Devsec object
// d2, int to active camera view
void Devsec::PopupCallback(Object *o, TEvent *e, int entry, void *d1, void *d2)
{
	Devsec *devsec = (Devsec*)d1;
	int active = (int)d2;

	if (e->type != XEVENT) return;

	if (e->xev->type != ButtonRelease) return;

	if (entry == 0) {
		devsec->DoLockCamera(active);
		return;
	}
	if (active >= entry) entry--;
	devsec->DoSwapCameras(active, entry);
}

// d1 , Pointer to Devsec object
// d2 , view number
void Devsec::ViewBtnCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	Devsec *devsec = (Devsec*)d1;
	int viewNumber = (int)d2;

	if (e->type != XEVENT) return;

	if (e->xev->type != ButtonRelease) return;

	devsec->DoLayout(viewNumber);
}

void Devsec::FullscreenBtnCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	Button *b = (Button*)o;
	Devsec *devsec = (Devsec*)d1;

	if (e->type != XEVENT) return;

	if (e->xev->type != ButtonRelease) return;
	sFullscreen = b->GetToggle();
	devsec->DoFullscreen();
}

void Devsec::CycleBtnCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	Devsec *devsec = (Devsec*)d1;
	Button *b = (Button*)o;

	if (e->type != XEVENT) return;

	if (e->xev->type != ButtonRelease) return;
	sCycling = b->GetToggle();
	if (sCycling) devsec->DoCycle(0);
	devsec->DoCycle(1);
}

void Devsec::ConfigBtnCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	Devsec *devsec = (Devsec*)d1;

	if (e->type != XEVENT) return;

	if (e->xev->type != ButtonRelease) return;

	if (devsec->CheckAuthentication() != DIALOG_BUTTON_OK) return;

	devsec->DoConfig();
}

void Devsec::FixateBtnCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	Devsec *devsec = (Devsec*)d1;

	if (e->type != XEVENT) return;

	if (e->xev->type != ButtonRelease) return;
	devsec->DoFixate();
}

void Devsec::WindowCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	Devsec *devsec = (Devsec*)d1;

	if (e->type != DTKEVENT) return;

	if (e->dev->type == DTK_WINDOW_DESTROY)
		devsec->DoWindowDestroy((Widget*)e->dev->destroy.object);
}

int Devsec::CheckAuthentication()
{
	int rv;
	FILE *f;

	if (sAuthentication == 0) return 1;

	f = fopen("/etc/dpasswd", "r");
	if (f == NULL) return DIALOG_BUTTON_OK;
	fclose(f);

	AuthDialog *dlg = new AuthDialog(mMainWindow, NULL, 352, 284, 320, 160,
			DIALOG_BUTTON_OKCANCEL);
	rv = RunDialog(dlg);
	delete dlg;
	return rv;
}

void Devsec::AppButtonCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	char buf[512];

	Devsec *d = (Devsec*)d1;
	char *command = (char *)d2;
	if (command == NULL) return;

	if (Devsec::sFullscreen) return;
	if (e->type != XEVENT) return;
	if (e->xev->type != ButtonRelease) return;

	if (d->CheckAuthentication() != DIALOG_BUTTON_OK) return;

	snprintf(buf, 512, "%s &", command);
	my_system(buf);
}

void Devsec::PowerButtonCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	Devsec *d = (Devsec*)d1;
	if (Devsec::sFullscreen) return;
	if (e->type != XEVENT) return;
	if (e->xev->type != ButtonRelease) return;
	if (d->CheckAuthentication() != DIALOG_BUTTON_OK) return;

	d->DoShutdown();
}

void Devsec::CycleTimerCallback(void *data1, void *data2)
{
	Devsec *devsec = (Devsec*)data1;

	if (devsec) devsec->DoCycle(0);
}

void Devsec::DeleteTimerCallback(void *data1, void *data2)
{
	char buf[256];
	char filename[64];
	int f;

	if (sDeleteOldFiles == 0) return;

	snprintf(filename, 64, ".check");
	snprintf(buf, 256, STREAM_DIR "/%s", filename);
	f = open(buf, O_CREAT | O_WRONLY);
	if (f > 0) close(f);
	else fprintf(stderr, "could not create temporary file\n");
	int minFree = Config::GetIntValue("global", "minFreeSpace", 20);
	snprintf(buf, 64, "devsec_helper delete %s %d; rm -f %s&",
			filename, minFree, filename);
	my_system(buf);
}

void Devsec::WatchdogTimerCallback(void *data1, void *data2)
{
	char buf[64];
	int fd = (int)data1;

	if (fd <= 0) return;

	snprintf(buf, 64, "devsec alive\n");
	write(fd, &buf[0], 12);
}

void Devsec::ZOMBIETimerCallback(void *data1, void *data2)
{
	Devsec *d = (Devsec*)data1;

	if (!Devsec::ZOMBIES) return;

	d->DoZombieKiller();
}

void Devsec::RemoteControlTimerCallback(void *data1, void *data2)
{
	Devsec *d = (Devsec*)data1;
	d->DoRemoteControl();
}

void Devsec::SigHandler(int signum)
{
	if (signum == SIGHUP) {
		if (mMainWindow) {
			XMapWindow(mMainWindow->GetXDisplay(),
					mMainWindow->GetXWindow());
		}
	} else if (signum == SIGCHLD) {
		ZOMBIES++;
	}
}

void Devsec::MeterTimerCallback(void *data1, void *data2)
{
	Devsec *d = (Devsec*)data1;
	d->DoUpdateMeters();
}
