#ifndef __DEVSEC_H
#define __DEVSEC_H

#include <signal.h>

#include "dtk_application.h"
#include "dtk_meter.h"
#include "dtk_widget.h"
#include "dtk_timer.h"
#include "dtk_config.h"
#include "dtk_config_dialog.h"
#include "dtk_socket.h"

#include "camera.h"
#include "log.h"

#ifndef CAMERAS
#define CAMERAS 16
#endif

#define REMOTE_CONTROL_PIPE "/tmp/devsec_remote_control_pipe"

int my_system(char *cmd);

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/extensions/xf86vmode.h>

class Devsec : public Application {
public:
	Devsec();
	virtual ~Devsec();
	virtual int Init(int argc, char **argv);
	virtual int Destroy();

	virtual int GetAuthentication();
	virtual int SetAuthentication(int flag);
	virtual int GetRemoteInViewList(int cameraNo);
	virtual int SetRemoteInViewList(int cameraNo, int flag);
	virtual int SetCycleTimeout(unsigned int timeout);
	virtual unsigned int GetCycleTimeout();
	virtual int SetDeleteOldFiles(int flag);
	virtual int GetDeleteOldFiles();
	virtual int SaveConfig();
	virtual int LoadConfig();

	virtual int SetPhoneNumber(char *number);
	virtual char *GetPhoneNumber();
	virtual int SetGlobalDetect(int flag);
	virtual int GetGlobalDetect();
	virtual int SetGlobalNotify(int flag);
	virtual int GetGlobalNotify();
	virtual int DoRemoteLayout(int layout);
	virtual int DoRemoteCycle(int flag);
	virtual int DoUpdateMeters();
	virtual int DoLayout(int layout);
	virtual inline int GetCurrentLayout() { return mCurrentLayout; };

	virtual int EventHandler(TEvent *ev);

	static int sAudibleRecord;

protected:
	virtual int CheckAuthentication();
	virtual int GetRemoteEvent(RemoteEvent *ev);

	virtual int DoViewId(int view, Window window,
			int w, int h, int visible);

	virtual int CreateMainWindow();
	virtual int CreateCameras(Widget *parent);

	virtual int HandleRemoteControlCommand(char *command);
	virtual int HandleRemoteControlCommands(char *commands);

	virtual int DoFullscreen();
	virtual int DoSwapCameras(int camera1, int camera2);
	virtual int DoLockCamera(int camera);
	virtual int DoCycle(int flag);
	virtual int DoFixate();
	virtual int DoConfig();
	virtual int DoWindowDestroy(Widget *window);
	virtual int DoZombieKiller();
	virtual int DoRemoteControl();
	virtual int DoShutdown();

	virtual int CheckPidFile(int iter);

	static void SigHandler(int signum);

	static void PopupCallback(Object *o, TEvent *e, int entry,
			void *d1, void *d2);
	static void ViewBtnCallback(Object *o, TEvent *e, void *d1, void* d2);
	static void FullscreenBtnCallback(Object *o, TEvent *e,
			void *d1, void *d2);
	static void CycleBtnCallback(Object *o, TEvent *e, void *d1, void *d2);
	static void ConfigBtnCallback(Object *o, TEvent *e, void *d1, void *d2);
	static void FixateBtnCallback(Object *o, TEvent *e, void *d1, void *d2);
	static void WindowCallback(Object *o, TEvent *e, void *d1, void *d2);
	static void AppButtonCallback(Object *o, TEvent *e, void *d1, void *d2);
	static void PowerButtonCallback(Object *o,TEvent *e,void *d1, void *d2);

	static void CycleTimerCallback(void *data1, void *data2);
	static void DeleteTimerCallback(void *data1, void *data2);
	static void WatchdogTimerCallback(void *data1, void *data2);
	static void ZOMBIETimerCallback(void *data1, void *data2);
	static void MeterTimerCallback(void *data1, void *data2);
	static void RemoteControlTimerCallback(void *data1, void *data2);

	static int sCycling;
	static int sFullscreen;
	static XF86VidModeModeInfo **sVidModes;
	static int sModeCount;

	int mNumCameras;
	int mCurrentLayout;

	Timer *mCycleTimer;
	Timer *mDeleteTimer;
	Timer *mWatchdogTimer;
	Timer *mZOMBIETimer;
	Timer *mRemoteControlTimer;

	unsigned int mCycleTimeout;
	int mRemoteCycle;
	int mRemoteLayout;
	int mRecordMotion;
	int mNotifyMotion;
	char *mPhoneNumber;

	static int sAuthentication;
	static int sDeleteOldFiles;
	static Widget *mMainWindow;
	static int ZOMBIES;

	Camera **mCameras;
	Camera **mLayoutList;

	ConfigDialog *mConfigDialog;
	Socket *mRemoteSocket;

	Meter *mHDDMeter;
	Meter *mMEMMeter;
};
#endif
