#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "dtk_basewidget.h"
#include "dtk_config.h"
#include "devsecremote.h"
#include "layouts.h"

#include "config.h"

DevsecRemote::DevsecRemote()
{
	mSocket = NULL;
	mWindow = NULL;
	mCameras = NULL;
	mCycleTimer = NULL;
	mCurrentLayout = 0;
	mVisible = NULL;
}

DevsecRemote::~DevsecRemote()
{
	if (mSocket) {
		RemoveChild(mSocket);
		delete mSocket;
	}
	if (mWindow) {
		RemoveChild(mWindow);
		delete mWindow;
	}
	if (mCycleTimer) {
		RemoveChild(mCycleTimer);
		delete mCycleTimer;
	}
}

int DevsecRemote::Init(int argc, char **argv)
{
	int i;
	Application::Init();

	Config::ReadConfig("global", DTK_CONFDIR "/devsec/global.conf");

	BaseWidget::SetDisplayStr(":0.1");
	if (BaseWidget::XConnect() == NULL) {
		fprintf(stderr, "cannot connect to remote display\n");
		exit(1);
	}

	mNumCameras = Config::GetIntValue("global", "Cameras", CAMERAS);
	mCameras = (Widget**)malloc(sizeof(Widget*)*mNumCameras);
	mLayoutList = (Widget**)malloc(sizeof(Widget*)*mNumCameras);
	mLayoutNos = (int*)malloc(sizeof(int)*mNumCameras);
	mVisible = (int*)malloc(sizeof(int*)*mNumCameras);

	mSocket = new SocketClient();
	mSocket->Connect("/tmp/devsecremote.sock");
	AddChild(mSocket);

	mWindow = new Widget(NULL, 0, 0, 800, 600);
	mWindow->WMDecorations(False);
	mWindow->SetColor(0, 0, 0);
	mWindow->Show();
	AddChild(mWindow);

	for (i = 0; i < mNumCameras; i++) {
		mCameras[i] = new Widget(mWindow, 0, 0, 384, 288);
		if (mSocket) {
			char buf[256];
			snprintf(buf, 256, "EV_VIEW_ID %d %d -1 -1 -1\n", i,
					mCameras[i]->GetXWindow());
			mSocket->Write(buf);
		}
	}

	DoConfig();
	return 0;
}

int DevsecRemote::Destroy()
{
	return 0;
}

int DevsecRemote::DoLayout(int l)
{
	int i, j;

	mCurrentLayout = l;

	l--;
	if (l < 0) l = 0;
	if (l > 7) l = 7;

	j = 0;
	for (i = 0; i < mNumCameras; i++) {

		if (layout[l].data[j][0]) {
			mLayoutList[i]->Show();
			mLayoutList[i]->MoveResize(16 + layout[l].data[j][1],
					12 + layout[l].data[j][2],
					layout[l].data[j][3],
					layout[l].data[j][4]);
			if (mSocket) {
				char buf[256];
				snprintf(buf, 256,"EV_VIEW_ID %d -1 %d %d %d\n",
						mLayoutNos[i],
						layout[l].data[j][3],
						layout[l].data[j][4],
						layout[l].data[j][0]);
				mSocket->Write(buf);
			}
		} else {
			mLayoutList[i]->Hide();
		}
		j++;
	}
	return 0;
}

int DevsecRemote::DoCycle(int flag)
{
	if (mCycleTimer == NULL) {
		mCycleTimer = new Timer(3,(void*)this,NULL,CycleTimerCallback);
		mCycleTimer->Start();
		AddChild(mCycleTimer);
	}

	if (flag == 0) {
		mCycleTimer->Stop();
	} else if (flag == 1) {
		mCycleTimer->Start();
	} else if (flag == 2) {
		Widget *tmp;
		int i, j, k;

		tmp = mLayoutList[0];
		k = mLayoutNos[0];
		for (i = 1; i < mNumCameras; i++) {
			mLayoutList[i-1] = mLayoutList[i];
			mLayoutNos[i-1] = mLayoutNos[i];
		}
		mLayoutList[i-1] = tmp;
		mLayoutNos[i-1] = k;
		DoLayout(mCurrentLayout);
	}

	return 0;
}

int DevsecRemote::DoConfig()
{
	int i, j, maxVisible;
	Config::ReadConfig("global", DTK_CONFDIR "/devsec/global.conf");

	maxVisible = Config::GetIntValue("global", "Cameras", CAMERAS);
	for (i = 0; i < maxVisible; i++) {
		char buf[256];

		snprintf(buf, 256, "remoteCamera%d", i);
		mVisible[i] = Config::GetIntValue("global", buf, 1);
	}

	j = 0;
	for (i = 0; i < maxVisible; i++) {
		mCameras[i]->Hide();
		if (mVisible[i]) {
			mLayoutList[j] = mCameras[i];
			mLayoutNos[j] = j;
			j++;
		}
	}
	mNumCameras = j;
	DoLayout(Config::GetIntValue("global","remoteLayout",1));
	DoCycle(Config::GetIntValue("global", "remoteCycle", 0));

	return 0;
}

int DevsecRemote::EventHandler(TEvent *ev)
{
	int rv = 0;
	
	rv = Application::EventHandler(ev);

	if (ev == NULL) return 0;
	if (ev->type == NOEVENT) return 0;
	if (ev->type != TIMEEVENT) return 0;

	if (mSocket->Pending()) {
		RemoteEvent rev;
		if (!GetRemoteEvent(&rev)) return 0;
		switch (rev.type) {
			case LAYOUT_EVENT_TYPE:
				DoLayout(rev.layout.layout);
				break;
			case CYCLE_EVENT_TYPE:
				DoCycle(rev.cycle.flag);
				break;
			case CONFIG_EVENT_TYPE:
				DoConfig();
				break;
		}
	}
	return 0;
}


int DevsecRemote::GetRemoteEvent(RemoteEvent *rev)
{
	char buf[256];
	int val = 0;
	if (rev == NULL) return 0;
	memset(rev, 0, sizeof(RemoteEvent));

	if (mSocket == NULL) return 0;

	if (mSocket->Read(buf, 255) < 0) {
		fprintf(stderr, "devsec lost\n");
		exit(1);
	}
	if (!strncmp(buf, "EV_LAYOUT", 9)) {
		sscanf(buf, "EV_LAYOUT %d", &val);
		rev->type = LAYOUT_EVENT_TYPE;
		rev->layout.layout = val;
		return 1;
	} else if (!strncmp(buf, "EV_CYCLE", 8)) {
		sscanf(buf, "EV_CYCLE %d", &val);
		rev->type = CYCLE_EVENT_TYPE;
		rev->cycle.flag = val;
		return 1;
	} else if (!strncmp(buf, "EV_CONFIG", 9)) {
		rev->type = CONFIG_EVENT_TYPE;
		return 1;
	}
	return 0;
}

void DevsecRemote::CycleTimerCallback(void *d1, void *d2)
{
	DevsecRemote *d = (DevsecRemote*)d1;

	if (d) d->DoCycle(2);
}
