#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "dtk_skin.h"
#include "dtk_checkbox.h"
#include "dtk_textentry.h"

#include "dialog_camera.h"
#include "devsec.h"
#include "camera.h"

int DialogCamera::ignore = 0;

DialogCamera::DialogCamera(Widget *parent, int x, int y, int w, int h,
		int numCameras, Camera **c, Devsec *d)
	: Widget(parent, x, y, w, h)
{
	mApp = d;
	mCameras = c;
	mNumCameras = numCameras;
	mCurrentCamera = -1;
	mCurrentLayout = mApp->GetCurrentLayout();
	CreateWindows();
}

DialogCamera::~DialogCamera()
{
}

int DialogCamera::entering()
{
	mCurrentLayout = mApp->GetCurrentLayout();
	return 0;
}

int DialogCamera::leaving()
{
	int i;
	SetPreviewWindow(-1);
	for (i = 0; i < mNumCameras; i++) {
		mCameras[i]->Show();
	}
	mApp->DoLayout(mCurrentLayout);
	return 0;
}

int DialogCamera::Show()
{
	int i;
	for (i = 0; i < mNumCameras; i++) {
		mCameras[i]->Hide();
	}
	if (mCurrentCamera > -1) {
		FillPanel(mCurrentCamera);
		SetPreviewWindow(mCurrentCamera);
	} else {
		OnCameraListChanged(0);
		mCameraList->SetText("Camera 0");
	}
	return Widget::Show();
}

int DialogCamera::Hide()
{
	leaving();
	return Widget::Hide();
}

int DialogCamera::SetPreviewWindow(int index)
{
	Widget *cameraParent;
	Window parent;
	Window cameraview;
	int w, h;
	
	if (index == -1) {
		if (mCurrentCamera == -1) return 0;
		cameraParent = (Widget*)mCameras[mCurrentCamera]->GetParentWidget();
		cameraview = mCameras[mCurrentCamera]->GetXWindow();
		if (cameraParent == NULL) {
			XReparentWindow(mDisplay, cameraview,
					DefaultRootWindow(mDisplay),
					0, 0);
			return 0;
		}

		parent = cameraParent->GetXWindow();
		cameraParent->GetSize(NULL, NULL, &w, &h);

		XReparentWindow(mDisplay, cameraview, parent, 0, 0);
		mCameras[mCurrentCamera]->Resize(w, h);
		mCameras[mCurrentCamera]->Hide();
		return 0;
	}
	if (index > mNumCameras) return 0;

	mCameras[index]->Show();
	parent = mPreviewWindow->GetXWindow();
	cameraview = mCameras[index]->GetXWindow();
	mPreviewWindow->GetSize(NULL, NULL, &w, &h);

	XReparentWindow(mDisplay, cameraview, parent, 0, 0);
	mCameras[mCurrentCamera]->Resize(w, h);
	return 0;
}

int DialogCamera::OnCameraListChanged(int index)
{
	SetPreviewWindow(-1);
	FillPanel(index);
	SetPreviewWindow(index);
	return 0;
}

int DialogCamera::OnSliderChanged(int slider)
{
	if (mCurrentCamera < 0) return 0;
	switch (slider) {
		case 1: 
			mCameras[mCurrentCamera]->SetSensitivity(
					(int)mSensitivity->GetValue());
			break;
		case 2:
			mCameras[mCurrentCamera]->V4LBrightness(
					(int)mBrightness->GetValue());
			break;
		case 3:
			mCameras[mCurrentCamera]->V4LContrast(
					(int)mContrast->GetValue());
			break;
		case 4:
			mCameras[mCurrentCamera]->V4LColor(
					(int)mColor->GetValue());
			break;
		case 5:
			mCameras[mCurrentCamera]->SetBitrate(
					(int)mBitrate->GetValue()+128);
			break;
	}
	return 0;
}

int DialogCamera::OnBoundsChanged()
{
	int x1, y1, x2, y2;

	if (mCurrentCamera < 0) return 0;
	x1 = mBoundsLeft->GetValue();
	y1 = mBoundsTop->GetValue();
	x2 = mBoundsRight->GetValue();
	y2 = mBoundsBottom->GetValue();
	mCameras[mCurrentCamera]->MotionBounds(x1, y1, x2, y2);
	return 0;
}

int DialogCamera::OnCheckToggled(int toggle)
{
	if (mCurrentCamera < 0) return 0;
	switch (toggle) {
		case 1:
			mCameras[mCurrentCamera]->ShowMotion(
					mShowMotion->GetToggle());
			break;
		case 2:
			mCameras[mCurrentCamera]->ShowBounds(
					mShowBounds->GetToggle());
			break;
	}
	return 0;
}

int DialogCamera::OnCameraNameChanged(int camera)
{
	char *name;

	name = mCameraName->GetBufferPtr();
	if (mCameraName->GetBufferLength() == 0) return 0;

	mCameras[camera]->SetCameraName(name);
	return 0;
}


int DialogCamera::CreateWindows()
{
	Widget *frame;
	Widget *image;
	Widget *label;
	char combostr[1024];

	SetColor(0xe600, 0xf700, 0xff00);

	image = new Widget(this, 0, 0, mW, 30);
	image->Show();
	image->SetPixmap(Skin::GetSkinPixmap(CONFIG_HDR_CAMERAS));

	frame = new Widget(this, 8, 48, (mW - 24) / 2, 178);
	frame->Show();
	frame->SetColor(0xe600, 0xf700, 0xff00);
	frame->DrawFrame(1);
	frame->SetText("General", 2, 12);

	label = new Widget(frame, 130, 24, 48, 16);
	label->Show();
	label->DrawFrame(0);
	label->SetText("Camera", 0, 12);
	label->SetColor(0xe600, 0xf700, 0xff00);

	FillComboBox(combostr, 1024);
	mCameraList = new ComboBox(frame, combostr, mNumCameras,
			40, 48, 240, 24);
	mCameraList->Show();
	mCameraList->SetCallback((void*)this, NULL, ComboChanged);
	mCameraList->SetToolTipText("Select a camera from the drop down list");

	label = new Widget(frame, 115, 104, 80, 16);
	label->Show();
	label->DrawFrame(0);
	label->SetText("Camera Name", 0, 12);
	label->SetColor(0xe600, 0xf700, 0xff00);

	mCameraName = new TextEntry(frame, 40, 128, 230, 24);
	mCameraName->Show();
	mCameraName->SetToolTipText("Enter the camera name here");

	frame = new Widget(this, (mW) / 2, 48, (mW - 24) / 2, 178);
	frame->Show();
	frame->SetColor(0xe600, 0xf700, 0xff00);
	frame->DrawFrame(1);
	frame->SetText("Motion Detection", 2, 12);

	label = new Widget(frame, 120, 24, 128, 24);
	label->Show();
	label->DrawFrame(0);
	label->SetText("Sensitivity", 0, 22);
	label->SetColor(0xe600, 0xf700, 0xff00);

	mSensitivity = new Slider(frame, 8, 56, (mW - 48) / 2, 16, HORIZONTAL);
	mSensitivity->Show();
	mSensitivity->SetValue(50.0);
	mSensitivity->SetRange(0, 320);
	mSensitivity->SetCallback((void*)this, (void*)1, SliderChanged);
	mSensitivity->SetToolTipText("Adjust the sensitivity of the motion detection");

	label = new Widget(frame, 8, 112, 48, 24);
	label->Show();
	label->DrawFrame(0);
	label->SetText("Bounds:", 0, 16);
	label->SetColor(0xe600, 0xf700, 0xff00);

	label = new Widget(frame, 64, 88, 48, 24);
	label->Show();
	label->DrawFrame(0);
	label->SetText("Left", 0, 22);
	label->SetColor(0xe600, 0xf700, 0xff00);

	label = new Widget(frame, 128, 88, 48, 24);
	label->Show();
	label->DrawFrame(0);
	label->SetText("Top", 0, 22);
	label->SetColor(0xe600, 0xf700, 0xff00);

	label = new Widget(frame, 192, 88, 48, 24);
	label->Show();
	label->DrawFrame(0);
	label->SetText("Right", 0, 22);
	label->SetColor(0xe600, 0xf700, 0xff00);

	label = new Widget(frame, 256, 88, 48, 24);
	label->Show();
	label->DrawFrame(0);
	label->SetText("Bottom", 0, 22);
	label->SetColor(0xe600, 0xf700, 0xff00);

	mBoundsLeft = new SpinBox(frame, 64, 112, 48, 24);
	mBoundsLeft->Show();
	mBoundsLeft->SetRange(0, 383);
	mBoundsLeft->SetCallback((void*)this, NULL, BoundsChanged);
	mBoundsLeft->SetToolTipText("Set the left of the motion detection area");

	mBoundsTop = new SpinBox(frame, 128, 112, 48, 24);
	mBoundsTop->Show();
	mBoundsTop->SetRange(13, 287);
	mBoundsTop->SetCallback((void*)this, NULL, BoundsChanged);
	mBoundsTop->SetToolTipText("Set the top of the motion detection area");

	mBoundsRight = new SpinBox(frame, 192, 112, 48, 24);
	mBoundsRight->Show();
	mBoundsRight->SetRange(0, 383);
	mBoundsRight->SetValue(383);
	mBoundsRight->SetCallback((void*)this, NULL, BoundsChanged);
	mBoundsRight->SetToolTipText("Set the right of the motion detection area");

	mBoundsBottom = new SpinBox(frame, 256, 112, 48, 24);
	mBoundsBottom->Show();
	mBoundsBottom->SetRange(13, 287);
	mBoundsBottom->SetValue(287);
	mBoundsBottom->SetCallback((void*)this, NULL, BoundsChanged);
	mBoundsBottom->SetToolTipText("Set the base of the motion detection area");

	mShowMotion = new CheckBox(frame, 8, 152);
	mShowMotion->Show();
	mShowMotion->SetCallback((void*)this, (void*)1, CheckToggled);
	mShowMotion->SetToolTipText("Display the motion detection box");

	label = new Widget(frame, 28, 152, 112, 16);
	label->Show();
	label->DrawFrame(0);
	label->SetText("Show Motion Box", 0, 12);
	label->SetColor(0xe600, 0xf700, 0xff00);
	
	mShowBounds = new CheckBox(frame, 156, 152);
	mShowBounds->Show();
	mShowBounds->SetCallback((void*)this, (void*)2, CheckToggled);
	mShowBounds->SetToolTipText("Display the motion bounds box");

	label = new Widget(frame, 176, 152, 112, 16);
	label->Show();
	label->DrawFrame(0);
	label->SetText("Show Bounds Box", 0, 12);
	label->SetColor(0xe600, 0xf700, 0xff00);

	frame = new Widget(this, 8, 230, 392, 306);
	frame->Show();
	frame->SetText("Camera Preview", 2, 12);
	frame->SetColor(0xe600, 0xf700, 0xff00);

	mPreviewWindow = new Widget(frame, 4, 14, 384, 288);
	mPreviewWindow->Show();
	mPreviewWindow->SetColor(0, 0, 0);

	frame = new Widget(this, 408, 230, 244, 306);
	frame->Show();
	frame->SetText("Stream Config", 2, 12);
	frame->SetColor(0xe600, 0xf700, 0xff00);

	label = new Widget(frame, 85, 24, 74, 16);
	label->Show();
	label->DrawFrame(0);
	label->SetText("Brightness", 0, 12);
	label->SetColor(0xe600, 0xf700, 0xff00);

	mBrightness = new Slider(frame, 16, 42, 212, 16, HORIZONTAL);
	mBrightness->Show();
	mBrightness->SetRange(0, 65535);
	mBrightness->SetCallback((void*)this, (void*)2, SliderChanged);
	mBrightness->SetToolTipText("Set the brightness of the camera");

	label = new Widget(frame, 90, 74, 64, 16);
	label->Show();
	label->DrawFrame(0);
	label->SetText("Contrast", 0, 12);
	label->SetColor(0xe600, 0xf700, 0xff00);

	mContrast = new Slider(frame, 16, 92, 212, 16, HORIZONTAL);
	mContrast->Show();
	mContrast->SetRange(0, 65535);
	mContrast->SetCallback((void*)this, (void*)3, SliderChanged);
	mContrast->SetToolTipText("Set the contrast of the camera");


	label = new Widget(frame, 98, 124, 48, 16);
	label->Show();
	label->DrawFrame(0);
	label->SetText("Color", 0, 12);
	label->SetColor(0xe600, 0xf700, 0xff00);

	mColor = new Slider(frame, 16, 142, 212, 16, HORIZONTAL);
	mColor->Show();
	mColor->SetRange(0, 65535);
	mColor->SetCallback((void*)this, (void*)4, SliderChanged);
	mColor->SetToolTipText("Set the color of the camera");

	label = new Widget(frame, 96, 190, 50, 16);
	label->Show();
	label->DrawFrame(0);
	label->SetText("Bitrate", 0, 12);
	label->SetColor(0xe600, 0xf700, 0xff00);

	mBitrate = new Slider(frame, 16, 208, 212, 16, HORIZONTAL);
	mBitrate->Show();
	mBitrate->SetRange(0, 2272);
	mBitrate->SetCallback((void*)this, (void*)5, SliderChanged);
	mBitrate->SetToolTipText("Set the bitrate of the mp4 stream");
}

int DialogCamera::FillPanel(int index)
{
	if (index > mNumCameras) return 0;

	mCurrentCamera = index;

	ignore = 1;
	mCameraName->SetCallback(NULL, NULL, NULL);
	mCameraName->SetBuffer(mCameras[index]->GetCameraName());
	mCameraName->SetCallback((void*)this, (void*)index, NameChanged);
	mSensitivity->SetValue(mCameras[index]->GetSensitivity());
	mBoundsLeft->SetValue(mCameras[index]->GetBoundsX1());
	mBoundsTop->SetValue(mCameras[index]->GetBoundsY1());
	mBoundsRight->SetValue(mCameras[index]->GetBoundsX2());
	mBoundsBottom->SetValue(mCameras[index]->GetBoundsY2());
	mShowMotion->SetToggle(mCameras[index]->GetShowMotion());
	mShowBounds->SetToggle(mCameras[index]->GetShowBounds());
	mBrightness->SetValue(mCameras[index]->GetBrightness());
	mContrast->SetValue(mCameras[index]->GetContrast());
	mColor->SetValue(mCameras[index]->GetColor());
	mBitrate->SetValue(mCameras[index]->GetBitrate()-128);
	ignore = 0;
	return 0;
}

int DialogCamera::FillComboBox(char *combostr, int length)
{
	int i;
	char buf[256];
	if (combostr == NULL) return 0;

	memset(combostr, 0, length);

	for (i = 0; i < mNumCameras; i++) {
		snprintf(buf, 256, "Camera %d", mCameras[i]->GetCameraNo());
		strncat(combostr, buf, length);
		strncat(combostr, ",", 256);
	}
	return 0;
}

void DialogCamera::ComboChanged(Object *o, TEvent *e, void *d1, void *d2)
{
	DialogCamera *d = (DialogCamera*)d1;
	ComboBox *c;
	
	if (ignore) return;
	if (e->type != DTKEVENT) return;
	if (e->dev->type != DTK_COMBO_CHANGED) return;
	c = (ComboBox*)e->dev->combobox.object;
	d->OnCameraListChanged(c->GetSelected());
}

void DialogCamera::SliderChanged(Object *o, TEvent *e, void *d1, void *d2)
{
	DialogCamera *d = (DialogCamera*)d1;
	int slider = (int)d2;

	if (ignore) return;
	if (e->type != DTKEVENT) return;
	if (e->dev->type != DTK_SLIDER_CHANGED) return;
	d->OnSliderChanged(slider);
}

void DialogCamera::BoundsChanged(Object *o, TEvent *e, void *d1, void *d2)
{
	DialogCamera *d = (DialogCamera*)d1;

	if (ignore) return;
	if (e->type != DTKEVENT) return;
	if (e->dev->type != DTK_SPINBOX_CHANGED) return;
	d->OnBoundsChanged();
}

void DialogCamera::CheckToggled(Object *o, TEvent *e, void *d1, void *d2)
{
	DialogCamera *d = (DialogCamera*)d1;
	int toggle = (int)d2;

	if (ignore) return;
	if (e->type != XEVENT) return;
	if (e->xev->type != ButtonRelease) return;
	d->OnCheckToggled(toggle);
}

void DialogCamera::NameChanged(Object *o, TEvent *e, void *d1, void *d2)
{
	DialogCamera *d = (DialogCamera*)d1;
	int camera = (int)d2;

	if (e->type != DTKEVENT) return;

	if (e->dev->type != DTK_TEXTENTRY_CHANGED) return;
	d->OnCameraNameChanged(camera);
}
