#include <stdio.h>

#include "dtk_skin.h"
#include "dialog_global.h"
#include "devsec.h"

DialogGlobal::DialogGlobal(Widget *parent, int x, int y, int w, int h,
		int numCameras, Camera **c, Devsec *d)
	: Widget(parent, x, y, w, h)
{
	mApp = d;
	mCameras = c;
	mNumCameras = numCameras;
	CreateWindows();
	DownloadData();
}

DialogGlobal::~DialogGlobal()
{
}

int DialogGlobal::entering()
{
	return 0;
}

int DialogGlobal::leaving()
{
	return 0;
}

int DialogGlobal::OnDeletePressed()
{
	mApp->SetDeleteOldFiles(mDeleteOldFiles->GetToggle());
	return 0;
}

int DialogGlobal::OnAudiblePressed()
{
	Devsec::sAudibleRecord = mAudibleRecord->GetToggle();
	return 0;
}

int DialogGlobal::OnTimeChanged()
{
	unsigned int s;

	s = mMinutes->GetValue() * 60;
	s += mSeconds->GetValue();

	mApp->SetCycleTimeout(s);
	return 0;
}

int DialogGlobal::DownloadData()
{
	unsigned int a, b, c;
	if (mApp == NULL) return 0;

	mDeleteOldFiles->SetToggle(mApp->GetDeleteOldFiles());
	mAudibleRecord->SetToggle(Devsec::sAudibleRecord);

	a = mApp->GetCycleTimeout();
	b = a / 60;
	c = a - b * 60;
	mMinutes->SetValue(b);
	mSeconds->SetValue(c);
	return 0;
}

int DialogGlobal::CreateWindows()
{
	Widget *frame;
	Widget *label;

	SetColor(0xe600, 0xf700, 0xff00);

	mTitleImage = new Widget(this, 0, 0, mW, 30);
	mTitleImage->Show();
	mTitleImage->SetPixmap(Skin::GetSkinPixmap(CONFIG_HDR_GLOBAL));

	frame = new Widget(this, 8, 48, mW - 16, 96);
	frame->Show();
	frame->SetText("Cycle Timeout", 2, 12);
	frame->DrawFrame(1);
	frame->SetColor(0xe600, 0xf700, 0xff00);

	mMinutes = new SpinBox(frame, 32, 32, 64, 24);
	mMinutes->Show();
	mMinutes->SetValue(0);
	mMinutes->SetRange(0, 100);
	mMinutes->SetCallback((void*)this, NULL, TimeChangeCallback);
	mMinutes->SetToolTipText("Cycle timeout (minutes)");

	label = new Widget(frame, 112, 32, 64, 24);
	label->Show();
	label->DrawFrame(0);
	label->SetText("Minutes", 0, 24);
	label->SetColor(0xe600, 0xf700, 0xff00);

	mSeconds = new SpinBox(frame, 176, 32, 64, 24);
	mSeconds->Show();
	mSeconds->SetValue(5);
	mSeconds->SetRange(0, 59);
	mSeconds->SetCallback((void*)this, NULL, TimeChangeCallback);
	mSeconds->SetToolTipText("Cycle timeout (seconds)");

	label = new Widget(frame, 256, 32, 64, 24);
	label->Show();
	label->DrawFrame(0);
	label->SetText("Seconds", 0, 24);
	label->SetColor(0xe600, 0xf700, 0xff00);

	frame = new Widget(this, 8, 176, mW - 16, 96);
	frame->Show();
	frame->SetText("File Management", 2, 12);
	frame->DrawFrame(1);
	frame->SetColor(0xe600, 0xf700, 0xff00);

	mDeleteOldFiles = new CheckBox(frame, 32, 32);
	mDeleteOldFiles->Show();
	mDeleteOldFiles->SetCallback((void*)this, NULL, DeletedCallback);
	mDeleteOldFiles->SetToolTipText("Delete the oldest files when free space is limited");

	label = new Widget(frame, 56, 28, 128, 16);
	label->Show();
	label->DrawFrame(0);
	label->SetText("Delete Old Files", 0, 16);
	label->SetColor(0xe600, 0xf700, 0xff00);

	frame = new Widget(this, 8, 304, mW - 16, 96);
	frame->Show();
	frame->SetText("Audio Options", 2, 12);
	frame->DrawFrame(1);
	frame->SetColor(0xe600, 0xf700, 0xff00);

	mAudibleRecord = new CheckBox(frame, 32, 32);
	mAudibleRecord->Show();
	mAudibleRecord->SetCallback((void*)this, NULL, AudibleCallback);
	mAudibleRecord->SetToolTipText("Beep when recording starts");

	label = new Widget(frame, 56, 28, 256, 16);
	label->Show();
	label->DrawFrame(0);
	label->SetText("Audible beep when motion is detected", 0, 16);
	label->SetColor(0xe600, 0xf700, 0xff00);

}

void DialogGlobal::DeletedCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	DialogGlobal *d = (DialogGlobal*)d1;
	if (e->type != XEVENT) return;
	if (e->xev->type != ButtonPress) return;
	d->OnDeletePressed();
}

void DialogGlobal::AudibleCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	DialogGlobal *d = (DialogGlobal*)d1;
	if (e->type != XEVENT) return;
	if (e->xev->type != ButtonPress) return;
	d->OnAudiblePressed();
}

void DialogGlobal::TimeChangeCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	DialogGlobal *d = (DialogGlobal*)d1;
	if (e->type != DTKEVENT) return;
	if (e->dev->type != DTK_SPINBOX_CHANGED) return;
	d->OnTimeChanged();
}
