#include <stdlib.h>
#include <stdio.h>

#include "devsec.h"
#include "dtk_skin.h"
#include "dtk_checkbox.h"
#include "dtk_scrollview.h"
#include "dialog_motion.h"

DialogMotion::DialogMotion(Widget *parent, int x, int y, int w, int h,
		int numCameras, Camera **c, Devsec *d)
	: Widget(parent, x, y, w, h)
{
	mApp = d;
	mCameras = c;
	mNumCameras = numCameras;
	CreateWindows();
}

DialogMotion::~DialogMotion()
{
}

int DialogMotion::entering()
{
	return 0;
}

int DialogMotion::leaving()
{
	return 0;
}

int DialogMotion::CreateWindows()
{
	Widget *image;
	Widget *frame;
	Widget *label;
	ScrollView *scrollview;

	SetColor(0xe600, 0xf700, 0xff00);
	image = new Widget(this, 0, 0, mW, 30);
	image->Show();
	image->SetPixmap(Skin::GetSkinPixmap(CONFIG_HDR_MOTION));

	mMotionDetect = new CheckBox(this, 12, 64);
	mMotionDetect->Show();
	mMotionDetect->SetCallback((void*)this, (void*)-1, DetectCallback);
	mMotionDetect->SetToggle(mApp->GetGlobalDetect());
	mMotionDetect->SetToolTipText("Enable or disable recording on motion detection for ALL cameras");

	label = new Widget(this, 34, 64, 184, 14);
	label->Show();
	label->DrawFrame(0);
	label->SetColor(0xe600, 0xf700, 0xff00);
	label->SetText("Record on Motion Enabled", 0, 12);

	mMotionNotify = new CheckBox(this, 12, 96);
	mMotionNotify->Show();
	mMotionNotify->SetCallback((void*)this, (void*)-1, NotifyCallback);
	mMotionNotify->SetToggle(mApp->GetGlobalNotify());
	mMotionNotify->SetToolTipText("Enable or disable notification on motion detection for ALL cameras");

	label = new Widget(this, 34, 96, 192, 14);
	label->Show();
	label->DrawFrame(0);
	label->SetColor(0xe600, 0xf700, 0xff00);
	label->SetText("Notify on Motion Enabled", 0, 12);

	label = new Widget(this, 388, 56, 184, 14);
	label->Show();
	label->DrawFrame(0);
	label->SetColor(0xe600, 0xf700, 0xff00);
	label->SetText("Notification Phone Number", 0, 12);

	mPhoneNumber = new TextEntry(this, 388, 74, 184, 24);
	mPhoneNumber->Show();
	mPhoneNumber->SetBuffer(mApp->GetPhoneNumber());
	mPhoneNumber->SetCallback((void*)this, NULL, NumberCallback);
	mPhoneNumber->SetToolTipText("Set the phone number to call when motion is detected");

	frame = new Widget(this, 12, 122, 640, 410);
	frame->Show();
	frame->SetText("Cameras", 2, 12);
	frame->SetColor(0xe600, 0xf700, 0xff00);
	
	label = new Widget(frame, 0, 24, 256, 16);
	label->Show();
	label->SetText("Camera Name", 80, 12);
	label->SetColor(0xe600, 0xf700, 0xff00);

	label = new Widget(frame, 256, 24, 128, 16);
	label->Show();
	label->SetText("Notification", 18, 12);
	label->SetColor(0xe600, 0xf700, 0xff00);

	label = new Widget(frame, 384, 24, 128, 16);
	label->Show();
	label->SetText("Record on Motion", 8, 12);
	label->SetColor(0xe600, 0xf700, 0xff00);

	label = new Widget(frame, 512, 24, 128, 16);
	label->Show();
	label->SetText("Record Always", 10, 12);
	label->SetColor(0xe600, 0xf700, 0xff00);

	int i;

	scrollview = new ScrollView(frame, 1, 40, 638, 368);
	scrollview->Show();
	scrollview->SetColor(0xffff, 0xffff, 0xffff);

	mCameraRecOnMotion=(CheckBox**)malloc(mNumCameras * sizeof(CheckBox*));
	mCameraRecAlways = (CheckBox**)malloc(mNumCameras * sizeof(CheckBox*));

	for (i = 0; i < mNumCameras; i++) {
		Widget *container;
		CheckBox *check;
		unsigned int r, g, b;

		if (i % 2) r = g = b = 0xe000;
		else r = g = b = 0xffff;

		label = new Widget(scrollview, 1, 1+ 24 * i, 255, 23);
		label->Show();
		label->DrawFrame(0);
		label->SetText(mCameras[i]->GetCameraName(), 2, 14);
		label->SetColor(r, g, b);

		container = new Widget(scrollview, 257, 1+24 * i, 127, 23);
		container->Show();
		container->DrawFrame(0);
		container->SetColor(r, g, b);

		check = new CheckBox(container, 57, 5);
		check->Show();
		check->SetCallback((void*)this, (void*)i, NotifyCallback);
		check->SetToggle(mCameras[i]->GetNotification());

		container = new Widget(scrollview, 385, 1+24 * i, 127, 23);
		container->Show();
		container->DrawFrame(0);
		container->SetColor(r, g, b);
		
		mCameraRecOnMotion[i] = new CheckBox(container, 57, 5);
		mCameraRecOnMotion[i]->Show();
		mCameraRecOnMotion[i]->SetCallback((void*)this, (void*)i,
				DetectCallback);
		mCameraRecOnMotion[i]->SetToggle(mCameras[i]->GetMotionRecord());

		container = new Widget(scrollview, 513, 1+24 * i, 127, 23);
		container->Show();
		container->DrawFrame(0);
		container->SetColor(r, g, b);
		
		mCameraRecAlways[i] = new CheckBox(container, 57, 5);
		mCameraRecAlways[i]->Show();
		mCameraRecAlways[i]->SetCallback((void*)this,
				(void*)i, AlwaysCallback);
		mCameraRecAlways[i]->SetToggle(mCameras[i]->GetRecordAlways());

	}
}

int DialogMotion::OnMotionDetectPressed(int camera, int on)
{
	if (camera == -1) {
		mApp->SetGlobalDetect(on);
	} else {
		mCameras[camera]->SetMotionRecord(on);
		if (on) {
			mCameras[camera]->RecordAlways(0);
			mCameraRecAlways[camera]->SetToggle(0);
		}
	}
	return 0;
}

int DialogMotion::OnMotionNotifyPressed(int camera, int on)
{
	if (camera == -1) {
		mApp->SetGlobalNotify(on);
	} else {
		mCameras[camera]->SetNotification(on);
	}
	return 0;
}

int DialogMotion::OnRecordAlwaysPressed(int camera, int on)
{
	mCameras[camera]->RecordAlways(on);
	if (on) {
		mCameras[camera]->SetMotionRecord(0);
		mCameraRecOnMotion[camera]->SetToggle(0);
	}
	return 0;
}

int DialogMotion::OnPhoneNumberChanged()
{
	mApp->SetPhoneNumber(mPhoneNumber->GetBufferPtr());
	return 0;
}

void DialogMotion::DetectCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	int on = ((CheckBox*)o)->GetToggle();
	int camera = (int)d2;
	DialogMotion *d = (DialogMotion*)d1;
	if (e->type != XEVENT) return;
	if (e->xev->type != ButtonPress) return;
	d->OnMotionDetectPressed(camera, on);
}

void DialogMotion::NotifyCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	int on = ((CheckBox*)o)->GetToggle();
	int camera = (int)d2;
	DialogMotion *d = (DialogMotion*)d1;
	if (e->type != XEVENT) return;
	if (e->xev->type != ButtonPress) return;
	d->OnMotionNotifyPressed(camera, on);
}

void DialogMotion::AlwaysCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	int on = ((CheckBox*)o)->GetToggle();
	int camera = (int)d2;
	DialogMotion *d = (DialogMotion*)d1;
	if (e->type != XEVENT) return;
	if (e->xev->type != ButtonPress) return;
	d->OnRecordAlwaysPressed(camera, on);
}

void DialogMotion::NumberCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	DialogMotion *d = (DialogMotion*)d1;
	if (e->type != DTKEVENT) return;
	if (e->dev->type != DTK_TEXTENTRY_CHANGED) return;
	d->OnPhoneNumberChanged();
}
