#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "user.h"
#include "delete.h"
#include "modem.h"
#include "power.h"
#include "debug.h"

int usage(char *argv[])
{
	fprintf(stderr, "%s notify <camera_number> <phone_number>\n", argv[0]);
	fprintf(stderr, "%s delete <camera_name> <min_percent_free>\n",argv[0]);
	fprintf(stderr, "%s adduser <username> <password>\n", argv[0]);
	fprintf(stderr, "%s deluser <username>\n", argv[0]);
	fprintf(stderr, "%s checkuser <username> <password>\n", argv[0]);
	fprintf(stderr, "%s userlist\n", argv[0]);
	fprintf(stderr, "%s powerdown\n", argv[0]);
	return 0;
}

char *read_file(char *filename, unsigned long *size)
{
	char *rv = NULL;
	int i;
	unsigned long filesize;
	FILE *f;

	ENTER;
	if (filename == NULL) return NULL;

	f = fopen(filename, "r");
	if (f == NULL) return NULL;

	fseek(f, 0, SEEK_END);
	filesize = ftell(f);
	fseek(f, 0, SEEK_SET);

	if (filesize <= 0) goto fin;

	rv = malloc(filesize);
	if (rv == NULL) goto fin;

	fread(rv, filesize, 1, f);
fin:
	fclose(f);
	if (size != NULL) *size = filesize;
	LEAVE;
	return rv;
}

int do_notify(int camera, char *phone_number)
{
	char *audio_buffer = NULL;
	unsigned long buffer_size;
	char fname[256];

	ENTER;
	modem_t *modem = NULL;

	if (phone_number == NULL) return -1;

	modem = modem_new();
	if (modem == NULL) return -1;

	snprintf(fname, 256, "/etc/devsec/messages/camera%d.ulaw", camera);
	audio_buffer = read_file(fname, &buffer_size);
	if (audio_buffer == NULL) {
		fprintf(stderr, "Could not find %s\n", fname);
		exit(1);
	}

	fprintf(stderr, "audio_buffer 0x%x, buffer_size %lu\n", audio_buffer, buffer_size);

	modem_set_number(modem, phone_number);
	modem_set_audio(modem, audio_buffer, buffer_size);
	modem_send_message(modem);
	LEAVE;
	return 0;
}

int main(int argc, char *argv[])
{
	ENTER;

	if (argc < 2) {
		usage(argv);
		exit(0);
	}
	if (!strncmp(argv[1], "notify", 6)) {
		if (argc < 3) {
			usage(argv);
			exit(0);
		}
		{
			int number = atoi(argv[2]);
			return do_notify(number, argv[3]);
		}
	}
	if (!strncmp(argv[1], "delete", 6)) {
		if (argc < 4) {
			usage(argv);
			exit(0);
		}
		return do_delete(argv[2], atoi(argv[3]));
	}
	if (!strncmp(argv[1], "adduser", 7)) {
		if (argc < 4) {
			usage(argv);
			exit(0);
		}
		return do_adduser(argv[2], argv[3]);
	}
	if (!strncmp(argv[1], "deluser", 7)) {
		if (argc < 3) {
			usage(argv);
			exit(0);
		}
		return do_deluser(argv[2]);
	}
	if (!strncmp(argv[1], "modifyuser", 10)) {
		if (argc < 4) {
			usage(argv);
			exit(0);
		}
		return do_modifyuser(argv[2], argv[3]);
	}
	if (!strncmp(argv[1], "checkuser", 9)) {
		if (argc < 4) {
			usage(argv);
			exit(0);
		}
		return do_checkuser(argv[2], argv[3]);
	}
	if (!strncmp(argv[1], "userlist", 8)) {
		return do_userlist();
	}
	if (!strncmp(argv[1], "powerdown", 10)) {
		return do_powerdown();
	}

	LEAVE;

	return 0;
}
