#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "user.h"

unsigned long checksum(char *passwd, int length)
{
	char buf[1024];
	unsigned long rv = 0;
	unsigned long current = 0;
	int step = 0;

	if (passwd == NULL) return 0;
	
	if (length > 256) length = 256;

	memset(buf, 0, 1024);

	memcpy(buf, passwd, length);

	while (length % sizeof(unsigned long)) {
		strncat(buf, " ", 256);
		length++;
	}

	for (step = 0; step < length / sizeof(unsigned long); step++) {
		current = ((unsigned long*)buf)[step];
		rv += current;
	}
	return current;
}

int add_entry(char *name, unsigned long cs)
{
	FILE *f;

	f = fopen(PASSWD, "a");
	if (f == NULL) return -1;

	fprintf(f, "%s:%lu\n", name, cs);
	fclose(f);
	return 0;
}

int del_entry(char *name)
{
	FILE *f1, *f2;
	char buf[512];
	int entries = 0;
	
	rename(PASSWD, PASSWD".old");

	f1 = fopen(PASSWD".old", "r");
	if (f1 == NULL) return -1;

	f2 = fopen(PASSWD, "w");

	if (f2 == NULL) {
		rename(PASSWD".old", PASSWD);
		fclose(f1);
		return -1;
	}

	while (fgets(buf, 512, f1) != NULL) {
		if (!strncmp(buf, name, strlen(name)))
			if (buf[strlen(name)] == ':') continue;
		entries++;
		fprintf(f2, "%s", buf);
	}
	fclose(f1);
	fclose(f2);
	unlink(PASSWD".old");
	if (!entries) unlink(PASSWD);
	return 0;
}

int get_entry(char *name, unsigned long *cs)
{
	FILE *f;
	char buf[512];
	unsigned long rv;

	if (cs == NULL) return -1;

	f = fopen(PASSWD, "r");

	if (f == NULL) return -1;

	while (fgets(buf, 512, f) != NULL) {
		if (!strncmp(buf, name, strlen(name))) {
			char *p = strchr(buf, ':');
			if (buf[strlen(name)] != ':') continue;
			if (p == NULL) continue;
			p++;
			sscanf(p, "%lu", &rv);
			*cs = rv;
			goto found;
		}
	}
	fclose(f);
	return 1;
found:
	fclose(f);
	return 0;
}

/* do_adduser
 * returns -1 if user exists
 * returns 0 if added
 * returns 1 if error
 */
int do_adduser(char *name, char *password)
{
	unsigned long cs;
	int rv;

	cs = checksum(password, strlen(password));

	rv = do_checkuser(name, password);
	if (rv != -1) return -1;

	return (!add_entry(name, cs) ? 0 : 1);
}

int do_deluser(char *name)
{
	if (do_checkuser(name, "") == -1) return -1;

	return del_entry(name);
}

int do_modifyuser(char *name, char *password)
{
	do_deluser(name);
	return do_adduser(name, password);
}

/* do_checkuser
 * returns -1 if user does not exist
 *         0 if user exists and password matches
 *         1 if user exists but wrong password
 */
int do_checkuser(char *name, char *password)
{
	unsigned long cs1, cs2;

	cs2 = 0;
	cs1 = checksum(password, strlen(password));
	if (get_entry(name, &cs2) != 0) return -1;
	if (cs1 != cs2) return 1;
	return 0;
}

int do_userlist()
{
	FILE *f;
	char buf[512];

	f = fopen(PASSWD, "r");
	if (f == NULL) return 0;

	while (fgets(buf, 512, f) != NULL) {
		char *p = strchr(buf, ':');
		if (p == NULL) continue;
		*p = '\0';
		fprintf(stdout, "%s\n", buf);
	}
	fclose(f);
	return 0;
}

