#ifndef __GRAPHICS_H
#define __GRAPHICS_H

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/extensions/XShm.h>

#include <pthread.h>

#define BUTTON_PRESS 1000
#define BUTTON_RELEASE 2000
#define KEY_PRESS 3000
#define KEY_RELEASE 4000

#ifndef FRAME_HEADER
#define FRAME_HEADER

struct frame_header {
	int frame_no;
	int format;
	int width;
	int height;
	unsigned long length;
};

struct ts_frame {
	struct frame_header fh;
	unsigned char *data;
	unsigned char *rgb;
};
#endif

class PreviewWindow {
public:
	PreviewWindow();
	virtual ~PreviewWindow();
	virtual int open_display(char *d);
	virtual int close_display();
	virtual int create_window(Window p);
	virtual int set_parent(Window p);
	virtual int set_size(int width, int height);
	virtual int show();
	virtual int hide();
	virtual int sync();
	virtual int get_pending();
	virtual int show_next_frame(struct ts_frame *f);

protected:
	virtual int ScaleImage(struct ts_frame *f,
			int srcW, int srcH, int dstW, int dstH);
	virtual int DestroyImage();
	virtual int CreateImage(int w, int h);
	Display *display;
	Window window;
	Window parent;
	GC gc;
	unsigned int width;
	unsigned int height;
	int mGotShm;
	XImage *mImage;
	XShmSegmentInfo mShmInfo;
};

class Text {
public:
	static int DrawText(char *yuvbuf, int x, int y, int w, char *text = NULL);
	static int SetText(char *text);
protected:
	static pthread_mutex_t mMutex;
	static char *mText;
};

#endif
