#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "motion.h"

#define video_width 384
#define video_height 288
#define video_area video_width*video_height

Motion::Motion()
{
	background = (rgb24_t*)malloc(video_area * sizeof(rgb24_t));
	diff = (unsigned char*)malloc(video_area * sizeof(unsigned char));
	diff2 = (unsigned char*)malloc(video_area * sizeof(unsigned char));
	memset(diff2, 0, video_area * sizeof(unsigned char));


	boxX1 = video_width;
	boxY1 = video_height;
	boxX2 = 0;
	boxY2 = 0;
}

Motion::~Motion()
{
	free(background);
	free(diff);
	free(diff2);
}

void Motion::image_set_threshold_y(int threshold)
{
	y_threshold = threshold * 7;
}

void Motion::image_bgset_y(rgb24_t *src)
{
	int i;
	int R, G, B;
	rgb24_t *p;
	short *q;

	p = src;
	q = (short *)background;
	for (i = 0; i < video_area; i++) {
		R = ((*p)&0xff0000)>>(16-1);
		G = ((*p)&0xff00)>>(8-2);
		B = (*p)&0xff;
		*q = (short)(R + G + B);
		p++;
	}
}

unsigned char *Motion::image_bgsubtract_y(rgb24_t *src)
{
	int i;
	int R, G, B;
	rgb24_t *p;
	short *q;
	unsigned char *r;
	int v;

	p = src;
	q = (short*)background;
	r = diff;
	for (i = 0; i < video_area; i++) {
		R = ((*p)&0xff0000)>>(16-1);
		G = ((*p)&0xff00)>>(8-2);
		B = (*p)&0xff;
		v = (R + G + B) - (int)(*q);
		*r = ((v + y_threshold)>>24) | ((y_threshold - v)>>24);

		p++;
		q++;
		r++;
	}
	return diff;
}

unsigned char *Motion::image_bgsubtract_update_y(rgb24_t *src)
{
	int i;
	int R, G, B;
	rgb24_t *p;
	short *q;
	unsigned char *r;
	int v;

	p = src;
	q = (short*)background;
	r = diff;
	for (i = 0; i < video_area; i++) {
		R = ((*p)&0xff0000)>>(16-1);
		G = ((*p)&0xff00)>>(8-2);
		B = (*p)&0xff;
		v = (R + G + B) - (int)(*q);
		*q = (short)(R + G + B);
		*r = ((v + y_threshold)>>24) | ((y_threshold - v)>>24);

		p++;
		q++;
		r++;
	}

	return diff;
}

void Motion::image_set_threshold_RGB(int r, int g, int b)
{
	unsigned char R, G, B;

	R = G = B = 0xff;
	R = R<<r;
	G = G<<g;
	B = B<<b;
	rgb_threshold = (rgb24_t)(R<<16 | G<<8 | B);
}

void Motion::image_bgset_RGB(rgb24_t *src)
{
	int i;
	rgb24_t *p;

	p = background;
	for (i = 0; i < video_area; i++) {
		*p++ = (*src++) & 0xfefefe;
	}
}

unsigned char *Motion::image_bgsubtract_RGB(rgb24_t *src)
{
	int i;
	rgb24_t *p, *q;
	unsigned a, b;
	unsigned char *r;

	p = src;
	q = background;
	r = diff;
	for(i=0; i<video_area; i++) {
		a = (*p++)|0x1010100;
		b = *q++;
		a = a - b;
		b = a & 0x1010100;
		b = b - (b>>8);
		b = b ^ 0xffffff;
		a = a ^ b;
		a = a & rgb_threshold;
		*r++ = (0 - a)>>24;
	}
	return diff;
}

unsigned char *Motion::image_bgsubtract_update_RGB(rgb24_t *src)
{
	int i;
	rgb24_t *p, *q;
	unsigned a, b;
	unsigned char *r;

	p = src;
	q = background;
	r = diff;
	for(i=0; i<video_area; i++) {
		a = *p|0x1010100;
		b = *q&0xfefefe;
		*q++ = *p++;
		a = a - b;
		b = a & 0x1010100;
		b = b - (b>>8);
		b = b ^ 0xffffff;
		a = a ^ b;
		a = a & rgb_threshold;
		*r++ = (0 - a)>>24;
	}
	return diff;
}

unsigned char *Motion::image_diff_filter(unsigned char *diff)
{
	int x, y;
	unsigned char *src, *dest;
	unsigned int count;
	unsigned int sum1, sum2, sum3;
	const int width = video_width;

	src = diff;
	dest = diff2 + width +1;
	for(y=1; y<video_height-1; y++) {
		sum1 = src[0] + src[width] + src[width*2];
		sum2 = src[1] + src[width+1] + src[width*2+1];
		src += 2;
		for(x=1; x<width-1; x++) {
			sum3 = src[0] + src[width] + src[width*2];
			count = sum1 + sum2 + sum3;
			sum1 = sum2;
			sum2 = sum3;
			*dest++ = (0xff*3 - count)>>24;
			src++;
		}dest += 2;
	}

	return diff2;
}

unsigned char *Motion::image_y_over(rgb24_t *src)
{
	int i;
	int R, G, B, v;
	unsigned char *p = diff;

	for (i = video_area; i > 0; i--) {
		R = ((*src)&0xff0000)>>(16-1);
		G = ((*src)&0xff00)>>(8-2);
		B = (*src)&0xff;
		v = y_threshold - (R + G + B);
		*p = (unsigned char)(v>>24);
		src++;
		p++;
	}
	return diff;
}

unsigned char *Motion::image_y_under(rgb24_t *src)
{
	int i;
	int R, G, B, v;
	unsigned char *p = diff;

	for(i = video_area; i>0; i--) {
		R = ((*src)&0xff0000)>>(16-1);
		G = ((*src)&0xff00)>>(8-2);
		B = (*src)&0xff;
		v = (R + G + B) - y_threshold;
		*p = (unsigned char)(v>>24);
		src++;
		p++;
	}
	return diff;
}

void Motion::image_hflip(rgb24_t *src, rgb24_t *dest, int w, int h)
{
	int x, y;

	src += w - 1;
	for (y = 0; y < h; y++) {
		for (x = 0; x < w; x++) {
			*dest++ = *src--;
		}
		src += w * 2;
	}
}

int Motion::motiondetect(rgb24_t *src)
{
	int rv;
	unsigned char *diff;
	int width;
	//int *p, *q;
	int x, y, h;
	int map_w, map_h;

	if (!bgIsSet) {
		setBackground(src);
	}

	diff = image_bgsubtract_update_y(src);
	width = video_width;
	map_w = video_width / 2 + 1;
	map_h = video_height / 2 + 1;

	rv = 0;
	diff += width + 2;
	for (y = map_h - 2; y > 0; y--) {
		for (x = map_w - 2; x > 0; x--) {
			h = (int)*diff + (int)*(diff+1) + (int)*(diff+width)
				+ (int)*(diff+width+1);
			if (h > 0) {
				add_point(x, y);
				rv = 1;
			}
			diff += 2;
		}
		diff += width + 2;
	}
	return rv;
}

int Motion::setBackground(rgb24_t *src)
{
	image_bgset_y(src);
	bgIsSet = 1;
	return 0;
}


int Motion::set_sensitivity(unsigned int sensitivity)
{
        mSensitivity = sensitivity;
        return 0;
}

int Motion::set_bounds(unsigned long x1, unsigned long y1,
                unsigned long x2, unsigned long y2)
{
        mX1 = x1;
        mY1 = y1;
        mX2 = x2;       
        mY2 = y2;
        return 0;
}

int Motion::set_showmotion(unsigned char showmotion)
{
        mShowMotion = showmotion;
        return 0;
}

int Motion::set_showbounds(unsigned char showbounds)
{
        mShowBounds = showbounds;
        return 0;
}

static void convert_RGB24toRGB32
(unsigned char *src, rgb24_t *dest, int width, int height)
{
        int i, length;

        length = width * height;
        for(i=0; i<length; i++) {
                *dest++ = *(unsigned int *)src & 0xffffff;
                src += 3;
        }
}

#define COMMON_CONVERTER_FROMRGB32(_convert_block_) \
(rgb24_t *src, int src_width, int src_height,\
 unsigned char *dest, int dest_width, int dest_height)\
{\
        int x, y;\
        int sx, sy;\
        int tx, ty;\
        rgb24_t *p, v;\
\
        if(src_width == dest_width && src_height == dest_height) {\
                for(x=0; x<src_width*src_height; x++) {\
                        v = *src++;\
                        _convert_block_ \
                }\
        } else {\
                tx = src_width * 256 / dest_width;\
                ty = src_height * 256 / dest_height;\
                sy = 0;\
                for(y=0; y<dest_height; y++) {\
                        p = src + (sy>>8) * src_width;\
                        sx = 0;\
                        for(x=0; x<dest_width; x++) {\
                                v = p[sx>>8];\
                                _convert_block_ \
                                sx += tx;\
                        }\
                        sy += ty;\
                }\
        }\
}

static void convert_RGB32toRGB24
COMMON_CONVERTER_FROMRGB32(dest[0] = v & 0xff; dest[1] = (v>>8) & 0xff; dest[2] = (v>>16) & 0xff; dest += 3;)

int Motion::detect_motion(unsigned char *buffer)
{
	static rgb24_t *b = NULL;
	int rv;

	if (b == NULL) {
		b = (rgb24_t*)malloc(384*288*sizeof(rgb24_t));
	}
	convert_RGB24toRGB32(buffer, b, 384, 288);
	rv = motiondetect(b);
	convert_RGB32toRGB24(b, 384, 288, buffer, 384, 288);

	if (rv) draw_box(buffer, boxX1, boxY1, boxX2, boxY2);
	return rv;
}

int Motion::add_point(unsigned int x, unsigned int y)
{
	if (boxX1 > x) boxX1 = x;
	if (boxY1 > y) boxY1 = y;
	if (boxX2 < x) boxX2 = x;
	if (boxY2 < y) boxY2 = y;
	return 0;
}

int Motion::draw_box(unsigned char *buffer,
		unsigned int x1, unsigned int y1,
		unsigned int x2, unsigned int y2)
{
	unsigned int i;

	for (i = x1; i < x2; i++) {
		buffer[(i + y1 * 384) * 3] = 0xff;
		buffer[(i + y1 * 384) * 3 + 1] = 0xff;
                buffer[(i + y1 * 384) * 3 + 2] = 0xff;
                buffer[(i + y2 * 384) * 3] = 0xff;
                buffer[(i + y2 * 384) * 3 + 1] = 0xff;
                buffer[(i + y2 * 384) * 3 + 2] = 0xff;
        }

        for (i = y1; i < y2; i++) {
                buffer[(x1 + i * 384) * 3] = 0xff;
                buffer[(x1 + i * 384) * 3 + 1] = 0xff;
                buffer[(x1 + i * 384) * 3 + 2] = 0xff;
                buffer[(x2 + i * 384) * 3] = 0xff;
                buffer[(x2 + i * 384) * 3 + 1] = 0xff;
                buffer[(x2 + i * 384) * 3 + 2] = 0xff;
        }

	return 0;
}
