#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "socket.h"

Socket::Socket(char *sockname)
{
	char *helo = "HELO\n";
	struct sockaddr_un server;

	if (sockname == NULL) {
		fprintf(stderr, "cannot have a null socket\n");
		exit (1);
	}

	mSockName = strdup(sockname);

	mSockFD = socket(AF_UNIX, SOCK_STREAM, 0);
	if (mSockFD < 0) {
		fprintf(stderr, "Error opening socket\n");
		exit(1);
	}

	server.sun_family = AF_UNIX;
	strcpy(server.sun_path, sockname);

	if (connect(mSockFD, (struct sockaddr*)&server,
				sizeof(struct sockaddr_un))) {
		close(mSockFD);
		mSockFD = 0;
		fprintf(stderr, "Error connecting to socket\n");
		exit(1);
	}
	write(helo, strlen(helo));
}

Socket::~Socket()
{
	if (mSockName) free((void*)mSockName);
	mSockName = NULL;
	if (mSockFD) close(mSockFD);
	mSockFD = 0;
}

int Socket::read(char *buf, unsigned int length)
{
	char c;
	int i = 0;

	if (buf == NULL) return -1;
	if (mSockFD <= 0) return -1;

	do {
		int j = 0;
		if ((j = ::read(mSockFD, &c, 1) <= 0)) {
			fprintf(stderr, "tsrep: Socket: error: %s\n",
					strerror(errno));
			exit(-1);
		}


		buf[i] = c;
		if ((unsigned int)i >= length) return 0;
		i++;
	} while ((c != '\n') && (c != '\0'));
	buf[i-1] = '\0';
	return 0;
}

int Socket::write(char *buf, unsigned int length)
{
	if (buf == NULL) return -1;
	if (mSockFD <= 0) return -1;

	::write(mSockFD, buf, length);
	return 0;
}

int Socket::pending()
{
	fd_set fds;
	struct timeval tv;
	int srv;
	int rv = 0;

	FD_ZERO(&fds);
	FD_SET(mSockFD, &fds);

	tv.tv_sec = 0;
	tv.tv_usec = 1;

	srv = select(mSockFD + 1, &fds, NULL, NULL, &tv);

	if ((srv > 0) && FD_ISSET(mSockFD, &fds)) {
			rv = 1;
	} else if (srv == 0) {
		rv = 0;
	} else {
		fprintf(stderr, "tsrep: Socket: error: %s\n", strerror(errno));
		exit(1);
	}
	return rv;
}
