#include <fcntl.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>

#ifdef USE_WATCHDOG_FD
#define FIFO "/tmp/watchdog3"

static int initial_timer = 60;
#endif

void write_log(char *msg)
{
	struct tm tm;
	time_t t;
	FILE *f;

	system("mkdir -p /streams/log");
	f = fopen("/streams/log/error.log", "a");
	if (f == NULL) {
		fprintf(stderr, "cannot open error log\n");
		return;
	}
	t = time(NULL);
	localtime_r(&t, &tm);

	fprintf(stderr, "%02d:%02d:%02d on %02d/%02d/%04d %s\n",
			tm.tm_hour, tm.tm_min, tm.tm_sec,
			tm.tm_mday, tm.tm_mon + 1, tm.tm_year + 1900,
			msg);
	fprintf(f, "%02d:%02d:%02d on %02d/%02d/%04d %s\n",
			tm.tm_hour, tm.tm_min, tm.tm_sec,
			tm.tm_mday, tm.tm_mon + 1, tm.tm_year + 1900,
			msg);
	fclose(f);
}

int start_devsec(char *arg)
{
	char *argv[3];
	int argc = 0;
	pid_t pid;

	write_log("starting devsec");
	system("killall -KILL devsec tsrep3 >> /streams/log/error.log");
	pid = fork();

#ifdef USE_WATCHDOG_FD
	initial_timer = 60;
#endif
	if (pid == 0) {
		argv[0] = "devsec";
		argv[1] = arg;
		argv[2] = NULL;
		execvp(argv[0], argv);
		fprintf(stderr, "error executing devsec\n");
		write_log("error executing devsec");
		exit(-1);
	}
	return pid;
}

#ifdef USE_WATCHDOG_FD
int create_fifo()
{
	int fd;
	mkfifo(FIFO, 0666);
	fd = open(FIFO, O_RDONLY | O_NONBLOCK);
	return fd;
}

int watch_fifo(int fd)
{
	int rv;
	fd_set fds;
	struct timeval tv;
	int s;
	char buf[256];

	FD_ZERO(&fds);
	FD_SET(fd, &fds);

	tv.tv_sec = initial_timer;
	tv.tv_usec = 0;

	s = select(fd + 1, &fds, NULL, NULL, &tv);

	rv = -1;
	if (s == 0) {
		rv = 0;
	} else if ((s > 0) && (FD_ISSET(fd, &fds))) {
		rv = read(fd, &buf[0], 256);
		if (rv == 0) rv = -1;
		else {
			rv = 1;
			initial_timer = 30;
		}
	} else rv = -1;
	return rv;
}

int main(int argc, char *argv[])
{
	int fd;
	int consumer = 0;

	if ((argc > 1) && !strncmp(argv[1], "--consumer", 10)) consumer = 1;

	fd = create_fifo();
	signal(SIGCHLD, SIG_IGN);
	start_devsec((consumer ? "--consumer" : NULL));
	for(;;) {
		switch (watch_fifo(fd)) {
			case 0:
				write_log("watchdog: timeout");
				fprintf(stderr, "timeout %d\n", time(NULL));
				start_devsec(NULL);
				break;
			case 1:
				//fprintf(stderr, "good\n");
				break;
			case -1:
				//fprintf(stderr, "fifo closed\n");
				close(fd);
				fd = open(FIFO, O_RDONLY | O_NONBLOCK);
				break;
		}
	}
	return 0;
}
#else
int main(int argc, char *argv[])
{
	int status;
	int consumer = 0;

	if ((argc > 1) && !strncmp(argv[1], "--consumer", 10)) consumer = 1;

	while (1) {
		pid_t pid = start_devsec((consumer ? "--consumer" : NULL));
		wait(&status);
	}
	return 0;
}
#endif
