#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <string.h>

typedef struct {
	char *variable;
	char *value;
} CValue;

#define ENVIRONMENT 1024

static CValue *sValues = NULL;
static int sNumValues = 0;

static int ReadLine(int fd, char *buffer, int length)
{
	char b;
	int i = 0;

	if (buffer == NULL) return -1;
	do {
		int j = 0;
		if ((j = read(fd, &b, 1) <= 0)) {
			return -1;
		}
		buffer[i] = b;
		i++;
		if (i >= length) return 0;
	} while ((b != '\n') && (b != '\0'));
	buffer[i-1] = '\0';
	return 0;
}

int GetValue(char *section, char *variable, char *buf[])
{
	int i = 0;
	char tmp[1024];
	if (sValues == NULL) {
		sValues = (CValue*)malloc(ENVIRONMENT * sizeof(CValue*));
		memset(sValues, 0, ENVIRONMENT * sizeof(CValue*));
		sNumValues = 0;
	}
	snprintf(tmp, 1024, "%s_%s", section, variable);
	for (i = 0; i < sNumValues; i++) {
		if (!strcasecmp(sValues[i].variable, tmp)) {
			memcpy(*buf, sValues[i].value,
					strlen(sValues[i].value)+1);
			return i;
		}
	}
	return -1;
}

int ParseConfig(char *section, char *buf)
{
	char *variable;
	char *value;
	char *c;
	char *tmp[1024];

	if (sNumValues == ENVIRONMENT) {
		return -1;
	}

	variable = buf;
	c = strchr(buf, '=');
	if (c == NULL) return -1;
	*c = '\0';
	c++;
	value = c;
	return SetCharValue(section, variable, value);
}

int ReadConfig(char *section, char *filename)
{
	FILE *f;
	char buf[1024];

	f = fopen(filename, "r");
	if (f == NULL) return -1;
	while (ReadLine(fileno(f), buf, 1023) >= 0) {
		ParseConfig(section, buf);
	}
	fclose(f);
	return 0;
}

int WriteConfig(char *section, char *filename)
{
	FILE *f;
	int i;
	int section_len = strlen(section) + 1;

	if (sValues == NULL) {
		fprintf(stderr, "nothing to save\n");
		return 0;
	}

	f = fopen(filename, "w");
	if (f == NULL) {
		fprintf(stderr, "cannot save %s\n", filename);
		return 0;
	}

	for (i = 0; i < sNumValues; i++) {
		if (!strncasecmp(section, sValues[i].variable,
					strlen(section))) {
			if (sValues[i].variable[section_len-1] != '_') continue;
			fprintf(f, "%s=%s\n",
				(char*)(sValues[i].variable+section_len),
				sValues[i].value);
		}
	}
	fclose(f);
	return 0;
}

int GetIntValue(char *section, char *variable, int def)
{
        char buf[1024];
        char *p = &buf[0];
        memset(buf, 0, 1024);
        if (GetValue(section, variable, &p) == -1) {
                return def;
        }
        return atoi(buf);
}

int GetLongValue(char *section, char *variable, long def)
{
        char buf[1024];
        char *p = &buf[0];
        memset(buf, 0, 1024);
        if (GetValue(section, variable, &p) == -1) {
                return def;
        }
        return atol(buf);
}

char *GetCharValue(char *section, char *variable, char *def)
{
        char buf[1024];
        char *p = &buf[0];
        memset(buf, 0, 1024);
        if (GetValue(section, variable, &p) == -1) {
                return ((def == NULL) ? NULL : strdup(def));
        }
        return strdup(buf);
}

int SetIntValue(char *section, char *variable, int value)
{
        char buf[1024];
        snprintf(buf, 1024, "%d", value);
        return SetCharValue(section, variable, buf);
}

int SetLongValue(char *section, char *variable, long value)
{
        char buf[1024];
        snprintf(buf, 1024, "%ld", value);
        return SetCharValue(section, variable, buf);
}

int SetCharValue(char *section, char *variable, char *value)
{
        char buf[1024];
        char *p = &buf[0];
        int index;

        if ((section == NULL) || (variable == NULL) || (value == NULL))
                return 0;

        index = GetValue(section, variable, &p);
        if (index == -1) index = sNumValues++;

        if (sNumValues > ENVIRONMENT) {
                sNumValues--;
                fprintf(stderr, "Environment overflow error\n");
                return -1;
        }
        snprintf(buf, 1024, "%s_%s", section, variable);
        if (sValues[index].variable == NULL) {
                sValues[index].variable = strdup(buf);
        }
        if (sValues[index].value != NULL) {
                free((void*)sValues[index].value);
        }
        sValues[index].value = strdup(value);
	return 0;
}

int FreeEnvironment()
{
}


