#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/vfs.h>
#include <sys/types.h>
#include <template.h>
#include <cgi.h>
#include "config.h"

extern int ReadConfig(char *section, char *filename);
extern int WriteConfig(char *section, char *filename);
extern int GetIntValue(char *section, char *variable, int def);
extern int GetLongValue(char *section, char *variable, long def);
extern char *GetCharValue(char *section, char *variable, char *def);
extern int SetIntValue(char *section, char *variable, int value);
extern int SetLongValue(char *section, char *variable, long value);
extern int SetCharValue(char *section, char *variable, char *value);
extern int FreeEnvironment();

int get_resources(struct tpl_engine *engine)
{
	double hdd = 0.0;
	double mem = 0.0;
	struct statfs buf;
	FILE *f;
	int mT, mF, mB, mC;
	char line[256];

	mT = mF = mB = mC = 0;
	statfs("/streams/images", &buf);
	hdd = 100.0 - (((double)(buf.f_bfree)*100.0)/(double)buf.f_blocks);
	f = fopen("/proc/meminfo", "r");
	if (f == NULL) return 0;
	while (fgets(line, 256, f) > 0) {
		int i;
		if (sscanf(line, "MemTotal: %d kB", &i)) mT = i;
		if (sscanf(line, "MemFree: %d kB", &i)) mF = i;
		if (sscanf(line, "Buffers: %d kB", &i)) mB = i;
		if (sscanf(line, "Cached: %d kB", &i)) mC = i;
	}
	fclose(f);
	mem = (double)(mT - (mF + mB + mC)) * 100.0 / mT;
	snprintf(line, 256, "%d%%", (int)mem);
	tpl_element_set(engine, "mem", line);
	snprintf(line, 256, "%d%%", (int)hdd);
	tpl_element_set(engine, "hdd", line);
}

int start_remote_stream(int camera_no, char *remote_address, int enable)
{
	char buf[256];
	if (remote_address == NULL) return 0;

	if (enable == 0) {
		snprintf(buf, 256, "echo \"CAMERA %d OFF\n\" > /tmp/devsec_remote_control_pipe", camera_no);
	} else {
		snprintf(buf, 256, "echo \"CAMERA %d %s 0\n\" > /tmp/devsec_remote_control_pipe",
				camera_no, remote_address);
	}
	system(buf);
	return 0;
}

int load_camera_entries(struct tpl_engine *engine, int live_camera)
{
	char buf[64];
	char *live_camera_name = NULL;
	int cameras = 0;
	int i;

	ReadConfig("global", DTK_CONFDIR "/devsec/global.conf");
	cameras = GetIntValue("global", "Cameras", 1);
	snprintf(buf, 64, "%d", cameras);
	tpl_element_set(engine, "cameras", buf);
	for (i = 0; i < cameras; i++) {
		char *camera_name;
		char section[32];
		snprintf(buf, 64, DTK_CONFDIR "/devsec/mp4live/camera%d.conf", i);
		snprintf(section, 32, "camera%d", i);
		ReadConfig(section, buf);
		snprintf(buf, 64, "Video%d", i);
		camera_name = GetCharValue(section, "CameraName", buf);
		snprintf(buf, 64, "%d", i);
		tpl_element_set(engine, "camera_no", buf);
		tpl_element_set(engine, "camera_name", camera_name);
		tpl_parse(engine, "camera_entry", "camera_entry", 1);
		tpl_parse(engine, "camera_name_list", "camera_name_list", 1);
		if (live_camera == i) live_camera_name = strdup(camera_name);
		free((void*)camera_name);
	}
#if 0
	snprintf(buf, 64, "%d", live_camera);
	tpl_element_set(engine, "live_camera_no", buf);
	tpl_element_set(engine, "live_camera_name", live_camera_name);
	free((void*)live_camera_name);
#endif
	return 0;
}

int main(int argc, char *argv[])
{
	struct tpl_engine *engine;
	char *result;
	int live_camera = 0;
	char *remote_address = NULL;
	int remote_enable=0;

	cgi_init();
	cgi_session_start();
	cgi_process_form();
	cgi_init_headers();
	
	engine = tpl_engine_new();
	tpl_file_load(engine, "templates/index.tpl");

	if (cgi_param("live_camera"))
		live_camera = atoi(cgi_param("live_camera"));
	if (cgi_param("remote_address"))
		remote_address = cgi_param("remote_address");
	if (cgi_param("remote_enable"))
		remote_enable = atoi(cgi_param("remote_enable"));
	else
		remote_enable = 0;

	start_remote_stream(live_camera, remote_address, remote_enable);
	if (remote_address)
		tpl_element_set(engine, "remote_address", remote_address);
	load_camera_entries(engine, live_camera);
	get_resources(engine);
	tpl_parse(engine, "index", "main", 0);
	result = tpl_element_get(engine, "main");
	if (result == NULL) {
		printf("error in template");
	}
	printf("%s", result);
	tpl_engine_delete(engine);

	cgi_end();
	return 0;
}
