#ifndef __BASEWIDGET_H
#define __BASEWIDGET_H

#include "dtk_object.h"

#include <X11/X.h>
#include <X11/Xlib.h>

#define XK_MISCELLANY
#define XK_LATIN1
#include <X11/keysymdef.h>

#define OPAQUE 0xffffffff
#define OPACITY "_NET_WM_WINDOW_OPACITY"

class BaseWidget : public Object {
public:
	BaseWidget() {};
	BaseWidget(BaseWidget *parent, int x, int y, int w, int h);
	virtual ~BaseWidget();
	virtual int Destroy();

	virtual int EventHandler(TEvent *ev);
	virtual int SetEventMask(long mask);

	virtual int Enable(int flag);
	virtual int SetTitle(char *title);
	virtual int SetText(char *text, int x, int y);
	virtual int SetTextSize(int size);
	virtual inline char *GetText() { return mText; };
	virtual int SetOpacity(float opacity);
	virtual int SetPixmap(Pixmap p);
	virtual inline int SetDoubleBuffer(int on) { mDoubleBuffer = on; };
	virtual int SetColor(unsigned int r, unsigned int g, unsigned int b);
	virtual int SetColor(unsigned long background);
	virtual int SetForegroundColor(unsigned int r,
			unsigned int g, unsigned int b);
	virtual int SetForegroundColor(unsigned long background);
	virtual int SetResizable(int flag);
	virtual int Draw();
	virtual int Show();
	virtual int Hide();
	virtual int IsVisible();
	virtual inline int GetX() { return mX; };
	virtual inline int GetY() { return mY; };
	virtual inline int GetW() { return mW; };
	virtual inline int GetH() { return mH; };
	virtual int Move(int x, int y);
	virtual int Resize(int w, int h);
	virtual int ReallyMove(int x, int y);
	virtual int ReallyResize(int w, int h);
	virtual int MoveResize(int x, int y, int w, int h);
	virtual int GetSize(int *x, int *y, int *w, int *h);
	virtual int Raise();
	virtual int DrawFrame(int flag);
	virtual int WMIgnore(bool yes);
	virtual int WMDecorations(bool yes);

	virtual int LostFocus();
	virtual int SetFocus();
	virtual BaseWidget *GetFocusedWidget();
	virtual BaseWidget *GetParentWidget();
	virtual BaseWidget *GetTopLevelWidget();

	virtual inline int SetCanHaveFocus(int flag) { mCanHaveFocus = flag; };
	
	virtual Window GetXWindow();
	static Display *GetXDisplay();
	static int SetDisplayStr(char *d);
	static Display *XConnect();

	int GetTextWidth(char *text);

	virtual int SetCallback(void *data1, void *data2,
			void (*cb)(Object*, TEvent*, void*, void*));
protected:
	virtual int HandleKeyPress(XEvent *ev);
	static BaseWidget *sFocusedWidget;
	
	static char *mDisplayStr;
	static Display *mDisplay;
	float mOpacity;
	int mCanHaveFocus;
	BaseWidget *mParent;
	Atom xa_DeleteWindow;
	Window mWindow;
	Pixmap mDoubleBufferPixmap;
	int mDoubleBuffer;
	GC mDBGC;
	Pixmap mPixmap;
	XFontStruct *mFont;
	int mTextSize;
	GC mGC;
	int mX;
	int mY;
	int mW;
	int mH;
	int mVisible;
	int mDirty;
	int mDrawFrame;
	int mEnabled;
	char *mText;
	int mTextX;
	int mTextY;
	void *mData1;
	void *mData2;
	void (*mCallback)(Object*, TEvent*, void*, void*);
	unsigned long mBackground;
	unsigned long mForeground;
};

#endif
