#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>

#include <X11/X.h>
#include <X11/Xlib.h>

#include "dtk_application.h"
#include "dtk_timer.h"
#include "dtk_widget.h"

static void sig_alarm_handler(int sig)
{
	fprintf(stderr, "XPending bug caught\n");
}

int Application::sQuit = 0;

Application::Application()
{
	ENTER;
}

Application::~Application()
{
	ENTER;
}

int Application::Init()
{
	ENTER;
	Widget::XConnect();
	return 0;
}

int Application::RunOnce()
{
	TEvent tEvent;
	XEvent xev;
	Display *d = Widget::GetXDisplay();

	if (d) {
		while(XPending(d)) {
			alarm(1);
			signal(SIGALRM, sig_alarm_handler);
			XNextEvent(d, &xev);
			signal(SIGALRM, SIG_DFL);
			alarm(0);
			tEvent.xev = &xev;
			tEvent.type = XEVENT;
			EventHandler(&tEvent);
		}
	}
	tEvent.type = TIMEEVENT;
	EventHandler(&tEvent);
	return 0;
}

int Application::Run()
{
	ENTER;
	while (sQuit == 0) {
		RunOnce();
		usleep(10000);
	}

	return 0;
}

int Application::RunDialog(Dialog *dlg)
{
	int rv;
	TEvent tEvent;
	XEvent xev;
	int reparent = 0;

	if (dlg->GetParent() == NULL) {
		AddChild(dlg);
		reparent = 1;
	}
	dlg->Show();
	while ((rv = dlg->GetResult()) == DIALOG_BUTTON_NONE) {
		Display *d = Widget::GetXDisplay();
		if (d) {
			while (XPending(d)) {
				XNextEvent(d, &xev);
				tEvent.xev = &xev;
				tEvent.type = XEVENT;
				EventHandler(&tEvent);
			}
		}
		tEvent.type = TIMEEVENT;
		EventHandler(&tEvent);
	}
	dlg->Hide();
	if (reparent) RemoveChild(dlg);
	return rv;
}

int Application::Quit()
{
	ENTER;
	sQuit = 1;
	return 0;
}
