#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <X11/X.h>
#include <X11/Xlib.h>

#include "dtk_skin.h"
#include "dtk_combobox.h"

ComboBox::ComboBox(Widget *parent, char *contents, int num_entries,
		int x, int y, int w, int h)
	: Widget(parent, x, y, w, h)
{
	DrawFrame(0);

	mNumEntries = num_entries;
	mContents = (contents ? strdup(contents) : NULL);

	mTextEntry = new TextEntry(this, 0, 0, w - h , h);
	mTextEntry->Show();
	mTextEntry->Enable(0);

	mButton = new Button(this, w - h, 0, h, h);
	mButton->Show();
	mButton->SetPixmap(Skin::GetSkinPixmap(COMBOBTN_NORMAL), STATE_NORMAL);
	mButton->SetPixmap(Skin::GetSkinPixmap(COMBOBTN_HOVER), STATE_HOVER);
	mButton->SetPixmap(Skin::GetSkinPixmap(COMBOBTN_PRESSED),STATE_PRESSED);
	mButton->SetCallback((void*)this, NULL, ButtonCallback);

	if (mParent) {
		Window junk;
		XTranslateCoordinates(mDisplay, mParent->GetXWindow(),
				GetTopLevelWidget()->GetXWindow(),
				x, y, &x, &y, &junk);
	}
	mPopup = new Popup(GetTopLevelWidget(), contents, num_entries,
			x, y + h, w - h, 8, 1);
	mPopup->Hide();
	mPopup->DrawFrame(1);
	mPopup->SetCallback((void*)this, NULL, PopupCallback);
	mPopup->SetPixmap(Skin::GetSkinPixmap(MENU_HOVER), STATE_HOVER);
	mPopup->SetPixmap(Skin::GetSkinPixmap(MENU_PRESSED), STATE_PRESSED);
}

ComboBox::~ComboBox()
{
	if (mContents) free((void*)mContents);
}

int ComboBox::OnPopupReleased(int selected)
{
	char *c1, *c2, *p;
	int i;

	mSelected = selected;

	if (mContents) {
		c1 = p = strdup(mContents);
		for (i = 0; i < mNumEntries; i++) {
			c2 = strchr(c1, ',');
			if (c2) *c2 = '\0';
			if (i == selected) {
				mTextEntry->SetBuffer(c1);
				break;
			}
			c1 = c2 + 1;
		}
	}
	if (mCallback) {
		DTKEvent dev;
		TEvent tev;

		dev.type = DTK_COMBO_CHANGED;
		dev.combobox.object = (void*)this;
		tev.type = DTKEVENT;
		tev.dev = &dev;
		mCallback(this, &tev, mData1, mData2);
	}
}

int ComboBox::OnButtonPressed()
{
	mPopup->Show();
	return 0;
}

char *ComboBox::GetEntry()
{
	return NULL;
}

int ComboBox::SetText(char *text)
{
	if (mTextEntry) {
		mTextEntry->SetBuffer(text);
	}
	return 0;
}

void ComboBox::PopupCallback(Object *o, TEvent *e, int entry,
		void *d1, void *d2)
{
	ComboBox *c = (ComboBox*)d1;

	if (e->type != XEVENT) return;
	if (e->xev->type != ButtonRelease) return;
	c->OnPopupReleased(entry);
}

void ComboBox::ButtonCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	ComboBox *c = (ComboBox*)d1;

	if (e->type != XEVENT) return;
	if (e->xev->type != ButtonPress) return;
	c->OnButtonPressed();
}

