#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <string.h>

#include "dtk_config.h"

#define ENVIRONMENT 1024

CValue *Config::sValues = NULL;
int Config::sNumValues = 0;

int Config::ReadConfig(char *section, char *filename)
{
	FILE *f;
	char buf[1024];

	f = fopen(filename, "r");
	if (f == NULL) return 0;
	while (ReadLine(fileno(f), buf, 1023) >= 0) {
		ParseConfig(section, buf);
	}
	fclose(f);
	return 0;
}

int Config::WriteConfig(char *section, char *filename)
{
	FILE *f;
	int i;
	int section_len = strlen(section) + 1;

	if (sValues == NULL) return -1;

	f = fopen(filename, "w");
	if (f == NULL) return 0;

	for (i = 0; i < sNumValues; i++) {
		if (!strncasecmp(section,sValues[i].variable,
					strlen(section))) {
			if (sValues[i].variable[section_len-1] != '_') continue;
			fprintf(f, "%s=%s\n",
				(char*)(sValues[i].variable+section_len),
				sValues[i].value);
		}
	}
	fclose(f);
	return 0;
}

int Config::GetIntValue(char *section, char *variable, int def)
{
	char buf[1024];
	char *p = &buf[0];
	memset(buf, 0, 1024);
	if (GetValue(section, variable, &p) == -1) {
		return def;
	}
	return atoi(buf);
}

long Config::GetLongValue(char *section, char *variable, long def)
{
	char buf[1024];
	char *p = &buf[0];
	memset(buf, 0, 1024);
	if (GetValue(section, variable, &p) == -1) {
		return def;
	}
	return atol(buf);
}

char *Config::GetCharValue(char *section, char *variable, char *def)
{
	char buf[1024];
	char *p = &buf[0];
	memset(buf, 0, 1024);
	if (GetValue(section, variable, &p) == -1) {
		return ((def == NULL) ? NULL : strdup(def));
	}
	return strdup(buf);
}

int Config::SetIntValue(char *section, char *variable, int value)
{
	char buf[1024];
	snprintf(buf, 1024, "%d", value);
	return SetCharValue(section, variable, buf);
}

int Config::SetLongValue(char *section, char *variable, long value)
{
	char buf[1024];
	snprintf(buf, 1024, "%ld", value);
	return SetCharValue(section, variable, buf);
}

int Config::SetCharValue(char *section, char *variable, char *value)
{
	char buf[1024];
	char *p = &buf[0];
	int index;

	if ((section == NULL) || (variable == NULL) || (value == NULL))
		return 0;

	index = GetValue(section, variable, &p);
	if (index == -1) index = sNumValues++;

	if (sNumValues > ENVIRONMENT) {
		sNumValues--;
		fprintf(stderr, "Environment overflow error\n");
		return -1;
	}
	snprintf(buf, 1024, "%s_%s", section, variable);
	if (sValues[index].variable == NULL) {
		sValues[index].variable = strdup(buf);
	}
	if (sValues[index].value != NULL) {
		free((void*)sValues[index].value);
	}
	sValues[index].value = strdup(value);
	return 0;
}

int Config::GetValue(char *section, char *variable, char *buf[])
{
	int i = 0;
	char tmp[1024];
	if (sValues == NULL) {
		sValues = (CValue*)malloc(ENVIRONMENT * sizeof(CValue*));
		memset(sValues, 0, ENVIRONMENT * sizeof(CValue*));
		sNumValues = 0;
	}
	snprintf(tmp, 1024, "%s_%s", section, variable);
	for (i = 0; i < sNumValues; i++) {
		if (!strcasecmp(sValues[i].variable, tmp)) {
			memcpy(*buf, sValues[i].value,
					strlen(sValues[i].value)+1);
			return i;
		}
	}
	return -1;
}

int Config::ParseConfig(char *section, char *buf)
{
	char *variable;
	char *value;
	char *c;
	char tmp[1024];

	if (sNumValues == ENVIRONMENT) {
		fprintf(stderr, "Environment overflow error\n");
		return -1;
	}

	variable = buf;
	c = strchr(buf, '=');
	if (c == NULL) return -1;
	*c = '\0';
	c++;
	value = c;
	return SetCharValue(section, variable, value);
}

int Config::ReadLine(int fd, char *buffer, int length)
{
	char b;
	int i = 0;

	if (buffer == NULL) return -1;
	do {
		int j = 0;
		if ((j = read(fd, &b, 1) <= 0)) {
			return -1;
		}
		buffer[i] = b;
		if (i >= length) return 0;
		i++;
	} while ((b != '\n') && (b != '\0'));
	buffer[i-1] = '\0';
	return 0;
}
