#include <stdlib.h>
#include <stdio.h>
#include "dtk_skin.h"
#include "dtk_button.h"
#include "dtk_listview.h"

ListView::ListView(Widget *parent, int x, int y, int w, int h)
	:Widget(parent, x, y, w, h)
{
	mSelectedEntry = 0;
	mNumEntries = 0;
	mNormal = 0;
	mHover = 0;
	mPressed = 0;
	mCallback = NULL;
	mScrolledParent = NULL;
}

ListView::~ListView()
{
}

int ListView::SetScrolledParent(ScrollView *sv)
{
	mScrolledParent = sv;
	return 0;
}

int ListView::AddEntry(char *text)
{
	Button *button;
	int h = 24;

	if (mTextSize > 24) h = mTextSize + 8;
	button = new Button(this, 0, 0, mW, h);
	button->Hide();
	button->SetText(text, 2, mTextSize + 4);
	button->SetTextSize(mTextSize);
	button->SetColor(0x0, 0x0, 0x7f7f, STATE_NORMAL);
	button->SetColor(0x0, 0x0, 0x7f7f, STATE_HOVER);
	button->SetColor(0x0, 0x7f7f, 0x7f7f, STATE_PRESSED);
	button->SetForegroundColor(0xffff, 0xffff, 0xffff, STATE_NORMAL);
	button->SetForegroundColor(0xffff, 0xffff, 0xffff, STATE_HOVER);
	button->SetForegroundColor(0xffff, 0xffff, 0xffff, STATE_PRESSED);

	if (mBackground) button->SetColor(mBackground);
	if (mForeground) button->SetForegroundColor(mForeground);
	button->SetPixmap(mPressed, STATE_PRESSED);
	button->SetToggleMode(1);
	button->DrawFrame(0);
	reflow();
	mNumEntries++;
	return 0;
}

int ListView::RemoveEntry(int entry)
{
	Widget *w = GetWidgetForEntry(entry);
	if (w) delete w;
	mSelectedEntry = 0;
	reflow();
	mNumEntries--;
	return 0;
}

int ListView::Clear()
{
	Widget *w;

	w = GetWidgetForEntry(1);
	while ((w != NULL) && (mFirstChild != NULL)) {
		delete w;
		w = GetWidgetForEntry(1);
	}
	mSelectedEntry = 0;
	reflow();
	return 0;
}

int ListView::GetSelectedEntry()
{
	return mSelectedEntry;
}

int ListView::SetSelectedEntry(int entry)
{
	Button *w;
	if (entry > 0) {
		w = (Button*)GetWidgetForEntry(entry);
		if (w) {
			w->SetToggle(1);
			if (mScrolledParent) {
				mScrolledParent->MakeVisible(w->GetX(),
						w->GetY(),
						w->GetW(), w->GetH());

			}
			if (mSelectedEntry == entry) return 0;
			if (mSelectedEntry) {
				w = (Button*)GetWidgetForEntry(mSelectedEntry);
				if (w) w->SetToggle(0);
			}
	mSelectedEntry = entry;
		}
	} else if (entry == -1) {
		if (mSelectedEntry) {
			w = (Button*)GetWidgetForEntry(mSelectedEntry);
			if (w) w->SetToggle(0);
		}
	}
	return 0;
}

char *ListView::GetEntryText(int entry)
{
	Button *w;
	w = (Button*)GetWidgetForEntry(entry);
	if (w) {
		return w->GetText();
	}
	return NULL;
}

int ListView::SetColor(unsigned int r, unsigned int g, unsigned int b)
{
	Button *btn;

	Widget::SetColor(r, g, b);
	return 0;
}

int ListView::SetPixmap(Pixmap p, int type)
{
	Button *btn;

	switch (type) {
		case STATE_NORMAL:
			mNormal = p;
			break;
		case STATE_HOVER:
			mHover = p;
			break;
		case STATE_PRESSED:
			mPressed = p;
			break;
	}
	for (btn = (Button*)mFirstChild; btn != NULL;
			btn = (Button*)btn->GetNextChild())
		btn->SetPixmap(p, type);
	return 0;
}

int ListView::SetCallback(void *data1, void *data2,
		void (*cb)(Object*,TEvent*,int,void*,void*))
{
	mData1 = data1;
	mData2 = data2;
	mCallback = cb;
	return 0;
}

int ListView::OnEntrySelected(int entry)
{
	Button *w = (Button*)GetWidgetForEntry(mSelectedEntry);

	if (mSelectedEntry != entry) {
		if (w) {
			w->SetToggle(0);
		}
	}

	mSelectedEntry = entry;

	if (mCallback) {
		DTKEvent d;
		TEvent e;
		d.type = DTK_LISTVIEW_SELECTION;
		d.listview.object = this;
		e.type = DTKEVENT;
		e.dev = &d;
		mCallback(this, &e, entry, mData1, mData2);
	}
	return 0;
}

Widget *ListView::GetWidgetForEntry(int entry)
{
	Widget *rv = NULL;

	if (entry == 0) return NULL;

	rv = (Widget*)mFirstChild;
	while (rv && (entry > 1)) {
		rv = (Widget*)rv->GetNextChild();
		entry--;
	}
	return rv;
}

int ListView::reflow()
{
	int x, y, w, h;
	int th;
	int i;
	Widget *child;

	i = 1; x = 0; y = 0; w = mW; h = 24; th = 0;

	if (mTextSize > 20) h = mTextSize * 2 - 16;
	child = (Widget*)mFirstChild;
	while (child) {
		child->MoveResize(x, y, w, h);
		child->Show();
		child->SetCallback((void*)this, (void*)i, ButtonCallback);
		y += h;
		th += h;
		i++;
		child = (Widget*)child->GetNextChild();
	}
	Resize(mW, th);
	if (mParent) {
		mParent->RemoveChild(this);
		mParent->AddChild(this);
	}
	return 0;
}

void ListView::ButtonCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	ListView *l = (ListView*)d1;
	int entry = (int)d2;

	if (e == NULL) return;
	if (e->type != XEVENT) return;
	if (e->xev->type != ButtonPress) return;

	l->OnEntrySelected(entry);
}

