#include <stdlib.h>
#include <stdio.h>

#include "dtk_object.h"

Object::Object()
{
	static int count = 1;
	ENTER;
	mFirstChild = NULL;
	mNextChild = NULL;
	mPrevChild = NULL;
	mParentObject = NULL;
	mObjectType = OBJTYPE_OBJECT;
}

Object::~Object()
{
	ENTER;
	Object *o = mFirstChild;
	Object *next = NULL;

	while (o != NULL) {
		next = o->GetNextChild();
		o->Destroy();
		delete o;
		o = next;
	}
}

int Object::Destroy()
{
	ENTER;
	return 0;
}

int Object::AddChild(Object *child)
{
	ENTER;
	Object *o = mFirstChild;
	Object *last = NULL;

	if (mFirstChild == NULL) {
		mFirstChild = child;
		mFirstChild->SetNextChild(NULL);
		mFirstChild->SetPrevChild(NULL);
		return 0;
	}

	while (o != NULL) {
		last = o;
		o = o->GetNextChild();
	}

	o = last;
	o->SetNextChild(child);
	child->SetPrevChild(o);
	child->SetNextChild(NULL);
	o->SetParent(this);
	return 0;
}

int Object::RemoveChild(Object *child)
{
	ENTER;
	Object *o, *prev, *next;

	o = mFirstChild;

	if (child == NULL) return 0;
	if (mFirstChild == NULL) return 0;

	if (child == mFirstChild) {
		mFirstChild = child->GetNextChild();
		if (mFirstChild) mFirstChild->SetPrevChild(NULL);
		return 0;
	}

	next = child->GetNextChild();
	prev = child->GetPrevChild();

	if (next) next->SetPrevChild(prev);
	if (prev) prev->SetNextChild(next);

	return 0;
}

Object *Object::GetNextChild()
{
	ENTER;
	return mNextChild;
}

Object *Object::GetPrevChild()
{
	ENTER;
	return mPrevChild;
}

Object *Object::GetParent()
{
	ENTER;
	return mParentObject;
}

int Object::SetNextChild(Object *child)
{
	ENTER;
	mNextChild = child;
	return 0;
}

int Object::SetPrevChild(Object *child)
{
	ENTER;
	mPrevChild = child;
	return 0;
}

int Object::SetParent(Object *parent)
{
	ENTER;
	mParentObject = parent;
	return 0;
}

int Object::EventHandler(TEvent *ev)
{
	ENTER;
	Object *child = mFirstChild;

	while (child != NULL) {
		if (child->EventHandler(ev)) return 1;
		if (child == child->GetNextChild()) {
			fprintf(stderr, "Bug 0x%x 0x%x\n",
					child, child->GetNextChild());
			return 1;
		}
		child = child->GetNextChild();
	}
	return 0;
}

int Object::InsertEvent(TEvent *ev)
{
	Object *obj, *tmp;

	obj = this;

	while ((tmp = obj->GetParent()) != NULL) obj = tmp;

	obj->EventHandler(ev);

	return 0;
}

int Object::GetObjectType()
{
	return mObjectType;
}

