#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "dtk_popup.h"
#include "dtk_button.h"

Popup::Popup(Widget *parent, char *contents, int num_entries,
		int x, int y, int w, int h, int fixed)
	:Widget(parent, x, y, w, h)
{
	mFixed = 1;
	CreatePopup(contents, num_entries);
}

Popup::Popup(Widget *parent, char *contents, int num_entries)
	: Widget(parent, 0, 0, 16, 8)
{
	mFixed = 0;
	CreatePopup(contents, num_entries);
}

Popup::~Popup()
{
	if (mContents) free((void*)mContents);
	mContents = NULL;
}

int Popup::CreatePopup(char *contents, int num_entries)
{
	Button *b;
	int i;
	char *c1, *c2, *p;
	int direction, ascent, descent;
	XCharStruct overall;
	int x, y, w, h;

	Hide();

	mNumEntries = num_entries;
	mContents = strdup(contents);
	mCallback = NULL;
	mData1 = NULL;
	mData2 = NULL;

	w = mW; h = mH;

	c1 = p = strdup(contents);
	for (i = 0; i < num_entries; i++) {
		c2 = strchr(c1, ',');
		if (c2 != NULL) *c2 = '\0';
		XTextExtents(mFont, c1, strlen(c1), &direction,
				&ascent, &descent, &overall);
		if (w < overall.width + 24) w = overall.width + 24;
		if (h < (ascent + descent + 4) * num_entries + 4)
			h = (ascent + descent + 4) * num_entries + 4;
		c1 = c2 + 1;
	}
	free((void*)p);

	if (!mFixed) Resize(w, h);
	else Resize(-1, h);

	x = 1; y = 1; w = mW - 2; h = (mH - 2) / num_entries;

	c1 = p = strdup(contents);
	for (i = 0; i < num_entries; i++) {
		c2 = strchr(c1, ',');
		if (c2 != NULL) *c2 = '\0';
		b = new Button(this, x, y, w, h);
		b->DrawFrame(0);
		b->SetText(c1, 8, ascent + 3);
		b->SetColor(0xff00, 0xff00, 0xff00);
		b->SetCallback((void*)this, (void*)i, Popup::EntryCallback);
		b->Show();
		y += h;
		c1 = c2 + 1;
	}
	free((void*)p);

	SetEventMask(ButtonPressMask | ButtonReleaseMask |
			ExposureMask | KeyReleaseMask);
}

int Popup::Show()
{
	Window wjunk;
	int ijunk, x, y;
	unsigned int ujunk;

	if (mParent && !mFixed) {
		int w, h;
		XQueryPointer(mDisplay, mParent->GetXWindow(),
				&wjunk, &wjunk, &ijunk, &ijunk,
				&x, &y, &ujunk);
		w = mParent->GetW();
		h = mParent->GetH();
		if (x + mW > w) x = w - mW;
		if (y + mH > h) y = h - mH;
		if (x < 0) x = 0;
		if (y < 0) y = 0;
		Move(x, y);
	}
	Widget::Show();
	XRaiseWindow(mDisplay, mWindow);
	XGrabKeyboard(mDisplay, mWindow, False, GrabModeAsync,
			GrabModeAsync, CurrentTime);

	return 0;
}

int Popup::Hide()
{
	Widget::Hide();
	XUngrabKeyboard(mDisplay, CurrentTime);
	return 0;
}

int Popup::SetPixmap(Pixmap p, int type)
{
	Button *btn;

	for (btn = (Button*)mFirstChild; btn != NULL;
			btn = (Button*)btn->GetNextChild())
		btn->SetPixmap(p, type);
	return 0;
}

int Popup::EventHandler(TEvent *ev)
{
	if (ev->type != XEVENT) return 0;

	if (ev->xev->type == KeyRelease) {
		if (ev->xev->xkey.keycode == 9) {
			Hide();
			if (mCallback == NULL) return 1;
			mCallback(this, ev, -1, mData1, mData2);
		}
	}
	return Widget::EventHandler(ev);
}

int Popup::SetCallback(void *data1, void *data2,
		void (*cb)(Object*, TEvent*, int, void*, void*))
{
	mData1 = data1;
	mData2 = data2;
	mCallback = cb;
	return 0;
}

int Popup::CallCallback(TEvent *ev, int data)
{
	if (mCallback == NULL) return 0;
	mCallback(this, ev, data, mData1, mData2);
	return 0;
}

int Popup::SetEntryText(char *text, int entry)
{
	Button *b;
	int i;

	if (entry > mNumEntries) return 0;

	b = (Button*)mFirstChild;

	if (b == NULL) return 0;

	for (i = 0; i < entry; i++) {
		b = (Button*)GetNextChild();
	}

	if (b == NULL) return 0;

	b->SetText(text, -1, -1);
	return 0;
}

void Popup::EntryCallback(Object *obj, TEvent *ev, void *data1, void *data2)
{
	Popup *p = (Popup*)data1;
	int entry = (int)data2;

	if (p == NULL) return;

	p->CallCallback(ev, entry);
	if (ev->type != XEVENT) return;
	if (ev->xev->type != ButtonRelease) return;
	p->Hide();
}
