#include <stdlib.h>
#include <stdio.h>

#include "dtk_scrollview.h"

ScrollView::ScrollView(Widget *parent, int x, int y, int w, int h)
	: Widget(parent, x, y, w, h)
{
	ignoreNextChild = 1;
	mScrollArea = new Widget(this, 0, 0, mW - 16, mH - 16);
	mScrollArea->GetSize(NULL, NULL, &w, &h);
	mScrollArea->Show();

	mScrollX = 0;
	mScrollY = 0;

	mHScroll = new Slider(this, 0, mH - 16, mW - 16, 16, HORIZONTAL);
	mHScroll->Show();
	mHScroll->SetCallback(this, (void*)1, ScrollCallback);
	mHScroll->Raise();

	mVScroll = new Slider(this, mW - 16, 0, 16, mH - 16, VERTICAL);
	mVScroll->Show();
	mVScroll->SetCallback(this, (void*)2, ScrollCallback);
	mHScroll->Raise();
	ignoreNextChild = 0;
}

ScrollView::~ScrollView()
{
	ignoreNextChild = 1;
}

int ScrollView::MakeVisible(int x, int y, int w, int h)
{
	int sX, sY, sW, sH;

	mScrollArea->GetSize(&sX, &sY, &sW, &sH);

	if ((y + sY + h < mH) && (y >= abs(sY))) {
		return 0;
	} else if (y < abs(sY)) {
		mScrollArea->ReallyMove(0, -y);
	} else if (y + sY + h > 0) {
		mScrollArea->ReallyMove(0, sY - h);
	}
	return 0;
}
int ScrollView::SetColor(unsigned int r, unsigned int g, unsigned int b)
{
	mScrollArea->SetColor(r, g, b);
	return Widget::SetColor(r, g, b);
}

int ScrollView::AddChild(Object *child)
{
	int sW, sH, cX, cY, cW, cH;

	if ((child->GetObjectType() == OBJTYPE_OBJECT) || ignoreNextChild)
		return Widget::AddChild(child);

	mScrollArea->AddChild(child);
	((BaseWidget*)child)->GetSize(&cX, &cY, &cW, &cH);
	mScrollArea->GetSize(NULL, NULL, &sW, &sH);
	if (sW < cX + cW) sW = cX + cW;
	if (sH < cY + cH) sH = cY + cH;
	mScrollArea->Resize(sW, sH);
	return 0;
}

int ScrollView::RemoveChild(Object *child)
{
	int sW, sH, cX, cY, cW, cH;
	BaseWidget *w;

	if ((child->GetObjectType() == OBJTYPE_OBJECT) || ignoreNextChild)
		                return Widget::RemoveChild(child);

	if ((child == mHScroll) ||
			(child == mVScroll) ||
			(child == mScrollArea)) {
		return Widget::RemoveChild(child);
	}

	if (mScrollArea == NULL) return 0;

	mScrollArea->RemoveChild(child);

	BaseWidget *widget = (BaseWidget*)mScrollArea->GetFirstChild();

	sW = 1;
	sH = 1;
	while (widget != NULL) {
		widget->GetSize(&cX, &cY, &cW, &cH);
		if (sW < cX + cW) sW = cX + cW;
		if (sH < cY + cH) sH = cY + cH;
		widget = (BaseWidget*)widget->GetNextChild();
	}
	mScrollArea->Resize(sW, sH);
	return 0;
}

Window ScrollView::GetXWindow()
{
	if (ignoreNextChild) {
		return mWindow;
	} else {
		return mScrollArea->GetXWindow();
	}
}
int ScrollView::OnVScrollMove()
{
	int dy = 0;
	int scrollX, scrollY, scrollH;
	float percent = mVScroll->GetValue();

	mScrollArea->GetSize(&scrollX, &scrollY, NULL, &scrollH);

	if (mH - 16 >= scrollH) return 0;

	dy = (int)(((mH - 16) - scrollH) * (100 - percent) / 100);
	mScrollArea->ReallyMove(scrollX, dy);
	return 0;
}

int ScrollView::OnHScrollMove()
{
	int dx = 0;
	int scrollX, scrollY, scrollW;
	float percent = mHScroll->GetValue();

	mScrollArea->GetSize(&scrollX, &scrollY, &scrollW, NULL);

	if (mW - 16 >= scrollW) return 0;

	dx = (int)(((mW - 16) - scrollW) * percent / 100);
	mScrollArea->ReallyMove(dx, scrollY);
	return 0;
}

void ScrollView::ScrollCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	ScrollView *s = (ScrollView*)d1;
	int i = (int)d2;

	if (e->type != DTKEVENT) return;
	if (i == 1) s->OnHScrollMove();
	else s->OnVScrollMove();
}

