#include <dlfcn.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "skin_interface.h"

int (*skin_init)(Display *display);
int (*skin_done)();
int (*skin_get_info)(int infotype, char *buffer, int length);
int (*skin_draw_image)(Display *display, Drawable d, unsigned long button);
Pixmap (*skin_get_pixmap)(Display *display, unsigned long button);

int dfl_skin_init(Display *d)
{
	fprintf(stderr, "SKIN_INIT: Could not load the specified skin\n");
	return NOPIXMAP;
}

int dfl_skin_done()
{
	fprintf(stderr, "SKIN_DONE: Could not load the specified skin\n");
	return NOPIXMAP;
}

int dfl_skin_get_info(int infotype, char *buffer, int length)
{
	char *info = "no info";
	int i;

	if (buffer == NULL) return MYBAD;
	for (i = 0; ((i < length-1) && (i < strlen(info))); i++)
		buffer[i] = info[i];
	buffer[i] = '\0';
	return OK;
}

int dfl_skin_draw_image(Display *display, Drawable d, unsigned long button)
{
	fprintf(stderr, "SKIN_DRAW_IMAGE: Could not load the specified skin\n");
	return NOPIXMAP;
}

Pixmap dfl_skin_get_pixmap(Display *display, unsigned long button)
{
	fprintf(stderr, "SKIN_GET_PIXMAP: Could not load the specified skin\n");
	return NOPIXMAP;
}

void *skin_load_so(char *so)
{
	void *rv = NULL;

	rv = dlopen(so, RTLD_LAZY);

	if (!rv) {
		char *c = strrchr(so,'.');
		if (c) *c = '\0';
		rv = dlopen(so, RTLD_LAZY);
	}

	if (rv) {
		skin_init = dlsym(rv, "dl_skin_init");
		skin_done = dlsym(rv, "dl_skin_done");
		skin_get_info = dlsym(rv, "dl_skin_get_info");
		skin_draw_image = dlsym(rv, "dl_skin_draw_image");
		skin_get_pixmap = dlsym(rv, "dl_skin_get_pixmap");
	} else {
		fprintf(stderr, "dlopen error: %s\n", dlerror());
	}
	if (!skin_init) skin_init = dfl_skin_init;
	if (!skin_done) skin_done = dfl_skin_done;
	if (!skin_get_info) skin_get_info = dfl_skin_get_info;
	if (!skin_draw_image) skin_draw_image = dfl_skin_draw_image;
	if (!skin_get_pixmap) skin_get_pixmap = dfl_skin_get_pixmap;
	return rv;
}

int skin_unload_so(void *handle)
{
	skin_init = NULL;
	skin_done = NULL;
	skin_draw_image = NULL;
	dlclose(handle);
	return 0;
}

