#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include "dtk_skin.h"
#include "dtk_slider.h"

Slider::Slider(Widget *parent, int x, int y, int w, int h, int orientation)
	: Widget(parent, x, y, w, h)
{
	mValue = 0;
	mMinValue = 0;
	mMaxValue = 100;
	mGrabbed = 0;

	mOrientation = orientation;

	if (mOrientation == HORIZONTAL) {
		SetPixmap(Skin::GetSkinPixmap(HSLIDER_TRACK));
		mSliderButton = new Button(this, 0, 0, h, h);
	} else {
		SetPixmap(Skin::GetSkinPixmap(VSLIDER_TRACK));
		mSliderButton = new Button(this, 0, 0, w, w);
	}

	mSliderButton->Show();
	mSliderButton->SetPixmap(Skin::GetSkinPixmap(SLIDER_NORMAL),
			STATE_NORMAL);
	mSliderButton->SetPixmap(Skin::GetSkinPixmap(SLIDER_HOVER),
			STATE_HOVER);
	mSliderButton->SetPixmap(Skin::GetSkinPixmap(SLIDER_PRESSED),
			STATE_PRESSED);
	mSliderButton->SetCallback((void*)this, NULL, ButtonCallback);
}

Slider::~Slider()
{
}


float Slider::GetValue()
{
	float rv;
	if (mOrientation == HORIZONTAL)
		rv = (float)mValue * mMaxValue / (float)(mW-mH);
	else
		rv = (float)((mH-mW) - mValue) * mMaxValue / (float)(mH-mW);

	return rv + mMinValue;
}

int Slider::SetValue(float value)
{
	int delta;

	if (mOrientation == HORIZONTAL) {
		delta = (int)((float)(mW-mH) * value / mMaxValue);
	} else {
		delta = (int)((float)(mH-mW) * (mMaxValue - value) / mMaxValue);
	}
	mValue = 0;
	MoveButton(delta);
	return 0;
}

int Slider::SetRange(int min, int max)
{
	if (min >= max) return 0;
	mMinValue = min;
	mMaxValue = max - min;
	return 0;
}

int Slider::OnButtonPress(int x, int y)
{
	mOldX = x;
	mOldY = y;
	mGrabbed = 1;
	return 0;
}

int Slider::OnButtonRelease()
{
	mGrabbed = 0;
}

int Slider::OnMouseMove(int x, int y)
{
	int dx;
	int dy;

	if (!mGrabbed) return 0;

	dx = x - mOldX;
	dy = y - mOldY;

	mOldX = x; mOldY = y;

	if (mOrientation == HORIZONTAL) {
		MoveButton(dx);
	} else {
		MoveButton(dy);
	}
	return 0;
}

int Slider::MoveButton(int delta)
{
	int newValue;

	if (mOrientation == HORIZONTAL) {
		newValue = mValue + delta;
		if (newValue < 0) {
			newValue = 0;
			return 0;
		}
		if (newValue > mW - mH) {
			newValue = mMaxValue;
			return 0;
		}
		mSliderButton->Move(newValue, -1);
	} else {
		newValue = mValue + delta;
		if (newValue < 0) {
			newValue = 0;
			return 0;
		}
		if (newValue > mH - mW) {
			newValue = mMaxValue;
			return 0;
		}
		mSliderButton->Move(-1, newValue);
	}
	mValue = newValue;
	if (mCallback) {
		DTKEvent dev;
		TEvent tev;
		dev.type = DTK_SLIDER_CHANGED;
		dev.slider.value = mValue;
		tev.type = DTKEVENT;
		tev.dev = &dev;
		mCallback(this, &tev, mData1, mData2);
	}
	return 0;
}

int Slider::PercentToPixels(float Percent)
{
	int rv = 0;

	if (mOrientation == HORIZONTAL)
		rv = (int)((float)mW * Percent / mMaxValue);
	else
		rv = (int)((float)mH * Percent / mMaxValue);
	return rv;
}

void Slider::ButtonCallback(Object *o, TEvent *e, void *data1, void *data2)
{
	Slider *s = (Slider*)data1;
	if (e->type != XEVENT) return;

	if (e->xev->type == ButtonPress) {
		s->OnButtonPress(e->xev->xbutton.x_root, e->xev->xbutton.y_root);
	} else if (e->xev->type == ButtonRelease) {
		s->OnButtonRelease();
	} else if (e->xev->type = MotionNotify) {
		s->OnMouseMove(e->xev->xmotion.x_root,e->xev->xmotion.y_root);
	}
}
