#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "dtk_socket.h"

Socket::Socket()
{
	mPending = 0;
	mSockFD = -1;
	mListenFD = -1;
}

Socket::~Socket()
{
	Disconnect();
}

int Socket::EventHandler(TEvent *ev)
{
	if (ev == NULL) return 0;

	if (ev->type == TIMEEVENT) {
		fd_set fds;
		struct timeval tv;
		int s;

		if (mListenFD == 0) return 0;

		if (mSockFD <= 0) {
			mSockFD = accept(mListenFD, 0, 0);
			if ((mSockFD == -1) && (errno == EAGAIN)) {
				return 0;
			} else if (mSockFD == -1) {
				fprintf(stderr, "Socket: Got error %s\n",
						strerror(errno));
				return 0;
			}
		}

		FD_ZERO(&fds);
		FD_SET(mSockFD, &fds);
		tv.tv_sec = 0;
		tv.tv_usec = 10;

		s = select(mSockFD + 1, &fds, NULL, NULL, &tv);
		if ((s > 0) && (FD_ISSET(mSockFD, &fds))) {
			mPending = 1;
		} else {
			mPending = 0;
		}
	}
	return 0;
}

int Socket::Connect(char *sockname)
{
	struct sockaddr_un server;
	long flags;

	if (sockname == NULL) return 0;

	mSockFD = -1;
	unlink(sockname);
	mListenFD = socket(AF_UNIX, SOCK_STREAM, 0);
	if (mListenFD < 0) {
		fprintf(stderr, "Socket: Cannot create socket\n");
		exit(1);
	}

	flags = (long)fcntl(mListenFD, F_GETFL);
	fcntl(mListenFD, F_SETFL, flags | O_NONBLOCK);

	server.sun_family = AF_UNIX;
	strcpy(server.sun_path, sockname);

	if (bind(mListenFD, (struct sockaddr*)&server,
				sizeof(struct sockaddr_un))) {
		fprintf(stderr, "Socket: Could not bind\n");
		exit(1);
	}
	listen(mListenFD, 5);

	return 0;
}

int Socket::Disconnect()
{
	if (mSockFD > 0) close(mSockFD);
	if (mListenFD > 0) close(mListenFD);
	mSockFD = -1;
	mListenFD = -1;
	return 0;
}

int Socket::Write(char *buffer)
{
	if (mSockFD <= 0) return 0;
	if (buffer == NULL) return 0;

	if (write(mSockFD, buffer, strlen(buffer)) < 0) {
		fprintf(stderr, "Socket: Could not write to the socket\n");
		return 0;
	}
	return 0;
}

int Socket::Read(char *buffer, int length)
{
	char b;
	int i = 0;

	if (buffer == NULL) return 0;
	do {
		int j = 0;
		if ((j = read(mSockFD, &b, 1) <= 0)) {
			fprintf(stderr, "Lost connection, %u\n", getpid());
			return -1;
		}
		buffer[i] = b;
		if (i >= length) return 0;
		i++;
	} while ((b != '\n') && (b != '\0'));
	buffer[i-1] = '\0';
	return 0;
}

int Socket::Pending()
{
	return mPending;
}




int SocketClient::EventHandler(TEvent *ev)
{
	if (ev == NULL) return 0;

	if (ev->type == TIMEEVENT) {
		fd_set fds;
		struct timeval tv;
		int s;

		if (mSockFD == 0) return 0;

		FD_ZERO(&fds);
		FD_SET(mSockFD, &fds);
		tv.tv_sec = 0;
		tv.tv_usec = 10;

		s = select(mSockFD + 1, &fds, NULL, NULL, &tv);
		if ((s > 0) && (FD_ISSET(mSockFD, &fds))) {
			mPending = 1;
		} else if (s == 0) {
			mPending = 0;
		} else {
			fprintf(stderr, "SocketClient: error: %s\n",
					strerror(errno));
			exit(1);
		}
	}
	return 0;
}

int SocketClient::Connect(char *sockname)
{
	struct sockaddr_un server;
	long flags;

	if (sockname == NULL) return 0;

	mSockFD = socket(AF_UNIX, SOCK_STREAM, 0);
	if (mSockFD < 0) {
		fprintf(stderr, "SocketClient: error opening socket\n");
		exit(1);
	}

	server.sun_family = AF_UNIX;
	strcpy(server.sun_path, sockname);

	if (connect(mSockFD, (struct sockaddr*)&server,
				sizeof(struct sockaddr_un))) {
		close(mSockFD);
		mSockFD = 0;
		fprintf(stderr, "SocketClient: Could not connect to socket\n");
		exit(1);
	}

	return 0;
}


