#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define XK_MISCELLANY
#include <X11/keysymdef.h>

#include "dtk_skin.h"
#include "dtk_spinbox.h"

SpinBox::SpinBox(Widget *parent, int x, int y, int w, int h)
	: Widget(parent, x, y, w, h)
{
	mUpButton = new Button(this, w - 12, 0, 12, h/2);
	mUpButton->Show();
	mUpButton->SetCallback((void*)this, (void*)1, ButtonCallback);
	mUpButton->SetPixmap(Skin::GetSkinPixmap(SPINUP_NORMAL),STATE_NORMAL);
	mUpButton->SetPixmap(Skin::GetSkinPixmap(SPINUP_HOVER),STATE_HOVER);
	mUpButton->SetPixmap(Skin::GetSkinPixmap(SPINUP_PRESSED),STATE_PRESSED);
	mUpButton->DrawFrame(0);

	mDownButton = new Button(this, w - 12, h/2, 12, h/2);
	mDownButton->Show();
	mDownButton->SetCallback((void*)this, (void*)2, ButtonCallback);
	mDownButton->SetPixmap(Skin::GetSkinPixmap(SPINDOWN_NORMAL),
			STATE_NORMAL);
	mDownButton->SetPixmap(Skin::GetSkinPixmap(SPINDOWN_HOVER),STATE_HOVER);
	mDownButton->SetPixmap(Skin::GetSkinPixmap(SPINDOWN_PRESSED),
			STATE_PRESSED);
	mDownButton->DrawFrame(0);

	mTextEntry = new TextEntry(this, 0, 0, w - 12, h);
	mTextEntry->Show();
	mTextEntry->SetCallback((void*)this, NULL, TextCallback);

	mMinValue = -32767;
	mMaxValue = 32767;
	mValue = 0;
}

SpinBox::~SpinBox()
{
}

int SpinBox::GetValue()
{
	return mValue;
}

int SpinBox::SetValue(int value)
{
	char buf[32];
	if (value < mMinValue) value = mMinValue;
	if (value > mMaxValue) value = mMaxValue;
	mValue = value;

	snprintf(buf, 32, "%d", mValue);
	mTextEntry->SetBuffer(buf); //, 4, mH / 2 + 4);
	return 0;
}

int SpinBox::SetRange(int min, int max)
{
	if (min >= max) return 0;

	if (min > mValue) mValue = min;
	if (max < mValue) mValue = max;

	mMinValue = min;
	mMaxValue = max;

	return SetValue(mValue);
}

int SpinBox::OnButtonPress(int direction)
{
	if (direction == 1) {
		SetValue(mValue + 1);
	} else {
		SetValue(mValue - 1);
	}
	return 0;
}

int SpinBox::OnTextChanged()
{
	char *text = mTextEntry->GetBufferPtr();
	int value;

	if (text == NULL) mValue = 0;
	else mValue = atoi(text);

	if (mCallback) {
		DTKEvent dev;
		TEvent tev;
		dev.type = DTK_SPINBOX_CHANGED;
		dev.spinbox.value = mValue;
		tev.type = DTKEVENT;
		tev.dev = &dev;
		mCallback(this, &tev, mData1, mData2);
	}
	return 0;
}

void SpinBox::ButtonCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	SpinBox *s = (SpinBox*)d1;
	int direction = (int)d2;

	if (e->type != XEVENT) return;
	if (e->xev->type != ButtonPress) return;
	s->OnButtonPress(direction);
}

void SpinBox::TextCallback(Object *o, TEvent *e, void *d1, void *d2)
{
	SpinBox *s = (SpinBox*)d1;

	if (e->type == DTKEVENT) {
		if (e->dev->type != DTK_TEXTENTRY_CHANGED) return;
		s->OnTextChanged();
	} else if (e->type == XEVENT) {
		char c;
		KeySym keysym;
		if (e->xev->type != KeyPress) return;
		keysym = XKeycodeToKeysym(mDisplay, e->xev->xkey.keycode, 0);
		switch (keysym) {
			case XK_Up:
				s->OnButtonPress(1);
				break;
			case XK_Down:
				s->OnButtonPress(0);
				break;
		}
	}
}
