#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <time.h>

#include "dtk_timer.h"

Timer::Timer(unsigned int duration, void *data1, void *data2,
		void (*go)(void*, void*))
{
	mRunning = 1;
	mDuration = duration;
	mCallback = go;
	mData1 = data1;
	mData2 = data2;
	mStartTime = time(NULL);
	mMSMode = 0;
}

Timer::Timer(struct timeval *tv, void *data1, void *data2,
		void (*go)(void*, void*))
{
	mRunning = 1;
	mMSDuration.tv_sec = tv->tv_sec;
	mMSDuration.tv_usec = tv->tv_usec;
	mCallback = go;
	mData1 = data1;
	mData2 = data2;
	gettimeofday(&mMSStartTime, NULL);
	mMSStartTime.tv_sec;
	mMSStartTime.tv_usec;
	mMSMode = 1;
}

Timer::~Timer()
{
}

int tv_cmp(struct timeval *a, struct timeval *b)
{
	if (a->tv_sec > b->tv_sec)
		return 1;
	if (a->tv_sec < b->tv_sec)
		return -1;
	if (a->tv_usec > b->tv_usec)
		return 1;
	if (a->tv_usec < b->tv_usec)
		return -1;
	return 0;
}

int Timer::EventHandler(TEvent *ev)
{
	if (mCallback == NULL) return 0;
	if (ev == NULL) return 0;
	if (ev->type != TIMEEVENT) return 0;

	if (!mRunning) return 0;

	if (mMSMode) {
		struct timeval tv;
		int diff;
		gettimeofday(&tv, NULL);

		diff = tv_cmp(&tv, &mMSStartTime);
		if (diff > 0) {
			mCallback(mData1, mData2);
			mMSStartTime.tv_sec = tv.tv_sec + mMSDuration.tv_sec;
			mMSStartTime.tv_usec = tv.tv_usec + mMSDuration.tv_usec;
		}
	} else {
		if (mStartTime + mDuration <= time(NULL)) {
			mStartTime = time(NULL);
			mCallback(mData1, mData2);
		}
	}
	return 0;
}

int Timer::Stop()
{
	mRunning = 0;
	return 0;
}

int Timer::Start()
{
	if (mMSMode) {
		gettimeofday(&mMSStartTime, NULL);
		mMSStartTime.tv_sec += mMSDuration.tv_sec;
		mMSStartTime.tv_usec += mMSDuration.tv_usec;
	} else mStartTime = time(NULL);
	mRunning = 1;
	return 0;
}

int Timer::SetDuration(unsigned int duration)
{
	mDuration = duration;
	if (mRunning) {
		Stop();
		Start();
	}
	return 0;
}

int Timer::SetDuration(struct timeval *tv)
{
	mMSDuration.tv_sec = tv->tv_sec;
	mMSDuration.tv_usec = tv->tv_usec;
	if (mRunning) {
		Stop();
		Start();
	}
	return 0;
}

