#include <stdlib.h>
#include <stdio.h>
#include <xine.h>

#include "Imlib.h"
#include <X11/xpm.h>

#include "videobox.h"

VideoBox::VideoBox(Widget *parent, int x, int y, int w, int h)
	: Widget(parent, x, y, w, h)
{
	double res_h, res_v, aspect_diff;

	xineDisplay = XOpenDisplay(mDisplayStr);

	if (xineDisplay == NULL) {
		fprintf(stderr, "video thread cannot connect to Xserver\n");
		return;
	}

	screen = DefaultScreen(xineDisplay);

	res_h = (DisplayWidth(xineDisplay, screen)*1000 /
			DisplayWidthMM(xineDisplay, screen));
	res_v = (DisplayHeight(xineDisplay, screen)*1000 /
			DisplayHeightMM(xineDisplay, screen));

	pixel_aspect = res_v / res_h;
	aspect_diff = pixel_aspect - 1.0;
	if ((aspect_diff < 0.01) && (aspect_diff > -0.01)) {
		pixel_aspect = 1.0;
	}

	stream = NULL;
	video_port = NULL;
	audio_port = NULL;

	xine = xine_new();

	xine_init(xine);
	vis.display = xineDisplay;
	vis.screen = screen;
	vis.d = mWindow;
	vis.dest_size_cb = DestSizeCB;
	vis.frame_output_cb = FrameOutputCB;
	vis.user_data = (void*)this;

	video_port = xine_open_video_driver(xine, "xshm",
			XINE_VISUAL_TYPE_X11,
			&vis);

	audio_port = xine_open_audio_driver (xine, NULL, NULL);

	stream = xine_stream_new(xine, audio_port, video_port);

	xine_event_create_listener_thread(
			xine_event_new_queue(stream), XineEventCB, this);

	mImData = Imlib_init(mDisplay);
}

VideoBox::~VideoBox()
{
	if (xine) {
		xine_exit(xine);
		xine == NULL;
	}
}

int VideoBox::OpenFile(char *filename)
{
	if (stream == NULL) return 0;
	int rv = xine_open(stream, filename);
	return 0;
}

int VideoBox::CloseFile()
{
	if (stream == NULL) return 0;
	xine_close(stream);
	return 0;
}

int VideoBox::Play()
{
	if (stream == NULL) return 0;
	int rv = xine_play(stream, 0, 0);
	xine_set_param(stream, XINE_PARAM_SPEED, XINE_SPEED_NORMAL);
	return 0;
}

int VideoBox::Pause()
{
	if (stream == NULL) return 0;
	xine_set_param(stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE);
	return 0;
}

int VideoBox::Stop()
{
	if (stream == NULL) return 0;
	xine_stop(stream);
	return 0;
}

int VideoBox::Faster()
{
	if (stream == NULL) return 0;
	int current_speed = xine_get_param(stream, XINE_PARAM_SPEED);
	if (current_speed == 0) 
		xine_set_param(stream, XINE_PARAM_SPEED, XINE_SPEED_SLOW_4);
	else
		xine_set_param(stream, XINE_PARAM_SPEED, current_speed * 2);
	return 0;
}

int VideoBox::Slower()
{
	if (stream == NULL) return 0;
	int current_speed = xine_get_param(stream, XINE_PARAM_SPEED);
	xine_set_param(stream, XINE_PARAM_SPEED, current_speed / 2);
	return 0;
}

int VideoBox::TakeShot(char *fname)
{
	if (stream == NULL) return 0;
	int current_speed = xine_get_param(stream, XINE_PARAM_SPEED);
	xine_set_param(stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE);
#if 0
	int status = XpmWriteFileFromPixmap(mDisplay, fname, mWindow, 0, NULL);
#endif
	ImlibImage *im = Imlib_create_image_from_drawable(mImData,
			mWindow, 0, 0, 0, mW, mH);
	ImlibSaveInfo isf;

	isf.quality = 1;
	isf.scaling = 0;
	isf.xjustification = 0;
	isf.yjustification = 0;
	isf.color = '\0';

	Imlib_save_image(mImData, im, fname, &isf);
	Imlib_destroy_image(mImData, im);
	xine_set_param(stream, XINE_PARAM_SPEED, current_speed);
	return 0;
}

void VideoBox::DestSizeCB(void *data, int video_width, int video_height,
		double video_pixel_aspect,
		int *dest_width, int *dest_height,
		double *dest_pixel_aspect)
{
	int w, h;
	VideoBox *d = (VideoBox*)data;
	if (d == NULL) return;

	d->GetSize(NULL, NULL, &w, &h);
	
	*dest_width = w;
	*dest_height = h;
	*dest_pixel_aspect = d->GetAspect();
}

void VideoBox::FrameOutputCB(void *data, int video_width, int video_height,
		double video_pixel_aspect,
		int *dest_x, int *dest_y,
		int *dest_width, int *dest_height,
		double *dest_pixel_aspect,
		int *win_x, int *win_y)
{
	int x, y, w, h;
	VideoBox *d = (VideoBox*)data;
	if (d == NULL) return;

	d->GetSize(&x, &y, &w, &h);

	*dest_x = 0;
	*dest_y = 0;
	*win_x = x;
	*win_y = y;
	*dest_width = w;
	*dest_height = h;

	*dest_pixel_aspect = d->GetAspect();
}

void VideoBox::XineEventCB(void *data, const xine_event_t *ev)
{
}

