#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "dtk_widget.h"
#include "dtk_tooltip.h"
#include "dtk_skin.h"

Widget::Widget(BaseWidget *parent, int x, int y, int w, int h)
	: BaseWidget(parent, x, y, w, h)
{
	ENTER;

	mToolTip = NULL;
	mToolTipText = NULL;
	SetEventMask(ButtonPressMask | ButtonReleaseMask |
			EnterWindowMask | LeaveWindowMask |
			ExposureMask);
}

Widget::~Widget()
{
	if (mToolTip) {
		delete mToolTip;
		mToolTip = NULL;
	}
	if (mToolTipText) free((void*)mToolTipText);
}

int Widget::EventHandler(TEvent *ev)
{
	int rv = 0;
	ENTER;

	rv = BaseWidget::EventHandler(ev);

	if (ev == NULL) return 0;
	if (ev->type == NOEVENT) return 0;

	if (ev->type == XEVENT) {
		switch (ev->xev->type) {
			case EnterNotify:
				if (ev->xev->xcrossing.window != mWindow)
					return 0;
				ShowToolTip(ev->xev->xcrossing.x,
						ev->xev->xcrossing.y);
				break;
			case LeaveNotify:
				if (ev->xev->xcrossing.window != mWindow)
					return 0;
				HideToolTip();
				break;
		}
	}
	return rv;
}

int Widget::SetToolTipText(char *text)
{
	if (mToolTipText) free((void*)mToolTipText);
	mToolTipText = (text ? strdup(text) : NULL);
	if (mToolTip) mToolTip->SetText(mToolTipText);
	return 0;
}

int Widget::ShowToolTip(int x, int y)
{
	int x1, y1;
	Window junk;
	BaseWidget *tl;

	if (mToolTipText == NULL) return 0;

	tl = GetTopLevelWidget();

	if (tl == NULL) return 0;

	XTranslateCoordinates(mDisplay, mWindow, tl->GetXWindow(),
			x, y, &x1, &y1, &junk);

	if (mToolTip == NULL) {
		mToolTip = new ToolTip(tl);
		mToolTip->SetPixmap(Skin::GetSkinPixmap(TOOLTIP));
		mToolTip->SetText(mToolTipText);
	} else mToolTip->Move(x1 + 5, y1 + 3);
	mToolTip->Show();
	return 0;
}

int Widget::HideToolTip()
{
	if (mToolTip == NULL) return 0;
	mToolTip->Hide();
	return 0;
}

Widget *Widget::GetTopLevelWidget()
{
	return (Widget*)BaseWidget::GetTopLevelWidget();
}

