#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <Imlib.h>

#include "skin_interface.h"
#include "config.h"

#define NAME "Ice 2 (png)"
#define AUTH "Peter Hartshorn"
#define DESC "Default theme for Devsec"

static Pixmap pixmapArray[PIXMAPS];
static ImlibData *id = NULL;

int dl_skin_init(Display *display)
{
	unsigned long i;
	id = Imlib_init(display);
	memset(pixmapArray, 0, sizeof(Pixmap) * PIXMAPS);
	fprintf(stderr, "preloading images\n");
	for (i = 0; i < PIXMAPS; i++) {
		check_pixmap(display, i);
	}
	fprintf(stderr, "\ndone\n");
	return OK;
}

int dl_skin_done()
{
	fprintf(stderr, "ice skin: I should free the pixmaps\n");
	return OK;
}

int dl_skin_get_info(int infotype, char *buffer, int length)
{
	char *info;
	
	int i;
	if (buffer == NULL) return MYBAD;

	switch (infotype) {
		case INFO_NAME:
			info = NAME;
			break;
		case INFO_AUTH:
			info = AUTH;
			break;
		case INFO_DESC:
			info = DESC;
			break;
		default:
			return MYBAD;
	}
	for (i = 0; ((i < length-1) && (i < strlen(info))); i++) {
		buffer[i] = info[i];
	}
	buffer[i] = '\0';
	return OK;
}

#define IMG(img) Imlib_load_image(id, DTK_PIXMAPDIR img/**/)
int check_pixmap(Display *display, unsigned long button)
{
	if (button > PIXMAPS) return -1;

	if (pixmapArray[button] == (Pixmap)0) {
		ImlibImage *im = NULL;
		switch (button) {
			case OK_NORMAL:
				im = IMG("ok_normal.xpm.png");
				break;
			case OK_HOVER:
				im = IMG("ok_hover.xpm.png");
				break;
			case OK_PRESSED:
				im = IMG("ok_pressed.xpm.png");
				break;
			case CANCEL_NORMAL:
				im = IMG("cancel_normal.xpm.png");
				break;
			case CANCEL_HOVER:
				im = IMG("cancel_hover.xpm.png");
				break;
			case CANCEL_PRESSED:
				im = IMG("cancel_pressed.xpm.png");
				break;
			case CHECK_OFF:
				im = IMG("check_off.xpm.png");
				break;
			case CHECK_ON:
				im = IMG("check_on.xpm.png");
				break;
			case CHECK_HOVER:
				im = IMG("check_hover.xpm.png");
				break;
			case SPINDOWN_NORMAL:
				im = IMG("spindown_normal.xpm.png");
				break;
			case SPINDOWN_HOVER:
				im = IMG("spindown_hover.xpm.png");
				break;
			case SPINDOWN_PRESSED:
				im = IMG("spindown_pressed.xpm.png");
				break;
			case SPINUP_NORMAL:
				im = IMG("spinup_normal.xpm.png");
				break;
			case SPINUP_HOVER:
				im = IMG("spinup_hover.xpm.png");
				break;
			case SPINUP_PRESSED:
				im = IMG("spinup_pressed.xpm.png");
				break;
			case BUTTON_NORMAL:
				im = IMG("button_normal.xpm.png");
				break;
			case BUTTON_HOVER:
				im = IMG("button_hover.xpm.png");
				break;
			case BUTTON_PRESSED:
				im = IMG("button_pressed.xpm.png");
				break;
			case TEXT_BACK:
				im = IMG("text_back.xpm.png");
				break;
			case SLIDER_NORMAL:
				im = IMG("slider_normal.xpm.png");
				break;
			case SLIDER_HOVER:
				im = IMG("slider_hover.xpm.png");
				break;
			case SLIDER_PRESSED:
				im = IMG("slider_pressed.xpm.png");
				break;
			case HSLIDER_TRACK:
				im = IMG("hslider_track.xpm.png");
				break;
			case VSLIDER_TRACK:
				im = IMG("vslider_track.xpm.png");
				break;
			case COMBOBTN_NORMAL:
				im = IMG("combobtn_normal.xpm.png");
				break;
			case COMBOBTN_HOVER:
				im = IMG("combobtn_hover.xpm.png");
				break;
			case COMBOBTN_PRESSED:
				im = IMG("combobtn_pressed.xpm.png");
				break;


			case CONFIG_NORMAL:
				im = IMG("config_normal.xpm.png");
				break;
			case CONFIG_HOVER:
				im = IMG("config_hover.xpm.png");
				break;
			case CONFIG_PRESSED:
				im = IMG("config_pressed.xpm.png");
				break;
			case CYCLE_NORMAL:
				im = IMG("cycle_normal.xpm.png");
				break;
			case CYCLE_HOVER:
				im = IMG("cycle_hover.xpm.png");
				break;
			case CYCLE_PRESSED:
				im = IMG("cycle_pressed.xpm.png");
				break;
			case FIXATE_NORMAL:
				im = IMG("stop_normal.xpm.png");
				break;
			case FIXATE_HOVER:
				im = IMG("stop_hover.xpm.png");
				break;
			case FIXATE_PRESSED:
				im = IMG("stop_pressed.xpm.png");
				break;
			case FULLSCREEN_NORMAL:
				im =IMG("fullscreen_normal.xpm.png");
				break;
			case FULLSCREEN_HOVER:
				im = IMG("fullscreen_hover.xpm.png");
				break;
			case FULLSCREEN_PRESSED:
				im = IMG("fullscreen_pressed.xpm.png");
				break;
			case MENU_NORMAL:
				im = IMG("menu_normal.xpm.png");
				break;
			case MENU_HOVER:
				im = IMG("menu_hover.xpm.png");
				break;
			case MENU_PRESSED:
				im = IMG("menu_pressed.xpm.png");
				break;
			case STATUSBAR:
				im = IMG("statusbar.xpm.png");
				break;
			case TOOLBAR:
				im = IMG("toolbar.xpm.png");
				break;
			case TOOLTIP:
				im = IMG("tooltip.xpm.png");
				break;
			case VIEW1_NORMAL:
				im = IMG("view1_normal.xpm.png");
				break;
			case VIEW1_HOVER:
				im = IMG("view1_hover.xpm.png");
				break;
			case VIEW1_PRESSED:
				im = IMG("view1_pressed.xpm.png");
				break;
			case VIEW2_NORMAL:
				im = IMG("view2_normal.xpm.png");
				break;
			case VIEW2_HOVER:
				im = IMG("view2_hover.xpm.png");
				break;
			case VIEW2_PRESSED:
				im = IMG("view2_pressed.xpm.png");
				break;
			case VIEW3_NORMAL:
				im = IMG("view3_normal.xpm.png");
				break;
			case VIEW3_HOVER:
				im = IMG("view3_hover.xpm.png");
				break;
			case VIEW3_PRESSED:
				im = IMG("view3_pressed.xpm.png");
				break;
			case VIEW4_NORMAL:
				im = IMG("view4_normal.xpm.png");
				break;
			case VIEW4_HOVER:
				im = IMG("view4_hover.xpm.png");
				break;
			case VIEW4_PRESSED:
				im = IMG("view4_pressed.xpm.png");
				break;
			case VIEW5_NORMAL:
				im = IMG("view5_normal.xpm.png");
				break;
			case VIEW5_HOVER:
				im = IMG("view5_hover.xpm.png");
				break;
			case VIEW5_PRESSED:
				im = IMG("view5_pressed.xpm.png");
				break;
			case VIEW6_NORMAL:
				im = IMG("view6_normal.xpm.png");
				break;
			case VIEW6_HOVER:
				im = IMG("view6_hover.xpm.png");
				break;
			case VIEW6_PRESSED:
				im = IMG("view6_pressed.xpm.png");
				break;
			case VIEW7_NORMAL:
				im = IMG("view7_normal.xpm.png");
				break;
			case VIEW7_HOVER:
				im = IMG("view7_hover.xpm.png");
				break;
			case VIEW7_PRESSED:
				im = IMG("view7_pressed.xpm.png");
				break;
			case VIEW8_NORMAL:
				im = IMG("view8_normal.xpm.png");
				break;
			case VIEW8_HOVER:
				im = IMG("view8_hover.xpm.png");
				break;
			case VIEW8_PRESSED:
				im = IMG("view8_pressed.xpm.png");
				break;
			case CONFIG_TOOLBAR:
				im = IMG("config_toolbar.xpm.png");
				break;
			case CONFIG_HDR_GLOBAL:
				im = IMG("config_hdr_global.xpm.png");
				break;
			case CONFIG_HDR_CAMERAS:
				im = IMG("config_hdr_cameras.xpm.png");
				break;
			case CONFIG_HDR_SYSTEM:
				im = IMG("config_hdr_system.xpm.png");
				break;
			case CONFIG_HDR_TV:
				im = IMG("config_hdr_tv.xpm.png");
				break;
			case CONFIG_HDR_USERS:
				im = IMG("config_hdr_users.xpm.png");
				break;
			case CONFIG_HDR_MOTION:
				im = IMG("config_hdr_motion.xpm.png");
				break;
			case CONFIG_ABOUT:
				im = IMG("config_about.xpm.png");
				break;
			case BTNGLOBAL_NORMAL:
				im = IMG("globalbtn_normal.xpm.png");
				break;
			case BTNGLOBAL_HOVER:
				im = IMG("globalbtn_hover.xpm.png");
				break;
			case BTNGLOBAL_PRESSED:
				im = IMG("globalbtn_pressed.xpm.png");
				break;
			case BTNCAMERAS_NORMAL:
				im = IMG("camerabtn_normal.xpm.png");
				break;
			case BTNCAMERAS_HOVER:
				im = IMG("camerabtn_hover.xpm.png");
				break;
			case BTNCAMERAS_PRESSED:
				im = IMG("camerabtn_pressed.xpm.png");
				break;
			case BTNTV_NORMAL:
				im = IMG("tvbtn_normal.xpm.png");
				break;
			case BTNTV_HOVER:
				im = IMG("tvbtn_hover.xpm.png");
				break;
			case BTNTV_PRESSED:
				im = IMG("tvbtn_pressed.xpm.png");
				break;
			case BTNUSERS_NORMAL:
				im = IMG("userbtn_normal.xpm.png");
				break;
			case BTNUSERS_HOVER:
				im = IMG("userbtn_hover.xpm.png");
				break;
			case BTNUSERS_PRESSED:
				im = IMG("userbtn_pressed.xpm.png");
				break;
			case BTNMOTION_NORMAL:
				im = IMG("findbtn_normal.xpm.png");
				break;
			case BTNMOTION_HOVER:
				im = IMG("findbtn_hover.xpm.png");
				break;
			case BTNMOTION_PRESSED:
				im = IMG("findbtn_pressed.xpm.png");
				break;
			case LAYOUT1_NORMAL:
				im = IMG("layout1_normal.xpm.png");
				break;
			case LAYOUT1_HOVER:
				im = IMG("layout1_hover.xpm.png");
				break;
			case LAYOUT1_PRESSED:
				im = IMG("layout1_pressed.xpm.png");
				break;
			case LAYOUT2_NORMAL:
				im = IMG("layout2_normal.xpm.png");
				break;
			case LAYOUT2_HOVER:
				im = IMG("layout2_hover.xpm.png");
				break;
			case LAYOUT2_PRESSED:
				im = IMG("layout2_pressed.xpm.png");
				break;
			case LAYOUT3_NORMAL:
				im = IMG("layout3_normal.xpm.png");
				break;
			case LAYOUT3_HOVER:
				im = IMG("layout3_hover.xpm.png");
				break;
			case LAYOUT3_PRESSED:
				im = IMG("layout3_pressed.xpm.png");
				break;
			case LAYOUT4_NORMAL:
				im = IMG("layout4_normal.xpm.png");
				break;
			case LAYOUT4_HOVER:
				im = IMG("layout4_hover.xpm.png");
				break;
			case LAYOUT4_PRESSED:
				im = IMG("layout4_pressed.xpm.png");
				break;
			case LAYOUT5_NORMAL:
				im = IMG("layout5_normal.xpm.png");
				break;
			case LAYOUT5_HOVER:
				im = IMG("layout5_hover.xpm.png");
				break;
			case LAYOUT5_PRESSED:
				im = IMG("layout5_pressed.xpm.png");
				break;
			case LAYOUT6_NORMAL:
				im = IMG("layout6_normal.xpm.png");
				break;
			case LAYOUT6_HOVER:
				im = IMG("layout6_hover.xpm.png");
				break;
			case LAYOUT6_PRESSED:
				im = IMG("layout6_pressed.xpm.png");
				break;
			case LAYOUT7_NORMAL:
				im = IMG("layout7_normal.xpm.png");
				break;
			case LAYOUT7_HOVER:
				im = IMG("layout7_hover.xpm.png");
				break;
			case LAYOUT7_PRESSED:
				im = IMG("layout7_pressed.xpm.png");
				break;
			case LAYOUT8_NORMAL:
				im = IMG("layout8_normal.xpm.png");
				break;
			case LAYOUT8_HOVER:
				im = IMG("layout8_hover.xpm.png");
				break;
			case LAYOUT8_PRESSED:
				im = IMG("layout8_pressed.xpm.png");
				break;
			case RIGHT_TOOLBAR:
				im = IMG("right_toolbar.xpm.png");
				break;
			case POWER_NORMAL:
				im = IMG("power_normal.xpm.png");
				break;
			case POWER_HOVER:
				im = IMG("power_hover.xpm.png");
				break;
			case POWER_PRESSED:
				im = IMG("power_pressed.xpm.png");
				break;
			case SYSINFO_NORMAL:
				im = IMG("sysinfo_normal.xpm.png");
				break;
			case SYSINFO_HOVER:
				im = IMG("sysinfo_hover.xpm.png");
				break;
			case SYSINFO_PRESSED:
				im = IMG("sysinfo_pressed.xpm.png");
				break;
			case MPLAYER_NORMAL:
				im = IMG("mplayer_normal.xpm.png");
				break;
			case MPLAYER_HOVER:
				im = IMG("mplayer_hover.xpm.png");
				break;
			case MPLAYER_PRESSED:
				im = IMG("mplayer_pressed.xpm.png");
				break;
			case KFIND_NORMAL:
				im = IMG("kfind_normal.xpm.png");
				break;
			case KFIND_HOVER:
				im = IMG("kfind_hover.xpm.png");
				break;
			case KFIND_PRESSED:
				im = IMG("kfind_pressed.xpm.png");
				break;
			case SCREENCAPTURE_NORMAL:
				im = IMG("screencapture_normal.xpm.png");
				break;
			case SCREENCAPTURE_HOVER:
				im = IMG("screencapture_hover.xpm.png");
				break;
			case SCREENCAPTURE_PRESSED:
				im = IMG("screencapture_pressed.xpm.png");
				break;
			case CDBAKE_NORMAL:
				im = IMG("cdbake_normal.xpm.png");
				break;
			case CDBAKE_HOVER:
				im = IMG("cdbake_hover.xpm.png");
				break;
			case CDBAKE_PRESSED:
				im = IMG("cdbake_pressed.xpm.png");
				break;
			case DIRSTAT_NORMAL:
				im = IMG("dirstat_normal.xpm.png");
				break;
			case DIRSTAT_HOVER:
				im = IMG("dirstat_hover.xpm.png");
				break;
			case DIRSTAT_PRESSED:
				im = IMG("dirstat_pressed.xpm.png");
				break;
			case BROWSE_NORMAL:
				im = IMG("browse_normal.xpm.png");
				break;
			case BROWSE_HOVER:
				im = IMG("browse_hover.xpm.png");
				break;
			case BROWSE_PRESSED:
				im = IMG("browse_pressed.xpm.png");
				break;
			case TIME_NORMAL:
				im = IMG("time_normal.xpm.png");
				break;
			case TIME_HOVER:
				im = IMG("time_hover.xpm.png");
				break;
			case TIME_PRESSED:
				im = IMG("time_pressed.xpm.png");
				break;
			case NETWORK_NORMAL:
				im = IMG("network_normal.xpm.png");
				break;
			case NETWORK_HOVER:
				im = IMG("network_hover.xpm.png");
				break;
			case NETWORK_PRESSED:
				im = IMG("network_pressed.xpm.png");
				break;
			case TRASH_NORMAL:
				im = IMG("trash_normal.xpm.png");
				break;
			case TRASH_HOVER:
				im = IMG("trash_hover.xpm.png");
				break;
			case TRASH_PRESSED:
				im = IMG("trash_pressed.xpm.png");
				break;
			case METER_NORMAL:
				im = IMG("meter_normal.xpm.png");
				break;
			case METER_HIGH:
				im = IMG("meter_high.xpm.png");
				break;
			case METER_CRITICAL:
				im = IMG("meter_critical.xpm.png");
				break;
		}
		if (im == NULL) {
			fprintf(stderr, "X");
			return 0;
		} else fprintf(stderr, ".");

		Imlib_render(id,im,im->rgb_width, im->rgb_height);
		pixmapArray[button] = Imlib_copy_image(id, im);
	}
	return 0;
}

int dl_skin_draw_image(Display *display, Drawable d, unsigned long button)
{
	GC gc;
	int x, y, w, h;

	if (button > PIXMAPS) {
		fprintf(stderr, "bounds error\n");
		return MYBAD;
	}
	x = y = 0;
	w = h = 24;
	switch (button) {
		case TOOLBAR:
			w = 32; h = 576;
			break;
		case STATUSBAR:
			x = 0; y = 576;
			w = 800; h = 24;
			break;
	}
			
	if (check_pixmap(display, button) == -1) return 0;

	gc = XCreateGC(display, d, 0, NULL);
	XCopyArea(display, pixmapArray[button], d, gc, 0, 0, w, h, x, y);
	XFreeGC(display, gc);
	return 0;
}

Pixmap dl_skin_get_pixmap(Display *display, unsigned long button)
{
	if (check_pixmap(display, button) == -1) return 0;
	return pixmapArray[button];
}
